@@ -9,7 +9,7 @@
 %%  See the file "license.terms" for information on usage and redistribution
 %%  of this file, and for a DISCLAIMER OF ALL WARRANTIES.
 %%
-%%     $Id: e3d_tds.erl,v 1.34 2004/02/05 06:32:00 bjorng Exp $
+%%     $Id: e3d_tds.erl,v 1.44 2004/10/12 17:46:14 bjorng Exp $
 %%
 
 -module(e3d_tds).
@@ -21,9 +21,6 @@
 -import(lists, [map/2,foldl/3,mapfoldl/3,reverse/1,reverse/2,
 		sort/1,keysort/2,usort/1,keydelete/3,keyreplace/4]).
 
-%% R9C required is required if debugging is turned on,
-%% sinced the debug printouts use the new hex formatting characters.
-
 %%-define(DEBUG, 1).
 
 %% Inline dbg/2 so that the call will disappear completely if
@@ -46,246 +43,258 @@ dbg(_, _) -> ok.
 import(Name) ->
     case file:read_file(Name) of
 	{ok,Bin} ->
-	    import_1(Bin);
+	    ?MODULE = ets:new(?MODULE, [named_table,ordered_set]),
+	    Dir = filename:dirname(Name),
+	    Res = import_1(Bin, Dir),
+	    ets:delete(?MODULE),
+	    Res;
 	{error,Reason} ->
 	    {error,file:format_error(Reason)}
     end.
 
-import_1(Bin) ->
-    case catch import_2(Bin) of
-	{'EXIT',Reason} -> exit(Reason);
-	{error,_}=Error -> Error;
-	#e3d_file{}=E3dFile -> {ok,E3dFile}
+import_1(Bin, Dir) ->
+    try import_2(Bin) of
+	#e3d_file{}=E3dFile ->
+	    {ok,E3dFile#e3d_file{dir=Dir}}
+    catch
+	throw:Error -> Error
     end.
 
-import_2(<<16#4D4D:16/little,Size:32/little,Rest/binary>>) ->
-    dbg("Main chunk ~p bytes~n", [Size]),
-    main(Rest, #e3d_file{});
+import_2(<<16#4D4D:16/little,_Size:32/little,T/binary>>) ->
+    File = fold_chunks(fun main/3, #e3d_file{}, T),
+    #e3d_file{objs=Objs0,mat=Mat0} = File,
+    Mat = reformat_material(Mat0),
+    Objs = case catch fix_transform(Objs0) of
+	       {'EXIT',Reason} ->
+		   io:format("~P\n", [Reason,20]),
+		   reverse(Objs0);
+	       Other -> Other
+	   end,
+    File#e3d_file{objs=Objs,mat=Mat};
 import_2(_) -> error("Not a .3ds file").
 
-main(<<16#0002:16/little,10:32/little,Ver:32/little,Rest/binary>>, Acc) ->
-    dbg("3DS Version ~p~n", [Ver]),
-    main(Rest, Acc);
-main(<<16#3D3D:16/little,Sz0:32/little,Rest0/binary>>, Acc0) ->
-    dbg("Editor: ~p bytes\n", [Sz0]),
-    Sz = Sz0 - 6,
-    <<Ed:Sz/binary,Rest/binary>> = Rest0,
-    Acc = editor(Ed, Acc0),
-    main(Rest, Acc);
-main(<<16#B000:16/little,Sz0:32/little,T0/binary>>, Acc) ->
-    dbg("Key Framer (ignoring) ~p bytes\n", [Sz0]),
-    Sz = Sz0 - 6,
-    <<_:Sz/binary,T/binary>> = T0,
-    main(T, Acc);
-main(<<Tag:16/little,Sz0:32/little,T0/binary>>, Acc) ->
-    dbg("Ignoring unknown chunk ~.16#; ~p bytes\n", [Tag,Sz0]),
-    Sz = Sz0 - 6,
-    <<_:Sz/binary,T/binary>> = T0,
-    main(T, Acc);
-main(<<>>, #e3d_file{mat=Mat0}=File) ->
-    Mat = reformat_material(Mat0),
-    File#e3d_file{mat=Mat}.
+main(16#0002, <<Ver:32/little>>, Acc) ->
+    dbg("3DS Version ~p\n", [Ver]),
+    Acc;
+main(16#3D3D, Editor, Acc) ->
+    dbg("Editor: ~p bytes\n", [size(Editor)]),
+    editor(Editor, Acc);
+main(16#B000, Keyframer, Acc) ->
+    dbg("\nKeyframer:\n", []),
+    keyframer(Keyframer),
+    Acc;
+main(Tag, Chunk, Acc) ->
+    dbg("~.16#: ~P\n", [Tag,Chunk,15]),
+    Acc.
 
-editor(<<16#4000:16/little,Sz0:32/little,T0/binary>>,
-       #e3d_file{objs=Objs0}=Acc) ->
-    Sz = Sz0 - 6,
-    <<Obj0:Sz/binary,T/binary>> = T0,
+%% editor(Bin, E3DFile) -> E3DFile
+%%  Collect objects and materials from the sub-chunks of 'editor'.
+
+editor(Bin, Acc) ->
+    fold_chunks(fun editor/3, Acc, Bin).
+
+editor(16#0100, <<Scale:32/?FLOAT>>, Acc) ->
+    dbg("Object Scale ~p ~n", [Scale]),
+    Acc;
+editor(16#3d3e, <<Ver:32/little>>, Acc) ->
+    dbg("Mesh Version ~p ~n", [Ver]),
+    Acc;
+editor(16#4000, Obj0, #e3d_file{objs=Objs0}=Acc) ->
     {Name,Obj1} = get_cstring(Obj0),
+    ets:insert(?MODULE, {current_name,Name}),
     dbg("\nObject block: ~s\n", [Name]),
     case block(Obj1) of
-	no_mesh -> editor(T, Acc);
+	no_mesh -> Acc;
 	Obj ->
 	    Objs = [#e3d_object{name=Name,obj=Obj}|Objs0],
-	    editor(T, Acc#e3d_file{objs=Objs})
+	    Acc#e3d_file{objs=Objs}
     end;
-editor(<<16#3d3e:16/little,_Sz:32/little,Ver:32/little,T/binary>>, Acc) ->
-    dbg("Mesh Version ~p ~n", [Ver]),
-    editor(T, Acc);
-editor(<<16#0100:16/little,_Sz:32/little,Scale:32/?FLOAT,T/binary>>,
-       Acc) ->
-    dbg("Object Scale ~p ~n", [Scale]),
-    editor(T, Acc);
-editor(<<16#AFFF:16/little,Sz0:32/little,T0/binary>>, #e3d_file{mat=M}=Acc) ->
-    Sz = Sz0 - 6,
-    <<Mat0:Sz/binary,T/binary>> = T0,
+editor(16#AFFF, Mat0, #e3d_file{mat=M}=Acc) ->
     Mat = material(Mat0, M),
-    editor(T, Acc#e3d_file{mat=Mat});
-editor(<<Tag:16/little,Sz0:32/little,T0/binary>>, Acc) ->
-    Sz = Sz0 - 6,
-    <<Chunk:Sz/binary,T/binary>> = T0,
-    dbg("Unknown editor chunk: ~.16#: ~P\n", [Tag,Chunk,20]),
-    editor(T, Acc);
-editor(<<>>, Acc) -> Acc.
+    Acc#e3d_file{mat=Mat};
+editor(Tag, Chunk, Acc) ->
+    dbg("~.16#: ~P\n", [Tag,Chunk,15]),
+    Acc.
 
-block(Bin) ->
-    block(Bin, no_mesh).
-block(<<16#3d3e:16/little,_Sz:32/little,Ver:32/little,T/binary>>, Acc) ->
+%% keyframer(Bin)
+%%  Go through the keyframer data and collect the only information
+%%  we are interested in: how the objects are related in the
+%%  hieararchy. We'll need that to calculate the correct transformation
+%%  matrix for each object.
+
+keyframer(Bin) ->
+    put(e3d_tds_node_id, -1),
+    fold_chunks(fun(Tag, Contents, _) -> keyframer(Tag, Contents) end, [], Bin),
+    erase(e3d_tds_node_id),
+    ok.
+
+keyframer(16#B002, Contents) ->
+    %% Node chunk - recurse into it.
+    fold_chunks(fun(Tag, Cont, _) -> keyframer(Tag, Cont) end, [], Contents),
+    put(e3d_tds_node_id, get(e3d_tds_node_id)+1);
+keyframer(16#B030, <<NodeId:16/little-signed>>) ->
+    %% Node id for this node.
+    put(e3d_tds_node_id, NodeId);
+keyframer(16#B010, Bin) ->
+    %% Name+parent node.
+    NameSz = size(Bin) - 7,
+    <<Name0:NameSz/binary,_:5/unit:8,Parent:16/little-signed>> = Bin,
+    Name = binary_to_list(Name0),
+    NodeId = get(e3d_tds_node_id),
+    dbg("node ~p: ~p, parent ~p\n", [NodeId,Name,Parent]), 
+    ets:insert(?MODULE, {NodeId,Name}),
+    ets:insert(?MODULE, {Name,Parent});
+keyframer(16#B00A, <<Rev:16/little,T0/binary>>) ->
+    %% Just print information (if debugging).
+    {Str,T} = get_cstring(T0),
+    dbg("Keyframe header: ~p ~p ~p\n", [Rev,Str,T]);
+keyframer(16#B013, <<X:32/?FLOAT,Y:32/?FLOAT,Z:32/?FLOAT>>) ->
+    %% Just print information (if debugging).
+    dbg(" pivot: ~p ~p ~p\n", [X,Y,Z]);
+keyframer(16#B021, <<A:32/?FLOAT,X:32/?FLOAT,Y:32/?FLOAT,Z:32/?FLOAT,_/binary>>) ->
+    %% Just print information (if debugging).
+    dbg(" rot_track_tag: ~p ~p ~p ~p\n", [A,X,Y,Z]);
+keyframer(Tag, Contents) ->
+    %% Ignore all other keyframer chunks.
+    dbg("~.16#: ~P\n", [Tag,Contents,15]).
+
+block(Block) ->
+    fold_chunks(fun block/3, no_mesh, Block).
+
+block(16#3d3e, <<Ver:32/little>>, Acc) ->
     dbg("Mesh Version ~p ~n", [Ver]),
-    block(T, Acc);
-block(<<16#4100:16/little,Sz0:32/little,T0/binary>>, no_mesh) ->
-    dbg("Triangular mesh ~p~n", [Sz0]),
-    Sz = Sz0 - 6,
-    <<TriMesh0:Sz/binary,T/binary>> = T0,
-    TriMesh1 = trimesh(TriMesh0, #e3d_mesh{type=triangle}),
-    TriMesh2 = add_uv_to_faces(TriMesh1),
-    TriMesh = clean_mesh(TriMesh2),
-    block(T, TriMesh);
-block(<<16#4700:16/little,Sz0:32/little,T0/binary>>, Mesh) ->
-    dbg("Camera (ignoring) ~p bytes\n", [Sz0]),
-    Sz = Sz0 - 6,
-    <<_:Sz/binary,T/binary>> = T0,
-    block(T, Mesh);
-block(<<Tag:16/little,Sz0:32/little,T0/binary>>, Mesh) ->
-    dbg("Ignoring unknown chunk ~.16#; ~p bytes\n", [Tag,Sz0]),
-    Sz = Sz0 - 6,
-    <<_:Sz/binary,T/binary>> = T0,
-    block(T, Mesh);
-block(<<>>, Acc) -> Acc.
+    Acc;
+block(16#4100, TriMesh0, no_mesh) ->
+    dbg("Triangular mesh: ~p\n", [size(TriMesh0)]),
+    TriMesh1 = fold_chunks(fun trimesh/3, #e3d_mesh{type=triangle}, TriMesh0),
+    TriMesh = add_uv_to_faces(TriMesh1),
+    clean_mesh(TriMesh);
+block(16#4700, Camera, Mesh) ->
+    dbg("Camera: ~p\n", [size(Camera)]),
+    Mesh;
+block(Tag, Chunk, Mesh) ->
+    dbg("~.16#: ~P\n", [Tag,Chunk,15]),
+    Mesh.
 
-trimesh(<<16#4110:16/little,Sz0:32/little,NumVs:16/little,T0/binary>>, Acc) ->
-    dbg("Vertices ~p bytes NumVs=~p\n", [Sz0,NumVs]),
-    Sz = Sz0 - 8,
-    <<Vs0:Sz/binary,T/binary>> = T0,
+trimesh(16#4110, <<NumVs:16/little,Vs0/binary>>, Acc) ->
+    dbg("~p vertices\n", [NumVs]),
     Vs = get_bin_vectors(Vs0),
-    trimesh(T, Acc#e3d_mesh{vs=Vs});
-trimesh(<<16#4120:16/little,Sz0:32/little,NFaces:16/little,T0/binary>>, Acc) ->
-    dbg("~p faces, ~p bytes\n", [NFaces,Sz0]),
-    Sz = Sz0 - 8,
+    Acc#e3d_mesh{vs=Vs};
+trimesh(16#4120, <<NFaces:16/little,Contents/binary>>, Acc) ->
+    dbg("~p faces\n", [NFaces]),
     Fsz = NFaces * 2 * 4,
-    Dsz = Sz - Fsz,
-    <<Faces0:Fsz/binary,Desc:Dsz/binary,T/binary>> = T0,
+    <<Faces0:Fsz/binary,Desc/binary>> = Contents,
     Faces1 = get_faces(Faces0),
-    {Faces,Smooth} = face_desc(Desc, {Faces1,[]}),
-    trimesh(T, Acc#e3d_mesh{fs=Faces,he=Smooth});
-trimesh(<<16#4140:16/little,Sz0:32/little,T0/binary>>, Acc) ->
-    dbg("Texture coordinates ~w\n", [Sz0]),
-    Sz = Sz0 - 8,
-    <<_:16,Tx0:Sz/binary,T/binary>> = T0,
+    {Faces,Smooth} = face_desc(Desc, Faces1),
+    Acc#e3d_mesh{fs=Faces,he=Smooth};
+trimesh(16#4140, <<NumTx:16/little,Tx0/binary>>, Acc) ->
+    dbg("~p texture coordinates\n", [NumTx]),
     Tx = get_uv(Tx0),
-    trimesh(T, Acc#e3d_mesh{tx=Tx});
-trimesh(<<16#4160:16/little,_Sz:32/little,T0/binary>>, Acc) ->
-    <<_V1X:32/?FLOAT,_V1Y:32/?FLOAT,_V1Z:32/?FLOAT,
-     _V2X:32/?FLOAT,_V2Y:32/?FLOAT,_V2Z:32/?FLOAT,
-     _V3X:32/?FLOAT,_V3Y:32/?FLOAT,_V3Z:32/?FLOAT,
-     OX:32/?FLOAT,OY:32/?FLOAT,OZ:32/?FLOAT,
-     T/binary>> = T0,
-    _CS = {1.0,0.0,0.0,0.0,
-	   0.0,1.0,0.0,0.0,
-	   0.0,0.0,1.0,0.0,
-	   OX,OY,OZ,1.0},
-    %%dbg("Local:~p\n", [CS]),
-    %%trimesh(T, Acc#e3d_mesh{matrix=Id});
-    %% Ignore local coordinate system.
-    trimesh(T, Acc);
-trimesh(<<Tag:16/little,Sz0:32/little,T0/binary>>, Acc) ->
-    Sz = Sz0 - 6,
-    <<Chunk:Sz/binary,T/binary>> = T0,
-    dbg("Unknown mesh chunk: ~.16#: ~P\n", [Tag,Chunk,20]),
-    trimesh(T, Acc);
-trimesh(<<>>, Acc) -> Acc.
+    Acc#e3d_mesh{tx=Tx};
+trimesh(16#4160, <<V1X:32/?FLOAT,V1Y:32/?FLOAT,V1Z:32/?FLOAT,
+		  V2X:32/?FLOAT,V2Y:32/?FLOAT,V2Z:32/?FLOAT,
+		  V3X:32/?FLOAT,V3Y:32/?FLOAT,V3Z:32/?FLOAT,
+		  OX:32/?FLOAT,OY:32/?FLOAT,OZ:32/?FLOAT>>, Acc) ->
+    Matrix = {V1X,V1Y,V1Z,
+	      V2X,V2Y,V2Z,
+	      V3X,V3Y,V3Z,
+	      OX,OY,OZ},
+    [{current_name,Name}] = ets:lookup(?MODULE, current_name),
+    ets:insert(?MODULE, {{local_matrix,Name},Matrix}),
+    dbg("~p: ~p\n", [Name,Matrix]),
+    Acc;
+trimesh(Tag, Chunk, Acc) ->
+    dbg("~.16#: ~P\n", [Tag,Chunk,15]),
+    Acc.
 
-face_desc(<<16#4130:16/little,Sz0:32/little,T0/binary>>, {Faces0,SG}) ->
-    Sz = Sz0 - 6,
-    <<MatList0:Sz/binary,T/binary>> = T0,
+face_desc(Bin, Faces) ->
+    fold_chunks(fun face_desc/3, {Faces,[]}, Bin).
+
+face_desc(16#4130, MatList0, {Faces0,SG}) ->
     {Name0,MatList} = get_cstring(MatList0),
     Name = list_to_atom(Name0),
-    dbg("Material ref: ~p\n", [Name]),
     MatFaces0 = get_mat_faces(MatList),
     MatFaces = sort(MatFaces0),
+    dbg("Material ~p used by ~p face(s)\n", [Name0,length(MatFaces)]),
     Faces = insert_mat(Faces0, MatFaces, 0, Name, []),
-    face_desc(T, {Faces,SG});
-face_desc(<<16#4150:16/little,Sz0:32/little,T0/binary>>, {Faces,_}) ->
-    dbg("Smoothing groups ~p bytes\n", [Sz0]),
-    Sz = Sz0 - 6,
-    <<Smooth0:Sz/binary,T/binary>> = T0,
-    Smooth = get_smooth_groups(Smooth0),
-    face_desc(T, {Faces,Smooth});
-face_desc(<<>>, Acc) -> Acc.
+    {Faces,SG};
+face_desc(16#4150, Smooth, {Faces,_}) ->
+    dbg("Smoothing groups for ~p faces\n", [size(Smooth) div 4]),
+    {Faces,get_smooth_groups(Smooth)}.
 
-material(<<16#A000:16/little,Sz0:32/little,T0/binary>>, Acc) ->
-    Sz = Sz0 - 6,
-    <<Name0:Sz/binary,T/binary>> = T0,
+%% material(Bin, [Material]) -> [Material]
+%%  Collect all materials.
+
+material(Bin, Acc) ->
+    fold_chunks(fun material/3, Acc, Bin).
+
+material(16#A000, Name0, Acc) ->
     {Name1,<<>>} = get_cstring(Name0),
     Name = list_to_atom(Name1),
     dbg("\nMaterial: ~p\n", [Name]),
-    material(T, [{Name,[]}|Acc]);
-material(<<16#A010:16/little,Sz:32/little,T/binary>>, Acc) ->
-    mat_chunk(ambient, Sz, T, Acc);
-material(<<16#A020:16/little,Sz:32/little,T/binary>>, Acc) ->
-    mat_chunk(diffuse, Sz, T, Acc);
-material(<<16#A030:16/little,Sz:32/little,T/binary>>, Acc) ->
-    mat_chunk(specular, Sz, T, Acc);
-material(<<16#A040:16/little,Sz:32/little,T/binary>>, Acc) ->
-    mat_chunk(shininess, Sz, T, Acc);
-material(<<16#A041:16/little,Sz:32/little,T/binary>>, Acc) ->
-    mat_chunk(shininess_level, Sz, T, Acc);
-material(<<16#A050:16/little,Sz0:32/little,T0/binary>>, [{Name,Props}|Acc]) ->
-    dbg("Material ~p, property opacity\n", [Name]),
-    Sz = Sz0 - 6,
-    <<Chunk0:Sz/binary,T/binary>> = T0,
-    Chunk = general(Chunk0),
-    material(T, [{Name,[{opacity,1-Chunk}|Props]}|Acc]);
-material(<<16#A052:16/little,Sz:32/little,T/binary>>, Acc) ->
-    mat_chunk(opacity_falloff, Sz, T, Acc);
-material(<<16#A053:16/little,Sz:32/little,T/binary>>, Acc) ->
-    mat_chunk(reflection_blur, Sz, T, Acc);
-material(<<16#A081:16/little,6:32/little,T/binary>>,
-	 [{Name,Props}|Acc]) ->
+    [{Name,[]}|Acc];
+material(16#A010, Chunk, Acc) ->
+    mat_chunk(ambient, Chunk, Acc);
+material(16#A020, Chunk, Acc) ->
+    mat_chunk(diffuse, Chunk, Acc);
+material(16#A030, Chunk, Acc) ->
+    mat_chunk(specular, Chunk, Acc);
+material(16#A040, Chunk, Acc) ->
+    mat_chunk(shininess, Chunk, Acc);
+material(16#A041, Chunk, Acc) ->
+    mat_chunk(shininess_level, Chunk, Acc);
+material(16#A050, Chunk, [{Name,Props}|Acc]) ->
+    Tr = general(Chunk),
+    [{Name,[{opacity,1-Tr}|Props]}|Acc];
+material(16#A052, Chunk, Acc) ->
+    mat_chunk(opacity_falloff, Chunk, Acc);
+material(16#A053, Chunk, Acc) ->
+    mat_chunk(reflection_blur, Chunk, Acc);
+material(16#A081, _, [{Name,Props}|Acc]) ->
     dbg("Twosided material\n", []),
-    material(T, [{Name,[{twosided,true}|Props]}|Acc]);
-material(<<16#A084:16/little,Sz:32/little,T/binary>>, Acc) ->
-    mat_chunk(emissive, Sz, T, Acc);
-material(<<16#A087:16/little,10:32/little,Wiresize:32/?FLOAT,T/binary>>, Acc) ->
+    [{Name,[{twosided,true}|Props]}|Acc];
+material(16#A084, Chunk, Acc) ->
+    mat_chunk(emissive, Chunk, Acc);
+material(16#A087, <<Wiresize:32/?FLOAT>>, Acc) ->
     dbg("Wire size: ~p\n", [Wiresize]),
-    material(T, Acc);
-material(<<16#A100:16/little,8:32/little,Shading:16/little,T/binary>>, Acc) ->
-     Str = case Shading of
-	       0 -> "Wire";
-	       1 -> "Flat";
-	       2 -> "Gouraud";
-	       3 -> "Phong";
-	       4 -> "Metal";
-	       _ -> "Unknown"
-	   end,
-    dbg("Material shading: ~s\n", [Str]),
-    material(T, Acc);
-material(<<16#A200:16/little,T/binary>>, Acc) ->
-    read_map(T, diffuse, Acc);
-material(<<16#A210:16/little,T/binary>>, Acc) ->
-    read_map(T, opacity, Acc);
-material(<<16#A230:16/little,T/binary>>, Acc) ->
-    read_map(T, bump, Acc);
-material(<<Tag:16/little,Sz0:32/little,T0/binary>>, [{Name,Props}|Acc]) ->
-    Sz = Sz0 - 6,
-    <<Chunk:Sz/binary,T/binary>> = T0,
-    dbg("Unknown material tag: ~.16#: ~P\n", [Tag,Chunk,20]),
-    material(T, [{Name,[{Tag,Chunk}|Props]}|Acc]);
-material(<<>>, Acc) -> Acc.
+    Acc;
+material(16#A100, <<Shading:16/little>>, Acc) ->
+    Str = case Shading of
+	      0 -> "Wire";
+	      1 -> "Flat";
+	      2 -> "Gouraud";
+	      3 -> "Phong";
+	      4 -> "Metal";
+	      _ -> "Unknown"
+	  end,
+    dbg("Shading: ~s\n", [Str]),
+    Acc;
+material(16#A200, Chunk, Acc) ->
+    read_map(diffuse, Chunk, Acc);
+material(16#A210, Chunk, Acc) ->
+    read_map(opacity, Chunk, Acc);
+material(16#A230, Chunk, Acc) ->
+    read_map(bump, Chunk, Acc);
+material(Tag, Chunk, [{Name,Props}|Acc]) ->
+    dbg("~.16#: ~P\n", [Tag,Chunk,20]),
+    [{Name,[{Tag,Chunk}|Props]}|Acc].
 
-read_map(<<Sz0:32/little,T0/binary>>, Type, [{Name,Props}|Acc]) ->
+read_map(Type, Chunk, [{Name,Props}|Acc]) ->
     dbg("Map: ~p\n", [Type]),
-    Sz = Sz0 - 6,
-    <<Chunk:Sz/binary,T/binary>> = T0,
-    Map = read_map_chunks(Chunk, none),
-    material(T, [{Name,[{map,Type,Map}|Props]}|Acc]).
+    MapPs = fold_chunks(fun texture/3, [], Chunk),
+    Map = proplists:get_value(filename, MapPs),
+    [{Name,[{map,Type,Map}|Props]}|Acc].
 
-read_map_chunks(<<16#A300:16/little,Sz0:32/little,T0/binary>>, _Acc) ->
-    Sz = Sz0 - 6 - 1,
-    <<Filename0:Sz/binary,_:8,T/binary>> = T0,
-    Filename = binary_to_list(Filename0),
+texture(16#A300, Chunk, Acc) ->
+    {Filename,_} = get_cstring(Chunk),
     dbg("Filename: ~s\n", [Filename]),
-    read_map_chunks(T, Filename);
-read_map_chunks(<<16#A351:16/little,Sz0:32/little,T0/binary>>, Acc) ->
-    Sz = Sz0 - 6,
-    <<Params:Sz/binary,T/binary>> = T0,
+    [{filename,Filename}|Acc];
+texture(16#A351, Params, Acc) ->
     dbg("Params: ~p\n", [Params]),
-    read_map_chunks(T, Acc);
-read_map_chunks(<<Tag:16/little,Sz0:32/little,T0/binary>>, Acc) ->
-    Sz = Sz0 - 6,
-    <<_Chunk:Sz/binary,T/binary>> = T0,
-    dbg("Unknown map sub-chunk: ~.16E: ~P\n", [Tag,_Chunk,20]),
-    read_map_chunks(T, Acc);
-read_map_chunks(<<>>, Acc) -> Acc.
+    Acc;
+texture(Tag, Chunk, Acc) ->
+    dbg("~.16#: ~P\n", [Tag,Chunk,20]),
+    Acc.
 
 reformat_material([{Name,Mat}|T]) ->
     Opac = proplists:get_value(opacity, Mat, 1.0),
@@ -318,12 +327,10 @@ reformat_mat([], _Opac, Ogl, Maps, Tds) ->
 reformat_color({Key,{R,G,B}}, Opac) ->
     {Key,{R,G,B,Opac}}.
     
-mat_chunk(Type, Sz0, Bin, [{Name,Props}|Acc]) ->
-    Sz = Sz0 - 6,
-    <<Chunk:Sz/binary,T/binary>> = Bin,
+mat_chunk(Type, Chunk, [{Name,Props}|Acc]) ->
     Value = general(Chunk),
     dbg("property ~p = ~p\n", [Type,Value]),
-    material(T, [{Name,[{Type,Value}|Props]}|Acc]).
+    [{Name,[{Type,Value}|Props]}|Acc].
 
 general(<<16#0010:16/little,_Sz:32/little, 
 	 R:32/?FLOAT,G:32/?FLOAT,B:32/?FLOAT,
@@ -340,13 +347,55 @@ general_rest(<<>>) -> ok;
 general_rest(<<Tag:16/little,Sz0:32/little,T0/binary>>) ->
     Sz = Sz0 - 6,
     <<Chunk:Sz/binary,T/binary>> = T0,
-    dbg("Unknown general tag: ~.16#: ~P\n", [Tag,Chunk,20]),
+    dbg("~.16#: ~P\n", [Tag,Chunk,15]),
     general_rest(T).
 
+fix_transform(Objs) ->
+    case ets:member(?MODULE, "$$$DUMMY") of
+	false -> reverse(Objs);
+	true -> fix_transform_0(Objs)
+    end.
+
+fix_transform_0(Objs) ->
+    lists:foldl(fun(O, A) -> [fix_transform_1(O)|A] end, [], Objs).
+
+fix_transform_1(#e3d_object{name=Name,obj=Mesh}=Obj) ->
+    Matrix = get_transform(Name),
+    dbg("~p: ~p\n", [Name,Matrix]),
+    Obj#e3d_object{obj=Mesh#e3d_mesh{matrix=Matrix}}.
+
+get_transform(Name) ->
+    case ets:lookup(?MODULE, Name) of
+	[] -> identity;
+	[{Name,ParentId}] ->
+	    case get_name_and_matrix(ParentId) of
+		identity -> identity;
+		{Parent,Matrix} ->
+		    e3d_mat:mul(get_transform(Parent), Matrix)
+	    end
+    end.
+
+get_name_and_matrix(Id) ->
+    case ets:lookup(?MODULE, Id) of
+	[] -> identity;
+	[{Id,Name}] ->
+	    case ets:lookup(?MODULE, {local_matrix,Name}) of
+		[] -> identity;
+		[{_,Matrix}] -> {Name,Matrix}
+	    end
+    end.
+
 %%%
 %%% Utilities.
 %%%
 
+fold_chunks(Fun, Acc0, <<Tag:16/little,Sz0:32/little,T0/binary>>) ->
+    Sz = Sz0 - 6,
+    <<Contents:Sz/binary,T/binary>> = T0,
+    Acc = Fun(Tag, Contents, Acc0),
+    fold_chunks(Fun, Acc, T);
+fold_chunks(_Fun, Acc, <<>>) -> Acc.
+
 insert_mat([_|_]=Fs, [MatFace|[MatFace|_]=Mfs], Face, Mat, Acc) ->
     insert_mat(Fs, Mfs, Face, Mat, Acc);
 insert_mat([F|Fs], [MatFace|_]=Mfs, Face, Mat, Acc) when Face < MatFace ->
