@@ -8,7 +8,7 @@
 %%
 %%  See the file "license.terms" for information on usage and redistribution
 %%  of this file, and for a DISCLAIMER OF ALL WARRANTIES.
-%%     $Id: wpc_autouv.erl,v 1.259 2004/06/02 04:57:55 bjorng Exp $
+%%     $Id: wpc_autouv.erl,v 1.271 2004/10/20 11:21:47 dgud Exp $
 
 -module(wpc_autouv).
 
@@ -85,7 +85,7 @@ start_uvmap_2(Action, Name, Id, #st{shapes=Shs}=St) ->
     Title = "AutoUV: " ++ ObjName,
     {X,Y,W,H} = init_drawarea(),
     Props = [{display_lists,Name}|wings_view:initial_properties()],
-    CreateToolbar = fun(N, P, Wi) -> wings:create_toolbar(N, P, Wi) end,
+    CreateToolbar = fun(N, P, Wi) -> wings_toolbar:create(N, P, Wi) end,
     wings_wm:toplevel(Name, Title, {X,Y,highest}, {W,H},
 		      [resizable,closable,menubar,{properties,Props},
 		       {toolbar,CreateToolbar}], Op),
@@ -157,7 +157,7 @@ create_uv_state(Charts, MatName, We, GeomSt) ->
 		 pan_x=-0.5,
 		 pan_y=-0.5,
 		 fov=90.0,
-		 hither=0.1,
+		 hither=0.0001,
 		 yon=50.0},
     wings_view:set_current(View),
 
@@ -322,15 +322,8 @@ add_material(Im = #e3d_image{}, _, MatName,St) ->
 %%%% Menus.
 
 command_menu(body, X, Y) ->
-    Rotate = [{"Free", free, "Rotate freely"},
-	      {"90"++[?DEGREE]++" CW",-90,
-	       "Rotate selection 90 degrees clockwise"},
-	      {"90"++[?DEGREE]++" CCW",90,
-	       "Rotate selection 90 degrees counter-clockwise"},
-	      {"180"++[?DEGREE],180,"Rotate selection 180 degrees"},
-	      separator,
-	      {"Flip Horizontal",flip_horizontal,"Flip selection horizontally"},
-	      {"Flip Vertical",flip_vertical,"Flip selection vertically"}],
+    Rotate = rotate_directions(),
+
     Scale =  [{"Uniform",    scale_uniform, "Scale in both directions"},
 	      {"Horizontal", scale_x, "Scale horizontally (X dir)"},
 	      {"Vertical",   scale_y, "Scale vertically (Y dir)"}],
@@ -341,6 +334,11 @@ command_menu(body, X, Y) ->
 	    {"Scale", {scale, Scale}, "Scale selected charts"},
 	    {"Rotate", {rotate, Rotate}, "Rotate selected charts"},
 	    separator,
+	    {"Flip",{flip,
+		     [{"Horizontal",horizontal,"Flip selection horizontally"},
+		      {"Vertical",vertical,"Flip selection vertically"}]},
+	     "Flip selected charts"},
+	    separator,
 	    {"Tighten",tighten,
 	     "Move UV coordinates towards average midpoint"},
 	    separator,
@@ -372,7 +370,10 @@ command_menu(edge, X, Y) ->
 	    {basic,separator},
 	    {"Move",move,"Move selected edges",[magnet]},
 	    {"Scale",{scale,Scale},"Scale selected edges"},
-	    {"Rotate",{rotate,Rotate},"Rotate selected edges"}
+	    {"Rotate",{rotate,Rotate},"Rotate selected edges"},
+	    separator,
+	    {"Stitch", stitch, "Stitch edges/charts"},
+	    {"Cut", cut_edges, "Cut selected edges"}
 	   ] ++ option_menu(),
     wings_menu:popup_menu(X,Y, auv, Menu);
 command_menu(vertex, X, Y) ->
@@ -384,9 +385,15 @@ command_menu(vertex, X, Y) ->
 	    {"Scale",{scale,Scale},"Scale selected vertices"},
 	    {"Rotate",{rotate,Rotate},"Rotate selected vertices"},
 	    separator,
+	    {"Flatten",{flatten,
+			[{"X", x, "Flatten horizontally"},
+			 {"Y", y, "Flatten vertically"}]}, 
+	     "Flatten selected vertices"},
 	    {"Tighten",tighten,
 	     "Move UV coordinates towards average midpoint",
-	     [magnet]}
+	     [magnet]},
+	    separator, 
+	    {"Unfold", lsqcm, "Unfold the chart (without moving the selected vertices)"}
 	   ] ++ option_menu(),
     wings_menu:popup_menu(X,Y, auv, Menu);
 command_menu(_, X, Y) ->
@@ -404,10 +411,7 @@ rotate_directions() ->
       "Rotate selection 90 degrees clockwise"},
      {"90"++[?DEGREE]++" CCW",90,
       "Rotate selection 90 degrees counter-clockwise"},
-     {"180"++[?DEGREE],180,"Rotate selection 180 degrees"},
-     separator,
-     {"Flip Horizontal",flip_horizontal,"Flip selection horizontally"},
-     {"Flip Vertical",flip_vertical,"Flip selection vertically"}].
+     {"180"++[?DEGREE],180,"Rotate selection 180 degrees"}].
 
 option_menu() ->
     [separator,
@@ -446,6 +450,13 @@ handle_event(Ev, St) ->
 	Other -> Other
     end.
 
+%% Short cut for move selected
+handle_event_1(#mousebutton{state=?SDL_PRESSED,
+			    button=?SDL_BUTTON_LEFT,
+			    mod=Mod},
+	       St = #st{sel = Sel}) 
+  when Sel /= [], (Mod band ?ALT_BITS) =/= 0 ->
+    handle_command(move,St);
 handle_event_1(Ev, St) ->
     case wings_pick:event(Ev, St) of
 	next -> handle_event_2(Ev, St);
@@ -460,7 +471,9 @@ handle_event_2(Ev, St) ->
 
 handle_event_3({current_state,geom_display_lists,GeomSt}, AuvSt) ->
     new_geom_state(GeomSt, AuvSt);
-handle_event_3(#mousebutton{state=?SDL_RELEASED,button=?SDL_BUTTON_RIGHT,x=X0,y=Y0},
+handle_event_3(#mousebutton{state=?SDL_RELEASED,
+			    button=?SDL_BUTTON_RIGHT,
+			    x=X0,y=Y0},
 	       #st{selmode=Mode0,sel=Sel}) ->
     {X,Y} = wings_wm:local2global(X0, Y0),
     Mode = case Sel of 
@@ -475,17 +488,24 @@ handle_event_3({action,{auv,create_texture}},_St) ->
 handle_event_3({action,{auv,{draw_options,Opt}}}, #st{bb=Uvs}=St) ->
     #uvstate{st=GeomSt0,matname=MatName0} = Uvs,
     Tx = ?SLOW(auv_texture:get_texture(St, Opt)),
-    {GeomSt,MatName} = add_material(Tx, undefined, MatName0, GeomSt0),
-    wings_wm:send(geom, {new_state,GeomSt}),
-    get_event(St#st{bb=Uvs#uvstate{st=GeomSt,matname=MatName}});
+    case MatName0 of 
+	none -> 
+	    Id = wings_image:new("UVmap", Tx),
+	    wings_image:window(Id),
+	    get_event(St);
+	_ ->
+	    {GeomSt,MatName} = add_material(Tx, undefined, MatName0, GeomSt0),
+	    wings_wm:send(geom, {new_state,GeomSt}),
+	    get_event(St#st{bb=Uvs#uvstate{st=GeomSt,matname=MatName}})
+    end;
 handle_event_3({action,{auv,{remap,Method}}}, St0) ->
     St = remap(Method, St0),
     get_event(St);
+handle_event_3({action,{auv,lsqcm}}, St0) ->
+    St = remap(lsqcm, St0),
+    get_event(St);
 
 %% Others
-handle_event_3({action,{auv,quit}}, _St) ->
-    cleanup_before_exit(),
-    delete;
 handle_event_3({vec_command,Command,_St}, _) when is_function(Command) ->
     %% Use to execute command with vector arguments (see wings_vec.erl).
     catch Command();
@@ -494,6 +514,9 @@ handle_event_3(close, _St) ->
     delete;
 handle_event_3({callback,Fun}, _) when is_function(Fun) ->
     Fun();
+handle_event_3({action,{auv,quit}}, _St) ->
+    cleanup_before_exit(),
+    delete;
 handle_event_3({action,{auv,Cmd}}, St) ->
     handle_command(Cmd, St);
 handle_event_3({action,{select,Command}}, St0) ->
@@ -508,14 +531,31 @@ handle_event_3({action,{edit,undo}}=Act, _) ->
     wings_wm:send(geom, Act);
 handle_event_3({action,{edit,redo}}=Act, _) ->
     wings_wm:send(geom, Act);
+handle_event_3({action,Ev}, St) ->
+    case Ev of
+	{_, {move,_}} ->
+	    handle_command(move,St);
+	{_, {rotate,_}} ->
+	    handle_command({rotate,free},St);
+	{_, {scale,{x,_}}} ->
+	    handle_command({scale,scale_x},St);
+	{_, {scale,{y,_}}} ->
+	    handle_command({scale,scale_y},St);
+	{_, {scale,_}} ->
+	    handle_command({scale,scale_uniform},St);
+	_ ->
+%%	    io:format("MissEvent ~p~n", [Ev]),
+	    keep
+    end;
 handle_event_3(got_focus, _) ->
     Msg1 = wings_util:button_format("Select"),
     Msg2 = wings_camera:help(),
     Msg3 = wings_util:button_format([], [], "Show menu"),
-    Message = wings_util:join_msg([Msg1,Msg2,Msg3]),
+    Msg4 = "[Alt]+L: Move selected",
+    Message = wings_util:join_msg([Msg1,Msg2,Msg3,Msg4]),
     wings_wm:message(Message),
     wings_wm:dirty();
-handle_event_3(_, _) ->
+handle_event_3(_Event, _) ->
     keep.
 
 new_state(#st{bb=#uvstate{}=Uvs}=St0) ->
@@ -536,11 +576,11 @@ handle_command({scale,scale_y}, St) ->
     drag(wings_scale:setup({y,center}, St));
 handle_command({rotate,free}, St) ->
     drag(wings_rotate:setup({free,center}, St));
-handle_command({rotate,flip_horizontal}, St0) ->
+handle_command({flip,horizontal}, St0) ->
     St1 = wpa:sel_map(fun(_, We) -> flip_horizontal(We) end, St0),
     St = update_selected_uvcoords(St1),
     get_event(St);
-handle_command({rotate,flip_vertical}, St0) ->
+handle_command({flip,vertical}, St0) ->
     St1 = wpa:sel_map(fun(_, We) -> flip_vertical(We) end, St0),
     St = update_selected_uvcoords(St1),
     get_event(St);
@@ -556,6 +596,26 @@ handle_command(delete, St) ->
     get_event(delete_charts(St));
 handle_command({'ASK',Ask}, St) ->
     wings:ask(Ask, St, fun handle_command/2);
+handle_command({flatten, Plane}, St0 = #st{selmode=vertex}) ->
+    {save_state, St1} = wings_vertex_cmd:flatten(Plane, St0),
+    St = update_selected_uvcoords(St1),
+    get_event(St);
+handle_command(stitch, St0 = #st{selmode=edge}) ->
+    St1 = stitch(St0),
+    AuvSt = #st{bb=#uvstate{id=Id,st=Geom}} = update_selected_uvcoords(St1),
+    %% Do something here, i.e. restart uvmapper.
+    St = rebuild_charts(gb_trees:get(Id,Geom#st.shapes),AuvSt,false,[]),
+    get_event(St);
+handle_command(cut_edges, St0 = #st{selmode=edge,bb=#uvstate{id=Id,st=Geom}}) ->
+    Es = wpa:sel_fold(fun(Es,#we{name=#ch{emap=Emap}},A) ->
+			      [auv_segment:map_edge(E,Emap)
+			       || E<-gb_sets:to_list(Es)] ++ A
+		      end, [], St0),
+    %% Do something here, i.e. restart uvmapper.
+    St = rebuild_charts(gb_trees:get(Id,Geom#st.shapes),St0,false,Es),
+    get_event(St);
+
+%%    get_event(St);
 handle_command(_, #st{sel=[]}) ->
     keep.
 
@@ -614,6 +674,58 @@ delete_charts(#st{shapes=Shs0}=St0) ->
 		       end, Shs0, St),
     St#st{shapes=Shs,sel=[]}.
 
+stitch(St0) ->
+    Map0 = wpa:sel_fold(fun(Es,#we{id=Id,name=#ch{emap=Emap}},A) ->
+				[{auv_segment:map_edge(E,Emap),{Id,E}}
+				 || E<-gb_sets:to_list(Es)] ++ A
+			end, [], St0),
+    Map1 = sofs:to_external(sofs:relation_to_family(sofs:relation(Map0))),
+    foldl(fun stitch/2, St0, Map1).
+
+stitch({_E, [_]}, St) -> St; %% Both edges not selected ignore
+stitch({_E, [{Id,E1id},{Id,E2id}]},St0 = #st{shapes=Sh0}) -> %% Same We
+    We = #we{name=#ch{vmap=Vmap},es=Etab,vp=Vpos0} = gb_trees:get(Id,Sh0),
+    #edge{vs=Vs1,ve=Ve1} = gb_trees:get(E1id,Etab),
+    #edge{vs=Vs2,ve=Ve2} = gb_trees:get(E2id,Etab),
+    Vs1map = auv_segment:map_vertex(Vs1, Vmap),
+    Same = case auv_segment:map_vertex(Vs2, Vmap) of
+	       Vs1map -> [{Vs1,Vs2},{Ve1,Ve2}];
+	       _ -> [{Vs1,Ve2},{Ve1,Vs2}]
+	   end,
+    Vpos = average_pos(Same, Vpos0),
+    St0#st{shapes = gb_trees:update(Id, We#we{vp=Vpos}, Sh0)};
+
+stitch({_E, [{Id1,E1id},{Id2,E2id}]}, St0 = #st{shapes=Sh0}) -> % Merge Charts
+    We1 = #we{name=#ch{vmap=Vmap1},es=Etab1,vp=Vpos1} = gb_trees:get(Id1,Sh0),
+    We2 = #we{name=#ch{vmap=Vmap2},es=Etab2,vp=Vpos2} = gb_trees:get(Id2,Sh0),
+    #edge{vs=Vs1,ve=Ve1} = gb_trees:get(E1id,Etab1),
+    #edge{vs=Vs2,ve=Ve2} = gb_trees:get(E2id,Etab2),
+    Vs1map = auv_segment:map_vertex(Vs1, Vmap1),
+    Same = case auv_segment:map_vertex(Vs2, Vmap2) of
+	       Vs1map -> [{Vs1,Vs2},{Ve1,Ve2}];
+	       _ -> [{Vs1,Ve2},{Ve1,Vs2}]
+	   end,
+    {Vp1,Vp2} = average_pos(Same, Vpos1, Vpos2),
+    Sh1 = gb_trees:update(Id1, We1#we{vp=Vp1}, Sh0),
+    Sh  = gb_trees:update(Id2, We2#we{vp=Vp2}, Sh1),
+    St0#st{shapes = Sh}.
+
+average_pos([{V1,V2}|R], Vpos0) ->
+    Pos = e3d_vec:average(gb_trees:get(V1,Vpos0),gb_trees:get(V2,Vpos0)),
+    Vpos1 = gb_trees:update(V1,Pos,Vpos0),
+    Vpos  = gb_trees:update(V2,Pos,Vpos1),
+    average_pos(R, Vpos);
+average_pos([],Vpos) -> Vpos.
+
+average_pos([{V1,V2}|R], Vpos1,Vpos2) ->
+    Pos = e3d_vec:average(gb_trees:get(V1,Vpos1),gb_trees:get(V2,Vpos2)),
+    Vp1 = gb_trees:update(V1,Pos,Vpos1),
+    Vp2 = gb_trees:update(V2,Pos,Vpos2),
+    average_pos(R, Vp1,Vp2);
+average_pos([],Vpos1,Vpos2) -> {Vpos1,Vpos2}.
+
+
+%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 drag_filter({image,_,_}) ->
     {yes,"Drop: Change the texture image"};
 drag_filter(_) -> no.
@@ -657,20 +769,31 @@ new_geom_state_1(Shs, #st{bb=#uvstate{id=Id,st=#st{shapes=Orig}}}=AuvSt) ->
 	{none,_} -> delete;
 	{{value,We},{value,We}} -> {AuvSt,false};
 	{{value,#we{es=Etab}},{value,#we{es=Etab}}} -> {AuvSt,false};
-	{{value,We},_} -> new_geom_state_2(We, AuvSt)
+	{{value,We},_} -> {rebuild_charts(We, AuvSt,true,[]),true}
     end.
 
-new_geom_state_2(We, #st{shapes=OldCharts}=St) ->
+rebuild_charts(We, #st{shapes=OldCharts}=St,Reuse,ExtraCuts) ->
     Faces = wings_we:uv_mapped_faces(We),
     FvUvMap = auv_segment:fv_to_uv_map(Faces, We),
-    {Charts1,Cuts} = auv_segment:uv_to_charts(Faces, FvUvMap, We),
+    {Charts0,Cuts0} = auv_segment:uv_to_charts(Faces, FvUvMap, We),
+    {Charts1,Cuts} =
+	case ExtraCuts of
+	    [] ->  {Charts0,Cuts0};
+	    _ ->
+		Cuts1 = foldl(fun(Edge,Cuts) -> gb_sets:add(Edge,Cuts) end, 
+			      Cuts0, ExtraCuts),
+		auv_segment:normalize_charts(Charts0,Cuts1,We)
+	end,
     Charts2 = auv_segment:cut_model(Charts1, Cuts, We),
-    Charts3 = update_charts(Charts2, gb_trees:values(OldCharts)),
+    Old = if Reuse -> gb_trees:values(OldCharts);
+	     true -> []
+	  end,
+    Charts3 = update_charts(Charts2, Old),
     Charts4 = build_map(Charts3, FvUvMap, []),
     Charts = gb_trees:from_orddict(keysort(1, Charts4)),
     wings_wm:set_prop(wireframed_objects,
 		      gb_sets:from_list(gb_trees:keys(Charts))),
-    {St#st{sel=[],shapes=Charts},true}.
+    St#st{sel=[],shapes=Charts}.
 
 update_charts(Cs, OldCs) ->
     Id = length(Cs),
@@ -831,8 +954,7 @@ update_body_sel(face, Elems, Charts) ->
     face_sel_to_body(Charts, Elems, []);
 update_body_sel(body, _, Charts) ->
     body_sel_to_body(Charts, []);
-update_body_sel(Mode, _, _) ->
-    io:format("~p: ~p\n", [?LINE,Mode]),
+update_body_sel(_Mode, _, _) ->
     [].
 
 face_sel_to_body([{K,We}|Cs], Faces, Sel) ->
@@ -903,50 +1025,68 @@ flip_horizontal(We) ->
 flip_vertical(We) ->
     flip(e3d_mat:scale(1.0, -1.0, 1.0), We).
 
-flip(Flip, We) ->
-    Center = wings_vertex:center(We),
+flip(Flip, We0) ->
+    Center = wings_vertex:center(We0),
     T0 = e3d_mat:translate(e3d_vec:neg(Center)),
     T1 = e3d_mat:mul(Flip, T0),
     T = e3d_mat:mul(e3d_mat:translate(Center), T1),
-    wings_we:transform_vs(T, We).
+    We = wings_we:transform_vs(T, We0),
+    wings_we:invert_normals(We).
 
-
-remap(Method, #st{sel=Sel}=St0) ->
+remap(Method, #st{sel=Sel,selmode=Mode}=St0) ->
     wings_pb:start("remapping"),
     wings_pb:update(0.001),
     N = length(Sel),
-    {St,_} = wings_sel:mapfold(fun(_, We, I) ->
-				       Msg = "chart " ++ integer_to_list(I),
-				       wings_pb:update(I/N, Msg),
-				       {remap(Method, We, St0),I+1}
-			       end, 1, St0),
+    GetUV = fun(V,Vtab) -> 
+		    {S,T,_} = gb_trees:get(V,Vtab),
+		    {V,{S,T}}
+	    end,
+    Remap = 
+	fun(Vs, We = #we{vp=Vtab}, I) ->
+		Msg = "chart " ++ integer_to_list(I),
+		wings_pb:update(I/N, Msg),
+		Pinned = case Mode of
+			     vertex -> 
+				 [GetUV(V,Vtab) || 
+				     V <- gb_sets:to_list(Vs)];
+			     _ -> none
+			 end,
+		{remap(Method, Pinned, We, St0),I+1}
+	end,
+    {St,_} = wings_sel:mapfold(Remap, 1, St0),
     wings_pb:done(update_selected_uvcoords(St)).
 
-remap(stretch_opt, We, St) ->
+remap(stretch_opt, _, We, St) ->
     Vs3d = orig_pos(We, St),
     ?SLOW(auv_mapping:stretch_opt(We, Vs3d));
-remap(Type, #we{name=Ch}=We0, St) ->
+remap(Type, Pinned, #we{name=Ch}=We0, St) ->
     [Lower,Upper] = wings_vertex:bounding_box(We0),
     {W,H,_} = e3d_vec:sub(Upper, Lower),
 
     %% Get 3d positions (even for mapped vs).
     Vs3d = orig_pos(We0, St),
-    Vs0 = auv_mapping:map_chart(Type, We0#we{vp=Vs3d}),
-    We1 = We0#we{vp=gb_trees:from_orddict(sort(Vs0))},    
-    Fs = wings_we:visible(We1),
-    {{Dx,Dy},Vs1} = auv_placement:center_rotate(Fs, We1),
-    Center = wings_vertex:center(We0),
-    Scale = if Dx > Dy -> W / Dx;
- 	       true -> H / Dy
-	    end,
-    Smat = e3d_mat:scale(Scale),
-    Cmat = e3d_mat:translate(Center),
-    Fix  = e3d_mat:mul(Cmat, Smat),
-    Vs = foldl(fun({VId, Pos}, Acc) -> 
-		       [{VId,e3d_mat:mul_point(Fix, Pos)}|Acc] 
-	       end, [], Vs1),
-    We0#we{vp=gb_trees:from_orddict(sort(Vs)),
-	   name=Ch#ch{size={Dx*Scale, Dy*Scale}}}.
+    case auv_mapping:map_chart(Type, We0#we{vp=Vs3d}, Pinned) of
+	{error, Msg} -> 
+	    wings_pb:done(),
+	    wings_util:message(Msg),
+	    We0;
+	Vs0 -> 
+	    We1 = We0#we{vp=gb_trees:from_orddict(sort(Vs0))},
+	    Fs = wings_we:visible(We1),
+	    {{Dx,Dy},Vs1} = auv_placement:center_rotate(Fs, We1),
+	    Center = wings_vertex:center(We0),
+	    Scale = if Dx > Dy -> W / Dx;
+		       true -> H / Dy
+		    end,
+	    Smat = e3d_mat:scale(Scale),
+	    Cmat = e3d_mat:translate(Center),
+	    Fix  = e3d_mat:mul(Cmat, Smat),
+	    Vs = foldl(fun({VId, Pos}, Acc) -> 
+			       [{VId,e3d_mat:mul_point(Fix, Pos)}|Acc] 
+		       end, [], Vs1),
+	    We0#we{vp=gb_trees:from_orddict(sort(Vs)),
+		   name=Ch#ch{size={Dx*Scale, Dy*Scale}}}
+    end.
 
 %% gb_tree of every vertex orig 3d position, (including the new cut ones)
 orig_pos(We = #we{name=#ch{vmap=Vmap}},St) ->
@@ -1042,7 +1182,7 @@ geom2auv_faces(Fs, _) ->
     Fs.
 
 auv2geom_vs(Vs, #we{name=#ch{vmap=Vmap}}) ->
-    [auv_segment:map_vertex(V, Vmap) || V <- Vs].
+    sort([auv_segment:map_vertex(V, Vmap) || V <- Vs]).
 
 geom2auv_vs(Vs, #we{name=#ch{vmap=Vmap},vp=Vtab}) ->
     geom2auv_vs_1(gb_trees:keys(Vtab), gb_sets:from_list(Vs), Vmap, []).
@@ -1054,8 +1194,16 @@ geom2auv_vs_1([V|Vs], VsSet, Vmap, Acc) ->
     end;
 geom2auv_vs_1([], _, _, Acc) -> sort(Acc).
 
-auv2geom_edges(Es, #we{name=#ch{me=Me}}) ->
-    Es -- Me.
+auv2geom_edges(Es, #we{name=#ch{emap=Emap}}) ->
+    sort([auv_segment:map_edge(E, Emap) || E <- Es]).
 
-geom2auv_edges(Es, _We) ->
-    Es.
+geom2auv_edges(Es, #we{name=#ch{emap=Emap0}}) ->
+    A2We = sofs:relation(gb_trees:to_list(Emap0)),
+    W2Ae = sofs:relation_to_family(sofs:converse(A2We)),
+    Tab = gb_trees:from_orddict(sofs:to_external(W2Ae)),
+    foldl(fun(Edge, Acc) ->
+		  case gb_trees:lookup(Edge, Tab) of
+		      none -> [Edge|Acc];
+		      {value,Hits} -> Hits ++ Acc
+		  end
+	  end, [], Es).
