@@ -8,15 +8,15 @@
 %%  See the file "license.terms" for information on usage and redistribution
 %%  of this file, and for a DISCLAIMER OF ALL WARRANTIES.
 %%
-%%     $Id: wings.erl,v 1.308 2004/05/19 19:36:22 dgud Exp $
+%%     $Id: wings.erl,v 1.324 2004/11/14 13:47:26 bjorng Exp $
 %%
 
 -module(wings).
 -export([start/0,start_halt/0,start_halt/1]).
 -export([caption/1,redraw/1,redraw/2,init_opengl/1,command/2]).
 -export([mode_restriction/1,clear_mode_restriction/0,get_mode_restriction/0]).
--export([create_toolbar/3]).
 -export([ask/3]).
+-export([init_menubar/0]).
 
 -export([register_postdraw_hook/3,unregister_postdraw_hook/2]).
 
@@ -50,12 +50,13 @@ halt_loop(Wings) ->
 	{'EXIT',Wings,{window_crash,Name,Reason}} ->
 	    Log = wings_util:crash_log(Name, Reason),
 	    io:format("\n\n"),
-	    io:format("Fatal internal error - log written to ~s\n", [Log]),
+	    io:format(?STR(halt_loop,1,"Fatal internal error - log written to ~s\n"),
+		      [Log]),
 	    ok;
 	{'EXIT',Wings,Reason} ->
-	    Log = wings_util:crash_log("<Unknown Window Name>", Reason),
+	    Log = wings_util:crash_log(?STR(halt_loop,2,"<Unknown Window Name>"), Reason),
 	    io:format("\n\n"),
-	    io:format("Fatal internal error - log written to ~s\n", [Log]),
+	    io:format(?STR(halt_loop,3,"Fatal internal error - log written to ~s\n"), [Log]),
 	    ok;
 	_Other ->				%Can't happen.
 	    halt_loop(Wings)
@@ -74,21 +75,12 @@ do_spawn(File, Flags) ->
 
 init(File) ->
     register(wings, self()),
-
+    
     OsType = os:type(),
     put(wings_os_type, OsType),
-
-    case OsType of
-	{win32,_} ->
-	    io:format("\n\nNote: The above messages about failing "
-		      "to locate TCP/IP parameters are normal.\n"
-		      "It is done to prevent Erlang from contacting "
-		      "DNS name servers on the Internet\n"
-		      "(harmless, but no need for Wings to do it)\n\n");
-	_ -> ok
-    end,
-
     wings_pref:init(),
+    wings_lang:init(),
+    
     group_leader(wings_console:start(), self()),
     wings_init:init(),
     wings_text:init(),
@@ -99,7 +91,7 @@ init(File) ->
 
     wings_camera:init(),
     wings_vec:init(),
-
+        
     Empty = gb_trees:empty(),
     St0 = #st{shapes=Empty,
 	      selmode=face,
@@ -130,8 +122,11 @@ init(File) ->
     
     Props = initial_properties(),
     {{X,Y},{W,H}} = wings_wm:win_rect(desktop),
-    wings_wm:toplevel(geom, "Geometry", {X,Y,highest}, {W,H-80},
-		      [resizable,{anchor,nw},{toolbar,fun create_toolbar/3},
+    wings_wm:toplevel(geom, geom_title(geom),
+		      {X,Y,highest}, {W,H-80},
+		      [resizable,{anchor,nw},
+		       {toolbar,
+			fun(A, B, C) -> wings_toolbar:create(A, B, C) end},
 		       menubar,{properties,Props}],
 		      Op),
     wings_wm:menubar(geom, get(wings_menu_template)),
@@ -161,12 +156,13 @@ new_viewer(St) ->
     Name = {geom,N},
     new_viewer(Name, Pos, Size, Props, ToolbarHidden, St).
 
-new_viewer({geom,N}=Name, {X,Y}, Size, Props, ToolbarHidden, St) ->
+new_viewer(Name, {X,Y}, Size, Props, ToolbarHidden, St) ->
     Op = main_loop_noredraw(St),
-    Title = "Geometry #" ++ integer_to_list(N),
+    Title = geom_title(Name),
     wings_wm:toplevel(Name, Title, {X,Y,highest}, Size,
 		      [resizable,closable,{anchor,nw},
-		       {toolbar,fun create_toolbar/3},
+		       {toolbar,
+			fun(A, B, C) -> wings_toolbar:create(A, B, C) end},
 		       menubar,
 		       {properties,Props}],
 		      Op),
@@ -362,9 +358,9 @@ handle_event_3(need_save, St) ->
 handle_event_3({new_default_command,DefCmd}, St) ->
     main_loop_noredraw(St#st{def=DefCmd});
 handle_event_3(got_focus, _) ->
-    Msg1 = wings_util:button_format("Select"),
+    Msg1 = wings_util:button_format(?STR(handle_event_3,1,"Select")),
     Msg2 = wings_camera:help(),
-    Msg3 = wings_util:button_format([], [], "Show menu"),
+    Msg3 = wings_util:button_format([], [], ?STR(handle_event_3,2,"Show menu")),
     Message = wings_util:join_msg([Msg1,Msg2,Msg3]),
     wings_wm:message(Message),
     wings_wm:dirty();
@@ -375,6 +371,11 @@ handle_event_3({note,_}, _) ->
     keep;
 handle_event_3({drop,Pos,DropData}, St) ->
     handle_drop(DropData, Pos, St);
+handle_event_3(language_changed, _) ->
+    This = wings_wm:this(),
+    wings_wm:toplevel_title(This, geom_title(This)),
+    wings_wm:menubar(This, get(wings_menu_template)),
+    keep;
 handle_event_3(ignore, _St) -> keep.
 
 do_hotkey(Ev, #st{sel=[]}=St0) ->
@@ -663,102 +664,106 @@ popup_menu(X, Y, #st{selmode=Mode}=St) ->
     end.
 
 init_menubar() ->
-    Menus = [{"File",file,fun(St) -> wings_file:menu(St) end},
-	     {"Edit",edit,fun edit_menu/1},
-	     {"View",view,fun(St) -> wings_view:menu(St) end},
-	     {"Select",select,fun(St) -> wings_sel_cmd:menu(St) end},
-	     {"Tools",tools,fun tools_menu/1},
-	     {"Window",window,fun window_menu/1},
-	     {"Help",help,fun(St) -> wings_help:menu(St) end}],
+   Menus = [{?STR(init_menubar,1,"File"),file,fun(St) -> wings_file:menu(St) end},
+          {?STR(init_menubar,2,"Edit"),edit,fun edit_menu/1},
+          {?STR(init_menubar,3,"View"),view,fun(St) -> wings_view:menu(St) end},
+          {?STR(init_menubar,4,"Select"),select,fun(St) -> wings_sel_cmd:menu(St) end},
+          {?STR(init_menubar,5,"Tools"),tools,fun tools_menu/1},
+          {?STR(init_menubar,6,"Window"),window,fun window_menu/1},
+          {?STR(init_menubar,7,"Help"),help,fun(St) -> wings_help:menu(St) end}],
     put(wings_menu_template, Menus).
 
 edit_menu(St) ->
-    UndoInfo = lists:flatten(["Delete undo history to reclaim memory (",
-			      undo_info(St),")"]),
-    [{"Undo/Redo",undo_toggle,"Undo or redo the last command"},
-     {"Redo",redo,"Redo the last command that was undone"},
-     {"Undo",undo,"Undo the last command"},
+    UndoInfo = lists:flatten([?STR(edit_menu,1,
+				   "Delete undo history to reclaim memory"),
+			      " (",undo_info(St),")"]),
+    [{?STR(edit_menu,3,"Undo/Redo"),undo_toggle,
+      ?STR(edit_menu,4,"Undo or redo the last command")},
+     {?STR(edit_menu,5,"Redo"),redo,
+      ?STR(edit_menu,6,"Redo the last command that was undone")},
+     {?STR(edit_menu,7,"Undo"),undo,
+      ?STR(edit_menu,8,"Undo the last command")},
      separator,
-     {command_name("Repeat", St),repeat},
-     {command_name("Repeat Args", St),repeat_args},
-     {command_name("Repeat Drag", St),repeat_drag},
+     {command_name(?STR(edit_menu,9,"Repeat"), St),repeat},
+     {command_name(?STR(edit_menu,10,"Repeat Args"), St),repeat_args},
+     {command_name(?STR(edit_menu,11,"Repeat Drag"), St),repeat_drag},
      separator,
      wings_pref:menu(St),
-     {"Plug-in Preferences",{plugin_preferences,[]}},
+     {?STR(edit_menu,12,"Plug-in Preferences"),{plugin_preferences,[]}},
      separator,
-     {"Purge Undo History",purge_undo,UndoInfo}|patches()].
+     {?STR(edit_menu,13,"Purge Undo History"),purge_undo,UndoInfo}|patches()].
 
 undo_info(St) ->
     {Un,Rn} = wings_undo:info(St),
     Undo = case Un of
-	       0 -> "there are no undo states";
-	       1 -> "there is one undo state";
-	       _ -> io_lib:format("there are ~p undo states", [Un])
+	       0 -> ?STR(undo_info,1,"there are no undo states");
+	       1 -> ?STR(undo_info,2,"there is one undo state");
+	       _ -> io_lib:format(?STR(undo_info,3,"there are ~p undo states"), [Un])
 	   end,
     case Rn of
 	0 -> Undo;
-	1 -> [Undo|"; one operation can be redone"];
-	_ -> [Undo|io_lib:format("; ~p operations can be redone", [Rn])]
+	1 -> [Undo|?STR(undo_info,4,"; one operation can be redone")];
+	_ -> [Undo|io_lib:format(?STR(undo_info,5,"; ~p operations can be redone"), [Rn])]
     end.
 
 tools_menu(_) ->
-    Dirs = [{"All",all},
-	    {"X",x},
-	    {"Y",y},
-	    {"Z",z},
-	    {"Radial X",radial_x},
-	    {"Radial Y",radial_y},
-	    {"Radial Z",radial_z}],
-    [{"Align",{align,Dirs}},
-     {"Center",{center,Dirs}},
+    Dirs = [{wings_s:dir(all),all},
+	    {wings_s:dir(x),x},
+	    {wings_s:dir(y),y},
+	    {wings_s:dir(z),z},
+	    {wings_s:dir(radial_x),radial_x},
+	    {wings_s:dir(radial_y),radial_y},
+	    {wings_s:dir(radial_z),radial_z}],
+    [{?STR(tools_menu,8,"Align"),{align,Dirs}},
+     {?STR(tools_menu,9,"Center"),{center,Dirs}},
      separator,
-     {"Save Bounding Box",save_bb},
-     {"Scale to Saved BB",{scale_to_bb,Dirs}},
-     {"Scale to Saved BB Proportionally",{scale_to_bb_prop,Dirs}},
-     {"Move to Saved BB",{move_to_bb,wings_menu_util:all_xyz()}},
+     {?STR(tools_menu,10,"Save Bounding Box"),save_bb},
+     {?STR(tools_menu,11,"Scale to Saved BB"),{scale_to_bb,Dirs}},
+     {?STR(tools_menu,12,"Scale to Saved BB Proportionally"),{scale_to_bb_prop,Dirs}},
+     {?STR(tools_menu,13,"Move to Saved BB"),{move_to_bb,wings_menu_util:all_xyz()}},
      separator,
-     {"Set Default Axis",set_default_axis,
-      "Define and store axis (with ref. point) for later use with any "
-      "\"Default Axis\" command (e.g. Scale|Default Axis)"},
+     {?STR(tools_menu,14,"Set Default Axis"),set_default_axis,
+      ?STR(tools_menu,15,"Define and store axis (with ref. point) for later use with any ")++
+      ?STR(tools_menu,16,"\"Default Axis\" command (e.g. Scale|Default Axis)")},
      separator,
-     {"Virtual Mirror",
+     {?STR(tools_menu,17,"Virtual Mirror"),
       {virtual_mirror,
-       [{"Create",create,
-	 "Given a face selection, set up a virtual mirror"},
-	{"Break",break,
-	 "Remove virtual mirrors for all objects"},
-	{"Freeze",freeze,
-	 "Create real geometry from the virtual mirrors"}]}},
+       [{?STR(tools_menu,18,"Create"),create,
+	 ?STR(tools_menu,19,"Given a face selection, set up a virtual mirror")},
+	{?STR(tools_menu,20,"Break"),break,
+	 ?STR(tools_menu,21,"Remove virtual mirrors for all objects")},
+	{?STR(tools_menu,22,"Freeze"),freeze,
+	 ?STR(tools_menu,23,"Create real geometry from the virtual mirrors")}]}},
      separator,
-     {"Screenshot", screenshot, 
-      "Grab an image of the window (export it from the outliner)"}].
+     {?STR(tools_menu,24,"Screenshot"), screenshot, 
+      ?STR(tools_menu,25,"Grab an image of the window (export it from the outliner)")}].
 
 window_menu(_) ->
     Name = case wings_wm:this() of
 	       {_,geom} ->
-		   "Geometry Graph";
+		   ?STR(window_menu,1,"Geometry Graph");
 	       {_,{geom,N}} ->
-		   "Geometry Graph #" ++ integer_to_list(N)
+		   ?STR(window_menu,2,"Geometry Graph #") ++ integer_to_list(N)
 	   end,
-    [{"Outliner",outliner,
-      "Open the outliner window (showing materials and objects)"},
+    [{?STR(window_menu,3,"Outliner"),outliner,
+      ?STR(window_menu,4,"Open the outliner window (showing materials and objects)")},
      {Name,object,
-      "Open a Geometry Graph window (showing objects)"},
-     {"Palette", palette, "Open the color palette window"},
+      ?STR(window_menu,5,"Open a Geometry Graph window (showing objects)")},
+     {?STR(window_menu,6,"Palette"), palette,?STR(window_menu,7,"Open the color palette window")},
      separator,
-     {"New Geometry Window",geom_viewer, "Open a new Geometry window"},
-     {"Console",console,"Open a console window for information messages"},
+     {?STR(window_menu,8,"New Geometry Window"),geom_viewer, ?STR(window_menu,9,"Open a new Geometry window")},
+     {?STR(window_menu,10,"Console"),console,?STR(window_menu,11,"Open a console window for information messages")},
      separator,
-     {"UV Editor Window",uv_editor_window,
-      "Open a UV Editor window for each selected object"}].
+     {?STR(window_menu,12,"UV Editor Window"),uv_editor_window,
+      ?STR(window_menu,13,"Open a UV Editor window for each selected object")}].
 
 patches() ->
     case wings_start:get_patches() of
 	none -> [];
 	{enabled,Desc} ->
-	    [separator,{"Use "++Desc,disable_patches,[crossmark]}];
+	    [separator,{?STR(patches,1,"Use ")++Desc,disable_patches,[crossmark]}];
 	{disabled,Desc} ->
-	    [separator,{"Use "++Desc,enable_patches}]
+	    [separator,{?STR(patches,1,"Use ")++Desc,enable_patches}]
     end.
 
 set_temp_sel(#st{sh=Sh,selmode=Mode}, St) ->
@@ -773,21 +778,21 @@ purgo_undo(St) ->
     This = wings_wm:this(),
     {Un,Rn} = wings_undo:info(St),
     Qs = {vframe,
-	  [{label,"Undo states: " ++ integer_to_list(Un)},
-	   {label,"Redo states: " ++ integer_to_list(Rn)},
+         [{label,?STR(purgo_undo,1,"Undo states: ") ++ integer_to_list(Un)},
+          {label,?STR(purgo_undo,2,"Redo states: ")  ++ integer_to_list(Rn)},
 	   separator|
 	   if
 	       Un+Rn =:= 0 ->
-		   [{label,"Nothing to remove"},
+		   [{label,?STR(purgo_undo,3,"Nothing to remove")},
 		    {hframe,[{button,ok}]}];
 	       true ->
-		   [{label,"Remove all states (NOT undoable)?"},
-		    {hframe,[{button,"Yes",
+		   [{label,?STR(purgo_undo,4,"Remove all states (NOT undoable)?")},
+		    {hframe,[{button,wings_s:yes(),
 			      fun(_) ->
 				      Action = {action,{edit,confirmed_purge_undo}},
 				      wings_wm:send(This, Action)
 			      end},
-			     {button,"No",cancel,[cancel]}]}]
+			     {button,wings_s:no(),cancel,[cancel]}]}]
 	   end]},
     wings_ask:dialog("", Qs, fun(_) -> ignore end).
 
@@ -808,47 +813,47 @@ info_1(#st{selmode=vertex,sel=[{_Id,Sel}]}=St) ->
 	0 -> "";
 	1 ->
 	    [V] = gb_sets:to_list(Sel),
-	    measure(io_lib:format("Vertex ~p selected", [V]), St);
+	    measure(io_lib:format(?STR(info_1,1,"Vertex ~p selected"), [V]), St);
 	N when N < 5 ->
 	    Vs = gb_sets:to_list(Sel),
-	    measure(item_list(Vs, "Vertices"), St);
+	    measure(item_list(Vs, ?STR(info_1,2,"Vertices")), St);
 	N ->
-	    io_lib:format("~p vertices selected", [N])
+	    io_lib:format(?STR(info_1,3,"~p vertices selected"), [N])
     end;
 info_1(#st{selmode=edge,sel=[{_,Sel}]}=St) ->
     case gb_sets:size(Sel) of
 	0 -> "";
 	1 ->
 	    [Edge] = gb_sets:to_list(Sel),
-	    measure(io_lib:format("Edge ~p selected", [Edge]), St);
+	    measure(io_lib:format(?STR(info_1,4,"Edge ~p selected"), [Edge]), St);
 	N when N < 5 ->
 	    Edges = gb_sets:to_list(Sel),
-	    item_list(Edges, "Edges");
+	    item_list(Edges, ?STR(info_1,5,"Edges"));
 	N ->
-	    io_lib:format("~p edges selected", [N])
+	    io_lib:format(?STR(info_1,6,"~p edges selected"), [N])
     end;
 info_1(#st{selmode=face,sel=[{_,Sel}]}=St) ->
     case gb_sets:size(Sel) of
 	0 -> "";
 	1 ->
 	    [Face] = gb_sets:to_list(Sel),
-	    measure(io_lib:format("Face ~p selected", [Face]), St);
+	    measure(io_lib:format(?STR(info_1,7,"Face ~p selected"), [Face]), St);
 	N when N < 5 ->
 	    Faces = gb_sets:to_list(Sel),
-	    item_list(Faces, "Faces");
+	    item_list(Faces,?STR(info_1,8,"Faces"));
 	N ->
-	    io_lib:format("~p faces selected", [N])
+	    io_lib:format(?STR(info_1,9,"~p faces selected"), [N])
     end;
 info_1(#st{selmode=Mode,sel=Sel}=St) ->
     On = length(Sel),
     N = foldl(fun({_,S}, A) -> A+gb_sets:size(S) end, 0, Sel),
     Str = case Mode of
 	      vertex ->
-		  io_lib:format("~p vertices selected in ~p objects", [N,On]);
+		  io_lib:format(?STR(info_1,10,"~p vertices selected in ~p objects"), [N,On]);
 	      edge ->
-		  io_lib:format("~p edges selected in ~p objects", [N,On]);
+		  io_lib:format(?STR(info_1,11,"~p edges selected in ~p objects"), [N,On]);
 	      face ->
-		  io_lib:format("~p faces selected in ~p objects", [N,On])
+		  io_lib:format(?STR(info_1,12,"~p faces selected in ~p objects"), [N,On])
 	  end,
     measure(Str, St).
 
@@ -858,7 +863,7 @@ measure(Base, #st{selmode=vertex,sel=[{Id,Vs}],shapes=Shs}) ->
 	    We = gb_trees:get(Id, Shs),
  	    [Va] = gb_sets:to_list(Vs),
 	    {X,Y,Z} = wings_vertex:pos(Va, We),
-	    [Base|io_lib:format(". Position ~s ~s ~s",
+	    [Base|io_lib:format(?STR(measure,1,". Position ~s ~s ~s"),
 				[wings_util:nice_float(X),
 				 wings_util:nice_float(Y),
 				 wings_util:nice_float(Z)])];
@@ -867,7 +872,7 @@ measure(Base, #st{selmode=vertex,sel=[{Id,Vs}],shapes=Shs}) ->
  	    [Va,Vb] = gb_sets:to_list(Vs),
  	    Dist = e3d_vec:dist(wings_vertex:pos(Va, We),
 				wings_vertex:pos(Vb, We)),
-	    [Base|io_lib:format(". Distance ~s",
+	    [Base|io_lib:format(?STR(measure,2,". Distance ~s"),
 				[wings_util:nice_float(Dist)])];
 	_ -> Base
     end;
@@ -881,7 +886,7 @@ measure(Base, #st{selmode=vertex,sel=[{IdA,VsA},{IdB,VsB}],shapes=Shs}) ->
  	    [Vb] = gb_sets:to_list(VsB),
  	    Dist = e3d_vec:dist(wings_vertex:pos(Va, WeA),
 				wings_vertex:pos(Vb, WeB)),
-	    [Base|io_lib:format(". Distance ~s",
+	    [Base|io_lib:format(?STR(measure,2,". Distance ~s"),
 				[wings_util:nice_float(Dist)])]
     end;
 measure(Base, #st{selmode=edge,sel=[{Id,Es}],shapes=Shs}) ->
@@ -894,7 +899,7 @@ measure(Base, #st{selmode=edge,sel=[{Id,Es}],shapes=Shs}) ->
 	    PosB = wings_vertex:pos(Vb, We),
  	    Dist = e3d_vec:dist(PosA, PosB),
 	    {X,Y,Z} = e3d_vec:average([PosA,PosB]),
-	    [Base|io_lib:format(". Midpt ~s ~s ~s. Length ~s",
+	    [Base|io_lib:format(?STR(measure,3,". Midpt ~s ~s ~s. Length ~s"),
 				[wings_util:nice_float(X),
 				 wings_util:nice_float(Y),
 				 wings_util:nice_float(Z),
@@ -908,7 +913,7 @@ measure(Base, #st{selmode=face,sel=[{Id,Fs}],shapes=Shs}) ->
  	    [Face] = gb_sets:to_list(Fs),
 	    {X,Y,Z} = wings_face:center(Face, We),
 	    Mat = wings_material:get(Face, We),
-	    [Base|io_lib:format(". Midpt ~s ~s ~s.\nMaterial ~s",
+	    [Base|io_lib:format(?STR(measure,4,". Midpt ~s ~s ~s.\nMaterial ~s"),
 				[wings_util:nice_float(X),
 				 wings_util:nice_float(Y),
 				 wings_util:nice_float(Z),
@@ -922,7 +927,7 @@ item_list(Items, Desc) ->
 
 item_list([Item|Items], Sep, Desc) ->
     item_list(Items, ", ", [Desc,Sep|integer_to_list(Item)]);
-item_list([], _Sep, Desc) -> [Desc|" selected"].
+item_list([], _Sep, Desc) -> [Desc|?STR(item_list,1," selected")].
 
 shape_info(We) when ?IS_LIGHT(We) ->
     wings_light:info(We);
@@ -930,8 +935,10 @@ shape_info(#we{id=Id,name=Name,fs=Ftab,es=Etab,vp=Vtab,mode=Mode}) ->
     Faces = gb_trees:size(Ftab),
     Edges = gb_trees:size(Etab),
     Vertices = gb_trees:size(Vtab),
-    io_lib:format("Object ~p \"~s\" has ~p polygons, ~p edges, ~p vertices.\n"
-		  "Mode is ~p",
+    io_lib:format(?STR(shape_info,1,
+		       "Object ~p \"~s\" has ~p polygons, "
+		       "~p edges, ~p vertices.\n"
+		       "Mode is ~p"),
 		  [Id,Name,Faces,Edges,Vertices,Mode]).
 
 shape_info(Objs, Shs) ->
@@ -944,8 +951,9 @@ shape_info([{Id,_}|Objs], Shs, On, Vn, En, Fn) ->
     Vertices = gb_trees:size(Vtab),
     shape_info(Objs, Shs, On+1, Vn+Vertices, En+Edges, Fn+Faces);
 shape_info([], _Shs, N, Vertices, Edges, Faces) ->
-    io_lib:format("~p objects, ~p faces, ~p edges, ~p vertices",
-		[N,Faces,Edges,Vertices]).
+    io_lib:format(?STR(shape_info,2,
+		       "~p objects, ~p faces, ~p edges, ~p vertices"),
+		  [N,Faces,Edges,Vertices]).
 
 caption(#st{file=undefined}=St) ->
     Caption = wings(),
@@ -956,7 +964,8 @@ caption(#st{saved=true,file=Name}=St) ->
     sdl_video:wm_setCaption(Caption, Caption),
     St;
 caption(#st{saved=auto,file=Name}=St) ->
-    Caption = wings() ++ " - " ++ filename:basename(Name) ++ "* [auto-saved]",
+    Caption = wings() ++ " - " ++ filename:basename(Name) ++
+	"* [" ++ ?STR(caption,1,"auto-saved") ++ "]",
     sdl_video:wm_setCaption(Caption, Caption),
     St;
 caption(#st{file=Name}=St) ->
@@ -965,19 +974,23 @@ caption(#st{file=Name}=St) ->
     St.
 
 command_name(_Repeat, #st{repeatable=ignore}) ->
-    "(Can't repeat)";
+    "("++cannot_repeat()++")";
 command_name(Repeat, #st{repeatable={_,Cmd}}=St) ->
     CmdStr = wings_util:stringify(Cmd),
     command_name(Repeat, CmdStr, St).
 
 command_name(_Repeat, CmdStr, #st{sel=[]}) ->
-    lists:flatten(["(Can't repeat \"",CmdStr,"\")"]);
+    cannot_repeat(CmdStr);
 command_name(Repeat, CmdStr, #st{selmode=Mode,repeatable=Cmd}) ->
-    S = case repeatable(Mode, Cmd) of
-	    no -> ["(Can't repeat \"",CmdStr,"\")"];
-	    _ ->  [Repeat++" \"",CmdStr,"\""]
-	end,
-    lists:flatten(S).
+    case repeatable(Mode, Cmd) of
+	no -> cannot_repeat(CmdStr);
+	_ ->  lists:flatten([Repeat," ",wings_util:quote(CmdStr)])
+    end.
+
+cannot_repeat(Cmd) ->
+    lists:flatten(["(",cannot_repeat(),wings_util:quote(Cmd),")"]).
+
+cannot_repeat() -> ?STR(cannot_repeat,1,"Can't repeat").
 
 replace_ask(Term, none) -> Term;
 replace_ask({'ASK',_}, AskArgs) -> AskArgs;
@@ -990,12 +1003,14 @@ define_command(?SDL_RELEASED, N, #st{repeatable=Cmd,def=DefCmd0}) ->
     This = wings_wm:this(),
     CmdStr = wings_util:stringify(Cmd),
     Button = case N of
-		 1 -> "L";
-		 2 -> "M"
+		 1 -> wings_s:lmb();
+		 2 -> wings_s:mmb()
 	     end,
-    wings_util:yes_no("Do you want to define \"" ++ CmdStr ++
-		      "\" as a default command ([Ctrl]+[" ++ Button ++
-		      "])?",
+    Q = lists:flatten([?STR(define_command,1,"Do you want to define"),
+		       " ",wings_util:quote(CmdStr), " ",
+		       ?STR(define_command,2,"as a default command"),
+		       " (",wings_s:key(ctrl),"+",Button,")?"]),
+    wings_util:yes_no(Q,
 		      fun() ->
 			      DefCmd = setelement(N, DefCmd0, Cmd),
 			      wings_wm:send(This, {new_default_command,DefCmd}),
@@ -1012,6 +1027,11 @@ use_command(#mousebutton{state=?SDL_RELEASED,button=N}=Ev,
     end;
 use_command(_, _) -> keep.
 
+geom_title(geom) ->
+    ?STR(geom_title,1,"Geometry");
+geom_title({geom,N}) ->
+    ?STR(geom_title,2,"Geometry #") ++ integer_to_list(N).
+
 do_use_command(#mousebutton{x=X,y=Y}, Cmd0, #st{sel=[]}=St0) ->
     case wings_pref:get_value(use_temp_sel) of
 	false ->
@@ -1044,10 +1064,13 @@ crash_handler(redraw, Log, _St) ->
     wings_wm:clear_background(),
     wings_io:ortho_setup(),
     wings_io:text_at(10, 2*?LINE_HEIGHT,
-		     "Internal error - log written to " ++ Log),
+		     ?STR(crash_handler,1,
+			  "Internal error - log written to") ++
+		     " " ++ Log),
     wings_io:text_at(10, 4*?LINE_HEIGHT,
-		     "Click a mouse button to continue working"),
-    wings_wm:message("[L] Continue working", ""),
+		     ?STR(crash_handler,2,
+			  "Click a mouse button to continue working")),
+    wings_util:button_message(?STR(crash_handler,3,"Continue working")),
     keep;
 crash_handler(#mousebutton{}, _, St) ->
     wings_wm:message(""),
@@ -1078,20 +1101,20 @@ handle_drop(DropData, {X0,Y0}, St) ->
 
 handle_drop_1(_, X, Y, #st{sel=[]}) ->
     wings_menu:popup_menu(X, Y, drop,
-			  [{"No Selection",cancel_drop,"Cancel drop operation"}]);
+			  [{ ?STR(handle_drop_1,1,"No Selection"),cancel_drop, ?STR(handle_drop_1,2,"Cancel drop operation")}]);
 handle_drop_1({material,Name}, X, Y, #st{selmode=face}) ->
-    Menu = [{"Assign material to selected faces",menu_cmd(assign_to_sel, Name),
-	     "Assign material \""++Name++"\" only to selected faces"},
-	    {"Assign material to all faces",
+    Menu = [{ ?STR(handle_drop_1,3,"Assign material to selected faces"),menu_cmd(assign_to_sel, Name),
+	      ?STR(handle_drop_1,4,"Assign material \"")++Name++ ?STR(handle_drop_1,5,"\" only to selected faces")},
+	    { ?STR(handle_drop_1,6,"Assign material to all faces"),
 	     menu_cmd(assign_to_body, Name),
-	     "Assign material \""++Name++
-	     "\" to all faces in objects having a selection"}],
+	      ?STR(handle_drop_1,7,"Assign material \"")++Name++
+              ?STR(handle_drop_1,8,"\" to all faces in objects having a selection")}],
     wings_menu:popup_menu(X, Y, drop, Menu);
 handle_drop_1({material,Name}, X, Y, _) ->
-    Menu = [{"Assign material to all faces",
+    Menu = [{ ?STR(handle_drop_1,9,"Assign material to all faces"),
 	     menu_cmd(assign_to_body, Name),
-	     "Assign material \""++Name++
-	     "\" to all faces in objects having a selection"}],
+	      ?STR(handle_drop_1,10,"Assign material \"")++Name++
+	     ?STR(handle_drop_1,11,"\" to all faces in objects having a selection")}],
     wings_menu:popup_menu(X, Y, drop, Menu).
     
 menu_cmd(Cmd, Id) ->
@@ -1240,21 +1263,6 @@ set_geom_props([], _) -> ok.
 initial_properties() ->
     [{display_lists,geom_display_lists}|wings_view:initial_properties()].
 
-%%%
-%%% The toolbar window with buttons.
-%%%
-
--define(BUTTON_WIDTH, 34).
--define(BUTTON_HEIGHT, 28).
-
--record(but,
-	{mode,					%Selection mode.
-	 sh,					%Smart highlighting (true|false).
-	 buttons,				%Buttons to show.
-	 all_buttons,				%All buttons.
-	 restr=none				%Restriction (none|[Mode]).
-	}).
-
 mode_restriction(Modes) ->
     Win = {toolbar,wings_wm:this()},
     wings_wm:send(Win, {mode_restriction,Modes}),
@@ -1275,198 +1283,3 @@ get_mode_restriction() ->
 	none -> [edge,vertex,face,body];
 	{value,Other} -> Other
     end.
-
-create_toolbar({toolbar,Client}=Name, Pos, W) ->
-    ButtonH = ?BUTTON_HEIGHT+6,
-    wings_wm:new(Name, Pos, {W,ButtonH}, init_button()),
-    wings_wm:set_prop(Name, display_lists, wings_wm:get_prop(Client, display_lists)).
-
-init_button() ->
-    {seq,push,get_button_event(#but{mode=face})}.
-
-get_button_event(But) ->
-    {replace,fun(Ev) -> button_event(Ev, But) end}.
-
-button_event(got_focus, _) ->
-    wings_wm:dirty(),
-    keep;
-button_event({window_updated,_}, But) ->
-    get_button_event(button_resized(But));
-button_event(redraw, #but{buttons=undefined}=But0) ->
-    But = button_resized(But0),
-    button_redraw(But),
-    get_button_event(But);
-button_event(redraw, But) ->
-    button_redraw(But),
-    keep;
-button_event(#mousebutton{button=B,x=X,state=?SDL_PRESSED}, But) when B =< 3 ->
-    button_was_hit(X, But),
-    keep;
-button_event(#mousebutton{button=B,x=X,state=?SDL_RELEASED}, But) when B =< 3 ->
-    button_help(X, But),
-    keep;
-button_event(#mousemotion{x=X}, But) ->
-    button_help(X, But),
-    keep;
-button_event({action,_}=Action, _) ->
-    {toolbar,Client} = wings_wm:this(),
-    wings_wm:send(Client, Action);
-button_event({current_state,#st{selmode=Mode,sh=Sh}}, #but{mode=Mode,sh=Sh}) ->
-    keep;
-button_event({current_state,#st{selmode=Mode,sh=Sh}}, But) ->
-    wings_wm:dirty(),
-    get_button_event(But#but{mode=Mode,sh=Sh});
-button_event({mode_restriction,Restr}, #but{restr=Restr}) ->
-    keep;
-button_event({mode_restriction,Restr}, #but{all_buttons=AllButtons}=But) ->
-    Buttons = button_restrict(AllButtons, Restr),
-    wings_wm:dirty(),
-    get_button_event(But#but{buttons=Buttons,restr=Restr});
-button_event(#keyboard{}=Ev, _) ->
-    {toolbar,Client} = wings_wm:this(),
-    wings_wm:send(Client, Ev);
-button_event(_, _) -> keep.
-
-button_resized(#but{restr=Restr}=But) ->
-    {toolbar,Client} = Self = wings_wm:this(),
-    {{X,Y},{W,_}} = wings_wm:win_rect(Client),
-    {_,H} = wings_wm:win_size(),
-    wings_wm:update_window(Self, [{x,X},{y,Y-H},{w,W}]),
-    AllButtons = buttons_place(W),
-    Buttons = button_restrict(AllButtons, Restr),
-    But#but{buttons=Buttons,all_buttons=AllButtons}.
-
-button_redraw(#but{mode=Mode,buttons=Buttons,sh=Sh0}) ->
-    Sh = button_sh_filter(Mode, Sh0),
-    wings_io:ortho_setup(),
-    {W,H} = wings_wm:win_size(),
-    wings_io:border(0, 0, W-1, H-1, ?PANE_COLOR),
-    wings_io:draw_icons(
-      fun() ->
-	      foreach(fun({X,Name}) ->
-			      wings_io:draw_icon(X, 3, button_value(Name, Mode, Sh))
-		      end, Buttons)
-      end),
-    button_redraw_sh(Sh, Buttons).
-
-button_redraw_sh(false, _) -> ok;
-button_redraw_sh(true, Buttons) ->
-    Pos = [X || {X,M} <- Buttons, button_sh_filter(M, true)],
-    case Pos of
-	[] -> ok;
-	[Left|_] ->
-	    gl:pushAttrib(?GL_POLYGON_BIT bor ?GL_LINE_BIT),
-	    gl:polygonMode(?GL_FRONT_AND_BACK, ?GL_LINE),
-	    Right = lists:last(Pos),
-	    gl:color3f(1, 1, 1),
-	    gl:recti(Left-1, 3, Right+?BUTTON_WIDTH, 3+?BUTTON_HEIGHT),
-	    gl:popAttrib()
-    end.
-
-button_sh_filter(_, false) -> false;
-button_sh_filter(vertex, true) ->
-    wings_pref:get_value(vertex_hilite);
-button_sh_filter(edge, true) ->
-    wings_pref:get_value(edge_hilite);
-button_sh_filter(face, true) ->
-    wings_pref:get_value(face_hilite);
-button_sh_filter(_, _) -> false.
-
-buttons_place(W) when W < 325 ->
-    Mid = (W - ?BUTTON_WIDTH) div 2,
-    [{Mid-trunc(1.5*?BUTTON_WIDTH),vertex},
-     {Mid-trunc(0.5*?BUTTON_WIDTH),edge},
-     {Mid+trunc(0.5*?BUTTON_WIDTH),face},
-     {Mid+trunc(1.5*?BUTTON_WIDTH),body}];
-buttons_place(W) ->
-    Mid = (W - ?BUTTON_WIDTH) div 2,
-    Lmarg = 5,
-    Rmarg = 5,
-    [{Lmarg,smooth},{Lmarg+?BUTTON_WIDTH,perspective},
-     {Mid-trunc(1.5*?BUTTON_WIDTH),vertex},
-     {Mid-trunc(0.5*?BUTTON_WIDTH),edge},
-     {Mid+trunc(0.5*?BUTTON_WIDTH),face},
-     {Mid+trunc(1.5*?BUTTON_WIDTH),body},
-     {W-2*?BUTTON_WIDTH-Rmarg,groundplane},
-     {W-?BUTTON_WIDTH-Rmarg,axes}].
-
-button_value(groundplane=Name, _, _) ->
-    button_value_1(Name, show_groundplane, true);
-button_value(axes=Name, _, _) ->
-    button_value_1(Name, show_axes, true);
-button_value(smooth=Name, _, _) ->
-    button_value_1(Name, workmode, false);
-button_value(perspective=Name, _, _) ->
-    button_value_1(Name, orthogonal_view, true);
-button_value(Mode, Mode, false) -> {Mode,down};
-button_value(Name, _, _) -> {Name,up}.
-
-button_value_1(Name, Key, Val) ->
-    {toolbar,Client} = wings_wm:this(),
-    case wings_wm:get_prop(Client, Key) of
-	Val -> {Name,down};
-	_ -> {Name,up}
-    end.
-
-button_was_hit(X, #but{buttons=Buttons}) ->
-    button_was_hit_1(X, Buttons).
-
-button_was_hit_1(X, [{Pos,Name}|_]) when Pos =< X, X < Pos+?BUTTON_WIDTH ->
-    Action = case Name of
-		 groundplane -> {view,show_groundplane};
-		 axes -> {view,show_axes};
-		 flatshade -> {view,flatshade};
-		 smooth -> {view,workmode};
-		 perspective -> {view,orthogonal_view};
-		 Other -> {select,Other}
-	     end,
-    {toolbar,Client} = wings_wm:this(),
-    wings_wm:send(Client, {action,Action});
-button_was_hit_1(X, [_|Is]) ->
-    button_was_hit_1(X, Is);
-button_was_hit_1(_X, []) ->
-    wings_wm:send(geom, {action,{select,deselect}}).
-
-button_help(X, #but{mode=Mode,buttons=Buttons}) ->
-    wings_wm:message(button_help_1(X, Buttons, Mode)).
-
-button_help_1(X, [{Pos,Name}|_], Mode) when Pos =< X, X < Pos+?BUTTON_WIDTH ->
-    button_help_2(Name, Mode);
-button_help_1(X, [_|Is], Mode) ->
-    button_help_1(X, Is, Mode);
-button_help_1(_, [], _) ->
-    "Deselect".
-
-button_help_2(vertex, vertex) -> "Select adjacent vertices";
-button_help_2(vertex, _) -> "Change to vertex selection mode";
-button_help_2(edge, edge) -> "Select adjcacent edges";
-button_help_2(edge, _) -> "Change to edge selection mode";
-button_help_2(face, face) -> "Select adjacent faces";
-button_help_2(face, _) -> "Change to face selection mode";
-button_help_2(body, body) -> "";
-button_help_2(body, _) -> "Change to body selection mode";
-button_help_2(Button, _) -> button_help_3(Button).
-
-button_help_3(groundplane) ->
-    [choose(show_groundplane, true, "Hide", "Show")|" ground plane"];
-button_help_3(axes) ->
-    [choose(show_axes, true, "Hide", "Show")|" axes"];
-button_help_3(perspective) ->
-    ["Change to ",choose(orthogonal_view, false,
-			 "orthogonal", "perspective")|" view"];
-button_help_3(smooth) ->
-    ["Show objects with ",choose(workmode, true,
-				 "smooth", "flat")|" shading"].
-
-button_restrict(Buttons, none) -> Buttons;
-button_restrict(Buttons0, Restr) ->
-    Buttons1 = sofs:from_external(Buttons0, [{atom,atom}]),
-    Buttons = sofs:restriction(2, Buttons1, sofs:set(Restr)),
-    sofs:to_external(Buttons).
-
-choose(Key, Val, First, Second) ->
-    {toolbar,Client} = wings_wm:this(),
-    case wings_wm:get_prop(Client, Key) of
-	Val -> First;
-	_ -> Second
-    end.
