@@ -3,12 +3,12 @@
 %%
 %%     This module contains most of the command for entire Wings objects.
 %%
-%%  Copyright (c) 2001-2003 Bjorn Gustavsson
+%%  Copyright (c) 2001-2004 Bjorn Gustavsson
 %%
 %%  See the file "license.terms" for information on usage and redistribution
 %%  of this file, and for a DISCLAIMER OF ALL WARRANTIES.
 %%
-%%     $Id: wings_body.erl,v 1.66 2004/05/14 08:23:46 raimo_niskanen Exp $
+%%     $Id: wings_body.erl,v 1.72 2004/11/13 04:39:25 bjorng Exp $
 %%
 
 -module(wings_body).
@@ -20,61 +20,101 @@
 
 menu(X, Y, St) ->
     Dir = wings_menu_util:directions(St),
-    Menu = [{basic,{"Object operations",ignore}},
+    FlipStr = ?STR(menu,5,"Flip the object around") ++ " ",
+
+    Menu = [{basic,{?STR(menu,1,"Object operations"),ignore}},
 	    {basic,separator},
-	    {"Move",{move,Dir}},
+	    {?STR(menu,2,"Move"),{move,Dir}},
 	    wings_menu_util:rotate(St),
 	    wings_menu_util:scale(St),
 	    separator,
-	    {"Flip",
-	     {flip,[{"X",x,"Flip the object around the X axis"},
-		    {"Y",y,"Flip the object around the Y axis"},
-		    {"Z",z,"Flip the object around the Z axis"}]}},
+	    {?STR(menu,3,"Flip"),
+	     {flip,[{wings_s:dir(x),x,FlipStr ++ wings_s:dir_axis(x)},
+		    {wings_s:dir(y),y,FlipStr ++ wings_s:dir_axis(y)},
+		    {wings_s:dir(z),z,FlipStr ++ wings_s:dir_axis(z)}]}},
 	    separator,
-	    {"Invert",invert,
-	     "Flip all normals, turning the object inside out"},
+	    {?STR(menu,10,"Invert"),invert,
+	     ?STR(menu,11,"Flip all normals, turning the object inside out")},
 	    separator,
-	    {"Tighten",tighten,
-	     "Move vertices towards average midpoint"},
-	    {"Smooth",smooth,
-	     "Subdivide all faces to give the object a smoother apperance"},
-	    {"Combine",combine,
-	     "Combine multiple objects into a single object"},
-	    {"Separate",separate,
-	     "Separate a combined objects into its components"},
+	    {?STR(menu,12,"Tighten"),tighten,
+	     ?STR(menu,13,"Move vertices towards average midpoint")},
+	    {?STR(menu,14,"Smooth"),smooth,
+	     ?STR(menu,15,"Subdivide all faces to give the object a smoother apperance")},
+	    {?STR(menu,16,"Combine"),combine,
+	     ?STR(menu,17,"Combine multiple objects into a single object")},
+	    {?STR(menu,18,"Separate"),separate,
+	     ?STR(menu,19,"Separate a combined objects into its components")},
 	    separator,
-	    {"Weld",weld,"Merge pair of faces that are nearly coincident",
+	    {?STR(menu,20,"Weld"),weld,?STR(menu,21,"Merge pair of faces that are nearly coincident"),
 	     [option]},
 	    separator,
-	    {"Cleanup",cleanup,"Remove various defects",[option]},
-	    {"Auto-Smooth",auto_smooth,
-	     "Set edges hard or soft depending on the angle between faces",
+	    {?STR(menu,22,"Cleanup"),cleanup,?STR(menu,23,"Remove various defects"),[option]},
+	    {?STR(menu,24,"Auto-Smooth"),auto_smooth,
+	     ?STR(menu,25,"Set edges hard or soft depending on the angle between faces"),
 	     [option]},
 	    separator,
-	    {"Duplicate",{duplicate,Dir}},
-	    {"Delete",delete,"Delete the selected objects"},
-	    {"Rename...",rename,"Rename selected objects"},
-	    {"To Area Light",to_arealight,
-	     "Convert selected objects into area lights"},
-	    {"From Area Light",from_arealight,
-	     "Convert selected area lights into objects"},
-	    separator,
-	    {"Mode",{mode,
-		     [{"Vertex Color",vertex_color,
-		       "Vertex colors will be shown"},
-		      {"Material",material,
-		       "Materials will be shown"}]},
-	     "Change object mode to vertex colors or material"},
-	    separator,
-	    {"Materials to Colors",materials_to_colors,
-	     "Convert materials to vertex colors"},
-	    {"Colors to Materials",colors_to_materials,
-	     "Convert vertex colors to materials"},
-	    separator,
-	    {"Vertex Color",vertex_color,
-	     "Apply vertex colors to selected objects"}],
+	    {?STR(menu,26,"Duplicate"),{duplicate,Dir}},
+	    {?STR(menu,27,"Delete"),delete,?STR(menu,28,"Delete the selected objects")},
+	    {?STR(menu,29,"Rename..."),rename,?STR(menu,30,"Rename selected objects")}|mode_dependent(St)],
     wings_menu:popup_menu(X, Y, body, Menu).
 
+mode_dependent(St) ->
+    SelObj = wings_sel:fold(fun(_, We, A) -> [We|A] end, [], St),
+    Area = foldl(fun(#we{has_shape=Shape}=We, A) ->
+			 Type = if 
+				    ?IS_ANY_LIGHT(We), Shape -> arealight;
+				    true -> object
+				end,
+			 case A of
+			     none -> Type;
+			     Type -> Type;
+			     _ -> mixed
+			 end
+		 end, none, SelObj),
+    ObjMode = foldl(fun(#we{mode=M}, none) -> M;
+		       (#we{mode=M}, M) -> M;
+		       (_, _) -> mixed
+		    end, none, SelObj),
+    Tail0 = vertex_color_item(Area),
+    Tail1 = mat_col_conv(ObjMode, Area, Tail0),
+    Tail2 = arealight_conv(Area, Tail1),
+    case mode_conv(ObjMode, Area, Tail2) of
+	[] -> [];
+	Tail -> [separator|Tail]
+    end.
+
+vertex_color_item(object) ->
+    [{?STR(vertex_color_item,1,"Vertex Color"),vertex_color,
+      ?STR(vertex_color_item,2,"Apply vertex colors to selected objects "
+	   "(first changing mode to vertex color mode if needed)")}];
+vertex_color_item(_) -> [].
+    
+mat_col_conv(vertex, object, T) ->
+    [{?STR(mat_col_conv,1,"Colors to Materials"),colors_to_materials,
+      ?STR(mat_col_conv,2,"Convert vertex colors to materials")}|T];
+mat_col_conv(material, object, T) ->
+    [{?STR(mat_col_conv,3,"Materials to Colors"),materials_to_colors,
+      ?STR(mat_col_conv,4,"Convert materials to vertex colors")}|T];
+mat_col_conv(_, _, T) -> T.
+
+arealight_conv(arealight, T) ->
+    [{?STR(arealight_conv,1,"Area Light to Object"),from_arealight,
+      ?STR(arealight_conv,2,"Convert selected area lights to objects")}|T];
+arealight_conv(object, T) ->
+    [{?STR(arealight_conv,3,"Object To Area Light"),to_arealight,
+      ?STR(arealight_conv,4,"Convert selected objects to area lights")}|T];
+arealight_conv(mixed, T) -> T.
+
+mode_conv(material, object, T) ->
+    [{?STR(mode_conv,1,"Vertex Color Mode"),vertex_color_mode,
+      ?STR(mode_conv,2,"Change object mode to vertex color mode "
+	   "(materials will be kept, but not shown)")}|T];
+mode_conv(vertex, object, T) ->
+    [{?STR(mode_conv,3,"Material Mode"),material_mode,
+      ?STR(mode_conv,4,"Change object mode to material mode "
+	   "(vertex colors will be kept, but not shown)")}|T];
+mode_conv(_, _, T) -> T.
+
 command({move,Type}, St) ->
     wings_move:setup(Type, St);
 command({rotate,Type}, St) ->
@@ -129,8 +169,10 @@ command(materials_to_colors, St) ->
     {save_state,materials_to_colors(St)};
 command(colors_to_materials, St) ->
     {save_state,colors_to_materials(St)};
-command({mode,Mode}, St) ->
-    {save_state,set_mode(Mode, St)};
+command(material_mode, St) ->
+    {save_state,set_mode(material, St)};
+command(vertex_color_mode, St) ->
+    {save_state,set_mode(vertex, St)};
 command({weld,Ask}, St) ->
     weld(Ask, St);
 command(vertex_color, St) ->
@@ -151,11 +193,11 @@ convert_selection(#st{sel=Sel0}=St) ->
 %%%
 
 cleanup(Ask, _) when is_atom(Ask) ->
-    Qs = [{"Short Edges",true,[{key,short_edges}]},
+    Qs = [{?STR(cleanup,1,"Short Edges"),true,[{key,short_edges}]},
 	  {hframe,
-	   [{label,"Length Tolerance"},{text,1.0E-3,[{range,{1.0E-5,10.0}}]}]},
-	  {"Isolated Vertices",true,[{key,isolated_vs}]}],
-    wings_ask:dialog(Ask, "Cleanup",
+	   [{label,?STR(cleanup,2,"Length Tolerance")},{text,1.0E-3,[{range,{1.0E-5,10.0}}]}]},
+	  {?STR(cleanup,3,"Isolated Vertices"),true,[{key,isolated_vs}]}],
+	   wings_ask:dialog(Ask, ?STR(cleanup,4,"Cleanup"),
 		     [{vframe,Qs}],
 		     fun(Res) -> {body,{cleanup,Res}} end);
 cleanup(Opts, St0) ->
@@ -180,8 +222,15 @@ cleanup_1([_|Opts], We) ->
 cleanup_1([], We) -> We.
 
 clean_isolated_vertices(We) ->
-    Isolated = wings_vertex:isolated(We),
-    wings_vertex:dissolve_isolated(Isolated, We).
+    case wings_vertex:isolated(We) of
+	[] -> We;
+	[_]=Isolated ->
+	    io:put_chars(?STR(clean_isolated_vertices,1,"Removed 1 isolated vertex\n")),
+	    wings_vertex:dissolve_isolated(Isolated, We);
+	Isolated ->
+	    io:format(?STR(clean_isolated_vertices,2,"Removed ~p isolated vertices\n"), [length(Isolated)]),
+	    wings_vertex:dissolve_isolated(Isolated, We)
+    end.
 		  
 clean_short_edges(Tolerance, #we{es=Etab,vp=Vtab}=We) ->
     Short = foldl(
@@ -211,7 +260,7 @@ cleanup_rep_1([F|Fs], We0) ->
 	none ->
 	    cleanup_rep_1(Fs, We0);
 	V ->
-	    io:format("Repeated vertex ~p in face ~p\n", [V,F]),
+	    io:format(?STR(cleanup_rep_1,1,"Repeated vertex ~p in face ~p\n"), [V,F]),
 	    We = cleanup_rep_2(F, V, We0),
 	    NewFaces = gb_sets:to_list(wings_we:new_items(face, We0, We)),
 	    cleanup_rep_1(NewFaces++Fs, We)
@@ -298,7 +347,7 @@ cleanup_waists_1([V|Vs], [{V,AllEs}|VsEs], #we{es=Etab0,vp=Vtab0,vc=Vct0}=We0) -
 	    Vtab = gb_trees:insert(NewV, gb_trees:get(V, Vtab0), Vtab0),
 	    Vct = gb_trees:insert(NewV, AnEdge, Vct0),
 	    We = We1#we{es=Etab,vp=Vtab,vc=Vct},
-	    io:format("Removed waist vertex: ~p\n", [V]),
+	    io:format(?STR(cleanup_waists_1,1,"Removed waist vertex: ~p\n"), [V]),
 
 	    %% Re-process the newly added vertex. (Some of the
 	    %% edges may not be reachable from the incident of
@@ -326,8 +375,8 @@ cleanup_2edged_faces(#we{fs=Ftab}=We) ->
 delete_2edged_faces_1([Face|Faces], We0) ->
     case wings_face:delete_if_bad(Face, We0) of
 	bad_edge ->
-	    wings_util:error("Face " ++ integer_to_list(Face) ++
-			     " has only one edge.");
+	    wings_util:error(?STR(delete_2edged_faces_1,1,"Face") ++ integer_to_list(Face) ++
+			     ?STR(delete_2edged_faces_1,2,"has only one edge"));
 	We -> delete_2edged_faces_1(Faces, We)
     end;
 delete_2edged_faces_1([], We) -> We.
@@ -344,9 +393,8 @@ invert_normals(St) ->
 %%%
 
 duplicate(Dir, #st{onext=Oid0}=St0) ->
-    Copy = "copy",
     St1 = wings_sel:fold(fun(_, We, St) ->
-				 wings_shape:insert(We, Copy, St)
+				 wings_shape:insert(We, copy, St)
 			 end, St0, St0),
     %% Select the duplicate items, not the original items.
     Zero = gb_sets:singleton(0),
@@ -362,10 +410,9 @@ duplicate(Dir, #st{onext=Oid0}=St0) ->
 %%%
 
 duplicate_object(Objects, #st{shapes=Shs}=St) ->
-    Copy = "copy",
     foldl(fun(Id, S) ->
 		  We = gb_trees:get(Id, Shs),
-		  wings_shape:insert(We, Copy, S)
+		  wings_shape:insert(We, copy, S)
 	  end, St, Objects).
 
 %%%
@@ -455,8 +502,8 @@ unify_modes([#we{mode=Mode}|Wes]) ->
 unify_modes([#we{mode=Mode}|Wes], Mode) ->
     unify_modes(Wes, Mode);
 unify_modes([_|_], _) ->
-    wings_util:error("Objects with vertex colors cannot be combined "
-		     "with objects with materials.");
+    wings_util:error(?STR(unify_modes,1,"Objects with vertex colors cannot be combined " 
+			  "with objects with materials."));
 unify_modes([], Mode) -> Mode.
 		    
 %%%
@@ -464,14 +511,13 @@ unify_modes([], Mode) -> Mode.
 %%%
 
 separate(St) ->
-    Sep = "sep",
     wings_sel:fold(
       fun(_, #we{id=Id}=We0, St0) ->
 	      case wings_we:separate(We0) of
 		  [_] -> St0;
 		  [We|Wes] ->
 		      St1 = foldl(fun(W, A) ->
-					  wings_shape:insert(W, Sep, A)
+					  wings_shape:insert(W, sep, A)
 				  end, St0, Wes),
 		      wings_shape:replace(Id, We, St1)
 	      end
@@ -485,8 +531,8 @@ auto_smooth(St) ->
     do_auto_smooth(60, St).
 
 auto_smooth(Ask, _) when is_atom(Ask) ->
-    wings_ask:ask(Ask, "Auto Smooth Parameters",
-		  [{"Crease Angle",60,[{range,{0,180}}]}],
+    wings_ask:ask(Ask,?STR(auto_smooth,1,"Auto Smooth Parameters"),
+		  [{?STR(auto_smooth,2,"Crease Angle"),60,[{range,{0,180}}]}],
 		  fun(Res) -> {body,{auto_smooth,Res}} end);
 auto_smooth([Angle], St) ->
     {save_state,do_auto_smooth(Angle, St)}.
@@ -533,7 +579,7 @@ rename(Objects, #st{shapes=Shs}=St) ->
 
 rename_1(Wes, St) ->
     Qs = rename_qs(Wes),
-    wings_ask:dialog("Rename", Qs,
+	wings_ask:dialog(?STR(rename_1,1,"Rename"), Qs,
 		     fun(NewNames) ->
 			     rename_1(NewNames, Wes, St)
 		     end).
@@ -648,7 +694,7 @@ colors_to_materials_2([], We, FaceMat, St) ->
     {wings_material:assign_materials(FaceMat, We),St}.
 
 color_material({R,G,B}=Color, #st{mat=Mat0}=St0) ->
-    Name0 = "color_" ++ fmt_int(R) ++ "_" ++ fmt_int(G) ++ "_" ++ fmt_int(B),
+    Name0 = ?STR(color_material,1,"color_")++ fmt_int(R) ++ "_" ++ fmt_int(G) ++ "_" ++ fmt_int(B),
     Name = list_to_atom(Name0),
     case gb_trees:is_defined(Name, Mat0) of
 	true -> {Name,St0};
@@ -673,8 +719,8 @@ fmt_int(N, L) -> fmt_int(N+1, [$0|L]).
 
 weld(Ask, _) when is_atom(Ask) ->
     Qs = [{hframe,
-	   [{label,"Distance Tolerance"},{text,1.0E-3,[{range,{1.0E-5,10.0}}]}]}],
-    wings_ask:dialog(Ask, "Weld", Qs,
+	   [{label,?STR(weld,1,"Distance Tolerance")},{text,1.0E-3,[{range,{1.0E-5,10.0}}]}]}],
+	     wings_ask:dialog(Ask, ?STR(weld,2,"Weld"), Qs,
 		     fun(Res) -> {body,{weld,Res}} end);
 weld([Tolerance], St0) ->
     St1 = combine(St0),
@@ -694,7 +740,7 @@ weld_1(Tol, #we{id=Id,fs=Fs0}=We0, Acc) ->
     Part = sofs:to_external(Part1),
     case weld_2(Part, Tol, We0) of
 	We0 ->
-	    wings_util:error("Found no faces to weld.");
+	    wings_util:error(?STR(weld_1,1,"Found no faces to weld."));
 	We ->
 	    {We,[{Id,weld_selection(lists:append(Part), We0, We)}|Acc]}
     end.
