@@ -8,7 +8,7 @@
 %%  See the file "license.terms" for information on usage and redistribution
 %%  of this file, and for a DISCLAIMER OF ALL WARRANTIES.
 %%
-%%     $Id: wings_camera.erl,v 1.104 2004/05/17 17:51:10 bjorng Exp $
+%%     $Id: wings_camera.erl,v 1.110 2004/10/16 12:22:02 bjorng Exp $
 %%
 
 -module(wings_camera).
@@ -40,6 +40,7 @@ init() ->
     wings_pref:set_default(camera_mode, mirai),
     wings_pref:set_default(num_buttons, 3),
     wings_pref:set_default(pan_speed, 25),
+    wings_pref:set_default(inverted_wheel_zoom, false),
     case {wings_pref:get_value(num_buttons),wings_pref:get_value(camera_mode)} of
 	{3,_} -> ok;
 	{_,nendo} -> ok;
@@ -51,25 +52,30 @@ prefs() ->
     ZoomFlag0 = wings_pref:get_value(wheel_zooms, true),
     ZoomFactor0 = wings_pref:get_value(wheel_zoom_factor, ?ZOOM_FACTOR),
     PanSpeed0 = wings_pref:get_value(pan_speed),
+    InvertZW = wings_pref:get_value(inverted_wheel_zoom),
+    Hook = {hook,fun (is_disabled, {_Var,_I,Sto}) ->
+			 not gb_trees:get(wheel_zooms, Sto);
+		     (_, _) -> void
+		 end},
     {vframe,
-     [{vframe,[mouse_buttons()],[{title,"Mouse Buttons"}]},
-      {vframe,[camera_modes()],[{title,"Camera Mode"}]},
+     [{vframe,[mouse_buttons()],[{title,?STR(prefs,1,"Mouse Buttons")}]},
+      {vframe,[camera_modes()],[{title,?STR(prefs,2,"Camera Mode")}]},
       {vframe,
        [{hframe,[{slider,{text,PanSpeed0,[{key,pan_speed},{range,{1,100}}]}}]}],
-       [{title,"Pan Speed"}]},
+       [{title,?STR(prefs,3,"Pan Speed")}]},
       {vframe,
-       [{"Wheel Zooms",ZoomFlag0,[{key,wheel_zooms}]},
+       [{?STR(prefs,4,"Wheel Zooms"),ZoomFlag0,[{key,wheel_zooms}]},
+	{vradio,[{?STR(prefs,5,"Forwards Zooms In"),false},
+		 {?STR(prefs,6,"Forwards Zooms Out"),true}],
+	 InvertZW,
+	 [{key,inverted_wheel_zoom},Hook]},
 	{hframe,
-	 [{label,"Zoom Factor"},
+	 [{label,?STR(prefs,7,"Zoom Factor"),[Hook]},
 	  {text,ZoomFactor0,
 	   [{key,wheel_zoom_factor},
 	    {range,{1,50}},
-	    {hook,fun (is_disabled, {_Var,_I,Sto}) ->
-			  not gb_trees:get(wheel_zooms, Sto);
-		      (_, _) -> void
-		  end}]},
-	  {label,"%"}],
-	 [{title,"Scroll Wheel"}]}]} ]}.
+	    Hook]},
+	  {label,"%",[Hook]}]} ],[{title,?STR(prefs,9,"Scroll Wheel")}] } ]}.
 
 mouse_buttons() ->
     {menu,[{desc(1),1,[{info,info(1)}]},
@@ -104,35 +110,28 @@ camera_modes() ->
 		(_, _) -> void
 	    end}]}.
 
-desc(1) -> "One";
-desc(2) -> "Two";
-desc(3) -> "Three";
-desc(blender) -> "Blender";
-desc(nendo) -> "Nendo";
-desc(mirai) -> "Mirai";
-desc(tds) -> "3ds max";
-desc(maya) -> "Maya";
-desc(mb) -> "Motionbuilder".
+desc(1) -> ?STR(desc,1,"One");
+desc(2) -> ?STR(desc,2,"Two");
+desc(3) -> ?STR(desc,3,"Three");
+desc(Other) -> wings_s:camera_mode(Other).
 
 info(1) ->
-    "Note: Only the Nendo camera mode can be used with "
-	"a one-button mouse";
+    ?STR(info,1,"Note: Only the Nendo camera mode can be used with a one-button mouse");
 info(2) ->
-    "Note: Only the Nendo and Blender camera modes can be used with "
-	"a two-button mouse";
+    ?STR(info,2,"Note: Only the Nendo and Blender camera modes can be used with a two-button mouse");
 info(3) -> "";
 info(nendo) -> "";
 info(blender) ->
-    ["Note: The ",desc(blender),
-     " camera mode requires at least 2 mouse buttons"];
+    [?STR(info,3,"Note: The "),desc(blender),
+     ?STR(info,4," camera mode requires at least 2 mouse buttons")];
 info(Mode) ->
-    ["Note: The ",desc(Mode)," camera mode requires 3 mouse buttons"].
+    [?STR(info,5,"Note: The "),desc(Mode),?STR(info,6," camera mode requires 3 mouse buttons")].
 
 help() ->
     case wings_pref:get_value(camera_mode) of
 	blender -> blender_help();
-	nendo -> button_format([], "Start camera");
-	mirai -> button_format([], "Start camera");
+	nendo -> button_format([], ?STR(help,1,"Start camera"));
+	mirai -> button_format([], ?STR(help,1,"Start camera"));
 	tds -> tds_help();
 	maya -> maya_help();
 	mb -> mb_help()
@@ -153,19 +152,23 @@ mod_format(0, But, Msg) ->
 mod_format(Mod, But, Msg) ->
     M0 = [But,?CSEP,Msg],
     M1 = if
-	     (Mod band ?SHIFT_BITS) =/= 0 -> ["[Shift]+"|M0];
+	     (Mod band ?SHIFT_BITS) =/= 0 ->
+		 [wings_s:key(shift),$+|M0];
 	     true -> M0
 	 end,
     M2 = if
-	     (Mod band ?ALT_BITS) =/= 0 -> ["[Alt]+"|M1];
+	     (Mod band ?ALT_BITS) =/= 0 ->
+		 [wings_s:key(alt),$+|M1];
 	     true -> M1
 	 end,
     M3 = if
-	     (Mod band ?CTRL_BITS) =/= 0 -> ["[Ctrl]+"|M2];
+	     (Mod band ?CTRL_BITS) =/= 0 ->
+		 [wings_s:key(ctrl),$+|M2];
 	     true -> M2
 	 end,
     if
-	(Mod band ?META_BITS) =/= 0 -> ["[Command]+"|M3];
+	(Mod band ?META_BITS) =/= 0 ->
+	    [wings_s:key(command),$+|M3];
 	true -> M3
     end.
 
@@ -213,32 +216,32 @@ button_format(LmbMsg, MmbMsg, RmbMsg) ->
 drop_last(S) ->
     reverse(tl(reverse(S))).
 
-lmb_name() -> "L:".
+lmb_name() -> [wings_s:lmb()|":"].
 
 mmb_name() ->
     mmb_name(wings_pref:get_value(num_buttons)).
 
-mmb_name(3) -> "M:";
+mmb_name(3) -> [wings_s:mmb()|":"];
 mmb_name(2) ->
     case wings_pref:get_value(camera_mode) of
-	blender -> ["[Alt]+"|lmb_name()];
-	nendo -> ["[Ctrl]+"|rmb_name(2)]
+	blender -> [wings_s:key(alt),$+|lmb_name()];
+	nendo -> [wings_s:key(ctrl),$+|rmb_name(2)]
     end;
-mmb_name(1) -> ["[Alt]+"|lmb_name()].
+mmb_name(1) -> [wings_s:key(alt),$+|lmb_name()].
 
 rmb_name() ->
     rmb_name(wings_pref:get_value(num_buttons)).
 
-rmb_name(1) -> "[Ctrl]+L:";
-rmb_name(_) -> "R:".
+rmb_name(1) -> [wings_s:key(ctrl),$+,lmb_name()];
+rmb_name(_) -> [wings_s:rmb()|":"].
 
 rmb_format(Message) ->
     RmbMod = mod_name(free_rmb_modifier()),
-    ["[",RmbMod,"]+",rmb_name(),?CSEP|Message].
+    [wings_s:key(RmbMod),$+,rmb_name(),?CSEP|Message].
 
-mod_name(?ALT_BITS) -> "Alt";
-mod_name(?CTRL_BITS) -> "Ctrl";
-mod_name(?META_BITS) -> "Command".
+mod_name(?ALT_BITS) -> wings_s:alt();
+mod_name(?CTRL_BITS) -> wings_s:ctrl();
+mod_name(?META_BITS) -> wings_s:command().
 
 join_msg([M0,M1|T]) ->
     join_msg(join_msg(M0, M1), join_msg(T));
@@ -312,11 +315,11 @@ get_blender_event(Camera, Redraw) ->
     {replace,fun(Ev) -> blender_event(Ev, Camera, Redraw) end}.
 	    
 blender_help() ->
-    TrackDolly = [{?SHIFT_BITS,2,"Track"},
-		  {?CTRL_BITS,2,"Dolly"}],
+    TrackDolly = [{?SHIFT_BITS,2,?STR(mode_help,1,"Track")},
+		  {?CTRL_BITS,2,?STR(mode_help,2,"Dolly")}],
     case allow_rotation() of
 	false -> format(TrackDolly);
-	true -> format([{0,2,"Tumble"}|TrackDolly])
+	true -> format([{0,2,?STR(mode_help,3,"Tumble")}|TrackDolly])
     end.
 
 %%%
@@ -388,17 +391,21 @@ get_nendo_event(Camera, Redraw, MouseRotates) ->
     {replace,fun(Ev) -> nendo_event(Ev, Camera, Redraw, MouseRotates) end}.
 
 nendo_message(true) ->
-    Help = join_msg([button_format("Accept", "Drag to Dolly"),
-		     "Move mouse to tumble",
-		     ["[Q]",?CSEP,"Move mouse to track"]]),
+    Help = join_msg([button_format(wings_s:accept(),
+				   ?STR(message,2,"Drag to Dolly")),
+		     ?STR(message,3,"Move mouse to tumble"),
+		     [?STR(message,4,"[Q]"),?CSEP,
+		      ?STR(message,5,"Move mouse to track")]]),
     message(Help);
 nendo_message(false) ->
     QText = case allow_rotation() of
 		false -> [];
-		true -> ["[Q]",?CSEP,"Move mouse to tumble"]
+		true -> [?STR(message,4,"[Q]"),?CSEP,
+			 ?STR(message,3,"Move mouse to tumble")]
 	    end,
-    Help = join_msg([button_format("Accept", "Drag to Dolly"),
-		     "Move mouse to track"|QText]),
+    Help = join_msg([button_format(wings_s:accept(),
+				   ?STR(message,2,"Drag to Dolly")),
+	 	     ?STR(message,7,"Restore view")|QText]),
     message(Help).
 
 %%%
@@ -474,21 +481,22 @@ get_mirai_event(Camera, Redraw, MouseRotates, View) ->
     {replace,fun(Ev) -> mirai_event(Ev, Camera, Redraw, MouseRotates, View) end}.
 
 mirai_message(true) ->
-    Help = join_msg([button_format("Accept",
-				   "Drag to Dolly",
-				   "Cancel/restore view"),
-		     "Move mouse to tumble",
-		     ["[Q]",?CSEP,"Move mouse to track"]]),
+    Help = join_msg([button_format(wings_s:accept(),
+				   ?STR(message,2,"Drag to Dolly"),
+				   ?STR(message,6,"Cancel/restore view")),
+		     ?STR(message,3,"Move mouse to tumble"),
+		     [?STR(message,4,"[Q]"),?CSEP,
+		      ?STR(message,5,"Move mouse to track")]]),
     message(Help);
 mirai_message(false) ->
     QText = case allow_rotation() of
 		false -> [];
-		true -> ["[Q]",?CSEP,"Move mouse to tumble"]
+		true -> [?STR(message,4,"[Q]"),?CSEP,?STR(message,3,"Move mouse to tumble")]
 	    end,
-    Help = join_msg([button_format("Accept",
-				   "Drag to Dolly",
-				   "Cancel/restore view"),
-		     "Move mouse to track"|QText]),
+    Help = join_msg([button_format(wings_s:accept(),
+				   ?STR(message,2,"Drag to Dolly"),
+				   ?STR(message,6,"Cancel/restore view")),
+		     ?STR(message,5,"Move mouse to track")|QText]),
     message(Help).
 
 %%%
@@ -499,7 +507,7 @@ tds(#mousebutton{button=2,x=X0,y=Y0,state=?SDL_PRESSED}, Redraw) ->
     {X,Y} = wings_wm:local2global(X0, Y0),
     Camera = #camera{x=X,y=Y,ox=X,oy=Y},
     grab(),
-    message(join_msg(tds_help(), button_format([], [], "Restore view"))),
+    message(join_msg(tds_help(), button_format([], [], ?STR(message,7,"Restore view")))),
     View = wings_view:current(),
     {seq,push,get_tds_event(Camera, Redraw, View)};
 tds(_, _) -> next.
@@ -530,11 +538,11 @@ get_tds_event(Camera, Redraw, View) ->
     {replace,fun(Ev) -> tds_event(Ev, Camera, Redraw, View) end}.
 
 tds_help() ->
-    TrackDolly = [{0,2,"Track"},
-		  {?CTRL_BITS bor ?ALT_BITS,2,"Dolly"}],
+    TrackDolly = [{0,2,?STR(mode_help,1,"Track")},
+		  {?CTRL_BITS bor ?ALT_BITS,2,?STR(mode_help,2,"Dolly")}],
     case allow_rotation() of
 	false -> format(TrackDolly);
-	true -> format([{?ALT_BITS,2,"Tumble"}|TrackDolly])
+	true -> format([{?ALT_BITS,2,?STR(mode_help,3,"Tumble")}|TrackDolly])
     end.
 
 %%%
@@ -590,11 +598,11 @@ maya_stop_camera(Camera) ->
     stop_camera(Camera).
 
 maya_help() ->
-    TrackDolly = [{?ALT_BITS,2,"Track"},
-		  {?ALT_BITS,3,"Dolly"}],
+    TrackDolly = [{?ALT_BITS,2,?STR(mode_help,1,"Track")},
+		  {?ALT_BITS,3,?STR(mode_help,2,"Dolly")}],
     case allow_rotation() of
 	false -> format(TrackDolly);
-	true -> format([{?ALT_BITS,1,"Tumble"}|TrackDolly])
+	true -> format([{?ALT_BITS,1,?STR(mode_help,3,"Tumble")}|TrackDolly])
     end.
 
 %%%
@@ -635,11 +643,11 @@ get_mb_event(Camera, Redraw) ->
     {replace,fun(Ev) -> mb_event(Ev, Camera, Redraw) end}.
 
 mb_help() ->
-    TrackDolly = [{?SHIFT_BITS,1,"Track"},
-		  {?CTRL_BITS,1,"Dolly"}],
+    TrackDolly = [{?SHIFT_BITS,1,?STR(mode_help,1,"Track")},
+		  {?CTRL_BITS,1,?STR(mode_help,2,"Dolly")}],
     case allow_rotation() of
 	false -> format(TrackDolly);
-	true -> format([{?SHIFT_BITS bor ?CTRL_BITS,1,"Tumble"}|TrackDolly])
+	true -> format([{?SHIFT_BITS bor ?CTRL_BITS,1,?STR(mode_help,3,"Tumble")}|TrackDolly])
     end.
     
 %%%
@@ -680,7 +688,11 @@ zoom_step(Dir) ->
 	true ->
 	    wings_wm:dirty(),
 	    #view{distance=Dist} = View = wings_view:current(),
-	    ZoomPercent = wings_pref:get_value(wheel_zoom_factor, ?ZOOM_FACTOR)/100,
+	    ZoomPercent0 = wings_pref:get_value(wheel_zoom_factor, ?ZOOM_FACTOR)/100,
+	    ZoomPercent = case wings_pref:get_value(inverted_wheel_zoom) of
+			      false -> ZoomPercent0;
+			      true -> -ZoomPercent0
+			  end,
 	    Delta = dist_factor(Dist)*Dir*ZoomPercent,
 	    wings_view:set_current(View#view{distance=Dist+Delta}),
 	    keep
