@@ -9,7 +9,7 @@
 %%  See the file "license.terms" for information on usage and redistribution
 %%  of this file, and for a DISCLAIMER OF ALL WARRANTIES.
 %%
-%%     $Id: wings_extrude_edge.erl,v 1.58 2004/05/19 03:45:22 bjorng Exp $
+%%     $Id: wings_extrude_edge.erl,v 1.60 2004/11/13 04:39:26 bjorng Exp $
 %%
 
 -module(wings_extrude_edge).
@@ -18,7 +18,6 @@
 -include("wings.hrl").
 -import(lists, [foldl/3,keydelete/3,member/2,sort/1,
 		reverse/1,reverse/2,last/1,foreach/2]).
--compile(inline).
 
 -define(DEFAULT_EXTRUDE_DIST, 0.2).
 -define(BEVEL_EXTRUDE_DIST_KLUDGE, 0.0001).
@@ -99,7 +98,7 @@ bevel_faces(Faces, #we{id=Id,mirror=MirrorFace}=We0, {Tvs,Limit0}) ->
     {We1,OrigVs,_,Forbidden} = extrude_edges(Edges, Dist, We0#we{mirror=none}),
     case {gb_trees:size(We0#we.es),gb_trees:size(We1#we.es)} of
 	{Same,Same} ->
-	    wings_util:error("Object is too small to bevel.");
+	wings_util:error(?STR(bevel_faces,1,"Object is too small to bevel."));
 	{_,_} ->
 	    We2 = wings_edge:dissolve_edges(Edges, We1),
 	    Tv0 = bevel_tv(OrigVs, We2, Forbidden),
@@ -434,7 +433,7 @@ connect(G, [C|Cs], ExtrudeDist, Wid, #we{mirror=Mirror}=We0, Closed) ->
 		    connect(G, Cs, ExtrudeDist, Wid, We0, [Face|Closed])
 	    end;
 	[_] ->
-	    wings_util:error("Can't extrude/bevel; try Cleanup.");
+	    wings_util:error(?STR(connect,1,"Can't extrude/bevel; try Cleanup."));
 	[Va0,Vb0] ->
 	    case digraph_get_path(G, Va0, Vb0) of
 		[{_,Mirror}|_] ->
