@@ -8,7 +8,7 @@
 %%  See the file "license.terms" for information on usage and redistribution
 %%  of this file, and for a DISCLAIMER OF ALL WARRANTIES.
 %%
-%%     $Id: wings_face_cmd.erl,v 1.107 2004/05/13 13:46:38 dgud Exp $
+%%     $Id: wings_face_cmd.erl,v 1.109 2004/11/13 04:39:26 bjorng Exp $
 %%
 
 -module(wings_face_cmd).
@@ -23,48 +23,46 @@
 
 menu(X, Y, St) ->
     Dir = wings_menu_util:directions(St),
-    Menu = [{basic,{"Face operations",ignore}},
+    Menu = [{basic,{?STR(menu,1,"Face operations"),ignore}},
 	    {basic,separator},
-	    {"Move",{move,Dir},[],[magnet]},
+	    {?STR(menu,2,"Move"),{move,Dir},[],[magnet]},
 	    wings_menu_util:rotate(St),
 	    wings_menu_util:scale(St),
 	    separator,
-	    {"Extrude",{extrude,Dir}},
-	    {"Extrude Region",{extrude_region,Dir}},
-	    {"Extract Region",{extract_region,Dir}},
+	    {?STR(menu,3,"Extrude"),{extrude,Dir}},
+	    {?STR(menu,4,"Extrude Region"),{extrude_region,Dir}},
+	    {?STR(menu,5,"Extract Region"),{extract_region,Dir}},
 	    separator,
 	    wings_menu_util:flatten(),
 	    separator,
-	    {"Inset",inset,"Inset a face inside the selected face"},
-	    {"Intrude",intrude,"Carve out interior of object, "
-	     "making selected faces holes"},
-	    {"Bevel",bevel,"Round off edges of selected faces"},
-	    {"Bridge",bridge,"Create a bridge or tunnel between two faces"},
+	    {?STR(menu,6,"Inset"),inset,?STR(menu,7,"Inset a face inside the selected face")},
+	    {?STR(menu,8,"Intrude"),intrude,?STR(menu,9,"Carve out interior of object, making selected faces holes")},
+	    {?STR(menu,10,"Bevel"),bevel,?STR(menu,11,"Round off edges of selected faces")},
+	    {?STR(menu,12,"Bridge"),bridge,?STR(menu,13,"Create a bridge or tunnel between two faces")},
 	    {advanced,separator},
-	    {"Bump",bump,"Create bump of selected faces"},
-	    {advanced,{"Lift",{lift,lift_fun(St)}}},
-	    {advanced,{"Put On",put_on_fun(),
-		       {"Move and rotate object, aligning "
-			"the selected face to another element",[],
-			"Clone object on to one or more elements"},[]}},
+	    {?STR(menu,14,"Bump"),bump,?STR(menu,15,"Create bump of selected faces")},
+	    {advanced,{?STR(menu,16,"Lift"),{lift,lift_fun(St)}}},
+	    {advanced,{?STR(menu,17,"Put On"),put_on_fun(),
+		       {?STR(menu,18,"Move and rotate object, aligning the selected face to another element"),[],
+		        ?STR(menu,19,"Clone object on to one or more elements")},[]}},
 	    separator,
-	    {"Mirror",mirror_fun(),
-	     {"Mirror object around selected faces and merge to object",[],
-	      "Mirror and create separate objects"},[]},
-    	    {"Dissolve",dissolve,"Eliminate all edges between selected faces"},
-	    {"Collapse",collapse,"Delete faces, replacing them with vertices"},
-	    {"Smooth",smooth,
-	     "Subdivide selected faces to smooth them (Catmull-Clark)"},
-	    {"Tesselate",{subdivide,wings_tesselation:submenu()}},
+	    {?STR(menu,20,"Mirror"),mirror_fun(),
+	     {?STR(menu,21,"Mirror object around selected faces and merge to object"),[],
+	  	?STR(menu,22,"Mirror and create separate objects")},[]},
+    	    {?STR(menu,23,"Dissolve"),dissolve,?STR(menu,24,"Eliminate all edges between selected faces")},
+	    {?STR(menu,25,"Collapse"),collapse,?STR(menu,26,"Delete faces, replacing them with vertices")},
+	    {?STR(menu,27,"Smooth"),smooth,
+		?STR(menu,28,"Subdivide selected faces to smooth them (Catmull-Clark)")},
+	    {?STR(menu,29,"Tesselate"),{subdivide,wings_tesselation:submenu()}},
 	    separator] ++ wings_material:material_menu(St) ++
-	[{"Vertex Color",vertex_color,
-	  "Apply vertex colors to selected faces"}],
+	[{?STR(menu,30,"Vertex Color"),vertex_color,
+	  ?STR(menu,31,"Apply vertex colors to selected faces")}],
     wings_menu:popup_menu(X, Y, face, Menu).
 
 lift_fun(St) ->
     fun(help, _Ns) ->
-	    {"Lift, rotating face around edge or vertex",[],
-	     "Lift in std. directions"};
+	    {?STR(lift_fun,1,"Lift, rotating face around edge or vertex"),[],
+	     ?STR(lift_fun,2,"Lift in std. directions")};
        (1, Ns) ->
 	    Funs = lift_selection(rotate, St),
 	    wings_menu:build_command({'ASK',Funs}, Ns);
@@ -211,7 +209,7 @@ extract_region(Type, St0) ->
     St1 = wings_sel:fold(
 	    fun(Faces, We0, #st{sel=Sel0,onext=Oid}=S0) ->
 		    We = We0#we{mirror=none},
-		    S = wings_shape:insert(We, "extract", S0),
+		    S = wings_shape:insert(We, extract, S0),
 		    Sel = [{Oid,Faces}|Sel0],
 		    S#st{sel=Sel}
 	    end, St0#st{sel=[]}, St0),
@@ -252,7 +250,7 @@ dissolve(Faces, #we{id=Id}=We0, Acc) ->
 	    Sel = wings_we:new_items(face, We0, We),
 	    {We,[{Id,Sel}|Acc]};
 	false ->
-	    wings_util:error("Dissolving would cause an inconsistent object structure.")
+	    wings_util:error(?STR(dissolve,1,"Dissolving would cause an inconsistent object structure."))
     end.
 		  
 dissolve_1(Faces, We) ->
@@ -375,7 +373,7 @@ mirror_sep_faces(Faces, We0, Acc) when is_list(Faces) ->
     Template = wings_we:invert_normals(We0),
     foldl(fun(Face, A) ->
 		  We = mirror_vs(Face, Template),
-		  wings_shape:insert(We, "mirror", A)
+		  wings_shape:insert(We, mirror, A)
 	  end, Acc, Faces);
 mirror_sep_faces(Faces, We, Acc) ->
     mirror_sep_faces(gb_sets:to_list(Faces), We, Acc).
@@ -586,7 +584,7 @@ smooth(St0) ->
 
 smooth(Faces0, #we{id=Id}=We0, Acc) ->
     Rs = wings_sel:face_regions(Faces0, We0),
-    wings_pb:start("smoothing"),
+    wings_pb:start(?STR(smooth,1,"smoothing")),
     We1 = wings_pb:done(smooth_regions(Rs, 1, length(Rs), We0)),
     NewFaces = wings_we:new_items(face, We0, We1),
     NewVs = wings_we:new_items(vertex, We0, We1),
@@ -692,8 +690,7 @@ bridge(_St) ->
 
 unify_modes(#we{mode=Mode}, #we{mode=Mode}) -> Mode;
 unify_modes(_, _) ->
-    wings_util:error("An object with vertex colors "
-		     "cannot be bridged with an object with materials.").
+    wings_util:error(?STR(unify_modes,1,"An object with vertex colors cannot be bridged with an object with materials.")).
 
 bridge_null_uvs(Mode, #we{mode=Mode}=We) -> We;
 bridge_null_uvs(uv, #we{es=Etab0}=We) ->
@@ -710,17 +707,17 @@ bridge(FaceA, FaceB, #we{vp=Vtab}=We) ->
     VsB = wings_face:vertices_ccw(FaceB, We),
     if
 	length(VsA) =/= length(VsB) ->
-	    bridge_error("Faces must have the same number of vertices.");
+	    bridge_error(?STR(bridge,1,"Faces must have the same number of vertices."));
 	true ->
 	    An = wings_face:face_normal_cw(VsA, Vtab),
 	    Bn = wings_face:face_normal_cw(VsB, Vtab),
 	    case e3d_vec:dot(An, Bn) of
 		Dot when Dot > 0.99 ->
-		    bridge_error("Faces must not point in the same direction.");
+		    bridge_error(?STR(bridge,2,"Faces must not point in the same direction."));
 		_Dot ->
 		    case wings_face:are_neighbors(FaceA, FaceB, We) of
 			true ->
-			    bridge_error("Faces must not be neighbors.");
+			    bridge_error(?STR(bridge,3,"Faces must not be neighbors."));
 			false ->
 			    bridge(FaceA, VsA, FaceB, VsB, We)
 		    end
@@ -830,7 +827,7 @@ get_edge(Edge, Etab) ->
     end.
 
 bridge_error() ->
-    bridge_error("Exactly two faces must be selected.").
+    bridge_error(?STR(bridge_error,1,"Exactly two faces must be selected.")).
 
 bridge_error(Error) ->
     wings_util:error(Error).
@@ -847,7 +844,7 @@ bridge_color(Edge, Face, Iter) ->
 %%%
 
 lift_selection(Dir, OrigSt) ->
-    Desc = "Select edge or vertex to act as hinge",
+    Desc = ?STR(lift_selection,1,"Select edge or vertex to act as hinge"),
     Fun = fun(check, St) ->
 		  lift_check_selection(St, OrigSt);
 	     (exit, {_,_,#st{selmode=Mode,sel=Sel}=St}) ->
@@ -870,7 +867,7 @@ lift_check_selection(#st{selmode=edge,sel=EdgeSel}, OrigSt) ->
 	    end, EdgeSel, OrigSt),
     case Res of
 	[] -> {none,""};
-	_ -> {none,"Face and edge selections don't match."}
+	_ -> {none,?STR(lift_check_selection,1,"Face and edge selections don't match.")}
     end;
 lift_check_selection(#st{selmode=vertex,sel=VsSel}, OrigSt) ->
     Res = wings_sel:fold(
@@ -884,7 +881,7 @@ lift_check_selection(#st{selmode=vertex,sel=VsSel}, OrigSt) ->
 	    end, VsSel, OrigSt),
     case Res of
 	[] -> {none,""};
-	_ -> {none,"Face and vertex selections don't match."}
+	_ -> {none,?STR(lift_check_selection,2,"Face and vertex selections don't match.")}
     end.
 
 lift({'ASK',Ask}, St) ->
@@ -918,7 +915,7 @@ lift_from_edge(Dir, EdgeSel, St0) ->
     end.
 
 lift_sel_mismatch() ->
-    wings_util:error("Face and edge selections don't match.").
+    wings_util:error(?STR(lift_sel_mismatch,1,"Face and edge selections don't match.")).
 	
 lift_from_edge(Dir, Faces, Edges, We0, Tv) ->
     case lift_face_edge_pairs(Faces, Edges, We0) of
@@ -1014,7 +1011,7 @@ lift_from_vertex(Dir, VsSel, St0) ->
     end.
 
 lift_vtx_sel_mismatch() ->
-    wings_util:error("Face and vertex selections don't match.").
+    wings_util:error(?STR(lift_vtx_sel_mismatch,1,"Face and vertex selections don't match.")).
 
 lift_from_vertex(Dir, Faces, Vs, We, Tv) ->
     case lift_face_vertex_pairs(Faces, Vs, We) of
@@ -1091,13 +1088,13 @@ put_on(#st{sel=[{_,Faces}]}=St) ->
 	1 ->
 	    wings:ask(put_on_selection(St), St, fun put_on/2);
 	_ ->
-	    wings_util:error("There must only be one face selected.")
+	    wings_util:error(?STR(put_on,1,"There must only be one face selected."))
     end;
 put_on(_) ->
-    wings_util:error("There must only be one face selected.").
+    wings_util:error(?STR(put_on,1,"There must only be one face selected.")).
 
 put_on_selection(OrigSt) ->
-    Desc = "Select target element on which to put source object",
+    Desc = ?STR(put_on_selection,1,"Select target element on which to put source object"),
     Fun = fun(check, St) -> put_on_check_selection(St, OrigSt);
 	     (exit, {_,_,#st{selmode=Mode,sel=Sel}=St}) ->
 		  case put_on_check_selection(St, OrigSt) of
@@ -1108,14 +1105,14 @@ put_on_selection(OrigSt) ->
     {[{Fun,Desc}],[],[],[face,edge,vertex]}.
 
 put_on_check_selection(#st{sel=[{Id,_}]}, #st{sel=[{Id,_}]}) ->
-    {none,"Selection must not be in the same object."};
+    {none,?STR(put_on_check_selection,1,"Selection must not be in the same object.")};
 put_on_check_selection(#st{sel=[{_,Elems}]}, _) ->
     case gb_trees:size(Elems) of
 	1 -> {none,""};
-	_ -> {none,"Select only one element."}
+	_ -> {none,?STR(put_on_check_selection,2,"Select only one element.")}
     end;
 put_on_check_selection(_, _) ->
-    {none,"Select only one element."}.
+    {none,?STR(put_on_check_selection,2,"Select only one element.")}.
 
 put_on({Mode,[{Id,Els}]}, #st{shapes=Shs}=St0) ->
     We0 = gb_trees:get(Id, Shs),
@@ -1146,13 +1143,13 @@ clone_on(#st{sel=[{_,Faces}]}=St) ->
 	1 ->
 	    wings:ask(clone_on_selection(), St, fun clone_on/2);
 	_ ->
-	    wings_util:error("There must only be one face selected.")
+	    wings_util:error(?STR(clone_on,1,"There must only be one face selected."))
     end;
 clone_on(_) ->
-    wings_util:error("There must only be one face selected.").
+    wings_util:error(?STR(clone_on,1,"There must only be one face selected.")).
 
 clone_on_selection() ->
-    Desc = "Select target elements on which to put clones",
+    Desc = ?STR(clone_on_selection,1,"Select target elements on which to put clones"),
     Fun = fun(check, _) ->
 		  {none,""};
 	     (exit, {_,_,#st{selmode=Mode,sel=Sel}}) ->
@@ -1189,7 +1186,7 @@ clone_3(El, We, Tr, N, Clone, #st{selmode=Mode}=St) ->
     M1 = e3d_mat:mul(M0, RotAxis),
     M = e3d_mat:mul(M1, Tr),
     NewWe = wings_we:transform_vs(M, Clone),
-    wings_shape:insert(NewWe, "_clone", St).
+    wings_shape:insert(NewWe, clone, St).
 
 %%
 %% Common help function.
