@@ -8,12 +8,13 @@
 %%  See the file "license.terms" for information on usage and redistribution
 %%  of this file, and for a DISCLAIMER OF ALL WARRANTIES.
 %%
-%%     $Id: wings_material.erl,v 1.115 2004/05/31 06:18:13 bjorng Exp $
+%%     $Id: wings_material.erl,v 1.120 2004/10/22 07:43:07 dgud Exp $
 %%
 
 -module(wings_material).
 -export([material_menu/1,command/2,new/1,color/4,default/0,
-	 mat_faces/2,add_materials/2,update_materials/2,
+	 mat_faces/2,add_materials/2,add_materials/3,
+	 update_materials/2,
 	 update_image/4,used_images/1,
 	 get/2,get_all/1,delete_face/2,delete_faces/2,cleanup/1,
 	 assign/3,replace_materials/2,assign_materials/2,
@@ -33,17 +34,17 @@
 		keyreplace/4,keydelete/3,keysearch/3,flatten/1]).
 
 material_menu(St) ->
-    [{basic,{"Material",
+    [{basic,{?STR(material_menu,1,"Material"),
 	     {material,
-	      [{"New...",new,
-		"Create a new material and assign to selected faces"},
+	      [{?STR(material_menu,2,"New..."),new,
+		?STR(material_menu,3,"Create a new material and assign to selected faces")},
 	       separator|mat_list(St)]}}},
-     {advanced,{"Material",{material,material_fun(St)}}}].
+     {advanced,{?STR(material_menu,4,"Material"),{material,material_fun(St)}}}].
 
 material_fun(St) ->
     fun(help, _Ns) ->
-	    {"Assign existing material to selection",[],
-	     "Create and assign new material"};
+	    {?STR(material_fun,1,"Assign existing material to selection"),[],
+	     ?STR(material_fun,2,"Create and assign new material")};
        (1, _Ns) ->
 	    mat_list(St);
        (3, _) ->
@@ -65,8 +66,9 @@ new(_) ->
     new_1(new).
 
 new_1(Act) ->
-    wings_ask:ask("New Material",
-		  [{"Material Name","New Material"}],
+    wings_ask:ask(?STR(new_1,1,"New Material"),
+		  [{?STR(new_1,2,"Material Name"),
+		    ?STR(new_1,3,"New Material")}],
 		  fun([Name]) ->
 			  Action = {action,{material,{Act,Name}}},
 			  wings_wm:send_after_redraw(geom, Action),
@@ -138,7 +140,7 @@ delete_material([], St) ->
 
 rename(Mats, St) ->
     Qs = rename_qs(Mats),
-    wings_ask:dialog("Rename", Qs,
+    wings_ask:dialog(?STR(rename,1,"Rename"), Qs,
 		     fun(NewNames) ->
 			     rename_1(NewNames, St, [])
 		     end).
@@ -206,11 +208,11 @@ select_material(Mat, St) ->
 
 set_material(Mat, #st{selmode=face}=St) ->
     wings_sel:map(fun(Faces, We) ->
-			  assign(Mat, Faces, We)
+			  assign(Mat, Faces, We#we{mode=material})
 		  end, St);
 set_material(Mat, #st{selmode=body}=St) ->
     wings_sel:map(fun(_, #we{fs=Ftab}=We) ->
-			  assign(Mat, gb_trees:keys(Ftab), We)
+			  assign(Mat, gb_trees:keys(Ftab), We#we{mode=material})
 		  end, St);
 set_material(_, St) -> St.
 
@@ -234,19 +236,23 @@ update_image(MatName, MapType, Image, #st{mat=Mtab}) ->
     wings_image:update(ImageId, Image).
 
 add_materials(Ms, St) ->
-    add_materials(Ms, St, []).
+    Dir = wings_pref:get_value(current_directory),
+    add_materials_1(Ms, Dir, St, []).
 
-add_materials([{Name,Mat0}|Ms], St0, NewNames) ->
+add_materials(Ms, Dir, St) ->
+    add_materials_1(Ms, Dir, St, []).
+
+add_materials_1([{Name,Mat0}|Ms], Dir, St0, NewNames) ->
     Mat1 = add_defaults(Mat0),
-    Maps = load_maps(prop_get(maps, Mat1, [])),
+    Maps = load_maps(prop_get(maps, Mat1, []), Dir),
     Mat = keyreplace(maps, 1, Mat1, {maps,Maps}),
     case add(Name, Mat, St0) of
 	#st{}=St ->
-	    add_materials(Ms, St, NewNames);
+	    add_materials_1(Ms, Dir, St, NewNames);
 	{#st{}=St,NewName} ->
-	    add_materials(Ms, St, [{Name,NewName}|NewNames])
+	    add_materials_1(Ms, Dir, St, [{Name,NewName}|NewNames])
     end;
-add_materials([], St, NewNames) -> {St,NewNames}.
+add_materials_1([], _, St, NewNames) -> {St,NewNames}.
 
 add_defaults([]) ->
     add_defaults([{opengl,[]},{maps,[]}]);
@@ -269,7 +275,8 @@ add_defaults_1(P) ->
 
 update_materials([{Name,Mat0}|Ms], St) ->
     Mat1 = add_defaults(Mat0),
-    Maps = load_maps(prop_get(maps, Mat1, [])),
+    Dir = wings_pref:get_value(current_directory),
+    Maps = load_maps(prop_get(maps, Mat1, []), Dir),
     Mat = keyreplace(maps, 1, Mat1, {maps,Maps}),
     update_materials(Ms, update(Name, Mat, St));
 update_materials([], St) -> St.
@@ -277,44 +284,46 @@ update_materials([], St) -> St.
 norm({_,_,_,_}=Color) -> Color;
 norm({R,G,B}) -> {R,G,B,1.0}.
     
-load_maps([{Key,Filename}|T]) when is_list(Filename) ->
-    case load_map(Filename) of
-	none -> load_maps(T);
-	Map -> [{Key,Map}|load_maps(T)]
+load_maps([{Key,Filename}|T], Dir) when is_list(Filename) ->
+    case load_map(Filename, Dir) of
+	none -> load_maps(T, Dir);
+	Map -> [{Key,Map}|load_maps(T, Dir)]
     end;
-load_maps([{Key,{W,H,Bits}}|T]) ->
+load_maps([{Key,{W,H,Bits}}|T], Dir) ->
     E3dImage = #e3d_image{type=r8g8b8,order=lower_left,
 			  width=W,height=H,image=Bits},
     Id = wings_image:new(atom_to_list(Key), E3dImage),
-    [{Key,Id}|load_maps(T)];
-load_maps([{Key,#e3d_image{}=E3dImage}|T]) ->
+    [{Key,Id}|load_maps(T, Dir)];
+load_maps([{Key,#e3d_image{}=E3dImage}|T], Dir) ->
     Id = wings_image:new(atom_to_list(Key), E3dImage),
-    [{Key,Id}|load_maps(T)];
-load_maps([{_,none}|T]) ->
-    load_maps(T);
-load_maps([{_,Id}=Map|T]) when is_integer(Id) ->
-    [Map|load_maps(T)];
-load_maps([]) -> [].
+    [{Key,Id}|load_maps(T, Dir)];
+load_maps([{_,none}|T], Dir) ->
+    load_maps(T, Dir);
+load_maps([{_,Id}=Map|T], Dir) when is_integer(Id) ->
+    [Map|load_maps(T, Dir)];
+load_maps([], _) -> [].
     
-load_map(MapName) ->
-    case catch load_map_1(MapName) of
+load_map(MapName, Dir) ->
+    try load_map_1(MapName, Dir) of
 	none -> none;
-	{'EXIT',R} ->
-	    io:format("~P\n", [R,20]),
-	    none;
 	Im when is_integer(Im) -> Im
+    catch
+	error:R ->
+	    io:format("~p\n", [R]),
+	    io:format("~P\n", [erlang:get_stacktrace(),20]),
+	    none
     end.
 
-load_map_1(none) -> none;
-load_map_1(File0) ->
-    File = filename:absname(File0, wings_pref:get_value(current_directory)),
+load_map_1(none, _) -> none;
+load_map_1(File0, Dir) ->
+    File = filename:absname(File0, Dir),
     Ps = [{filename,File},{order,lower_left},{alignment,1}],
     case wpa:image_read(Ps) of
 	#e3d_image{}=Im ->
 	    Name = filename:rootname(filename:basename(File)),
 	    wings_image:new(Name, Im#e3d_image{filename=File});
 	{error,Error} ->
-	    io:format("Failed to load \"~s\": ~s\n",
+	    io:format(?STR(load_map_1,1,"Failed to load") ++ " \"~s\": ~s\n",
 		      [File,file:format_error(Error)]),
 	    none
     end.
@@ -457,7 +466,7 @@ is_mat_transparent(Mat) ->
 edit(Name, Assign, #st{mat=Mtab}=St) ->
     Mat = gb_trees:get(Name, Mtab),
     {dialog,Qs,Fun} = edit_dialog(Name, Assign, St, Mat),
-    wings_ask:dialog("Material Properties: "++atom_to_list(Name), Qs, Fun).
+    wings_ask:dialog(?STR(edit,1,"Material Properties: ")++atom_to_list(Name), Qs, Fun).
     
 
 edit_dialog(Name, Assign, St=#st{mat=Mtab0}, Mat0) ->
@@ -476,10 +485,10 @@ edit_dialog(Name, Assign, St=#st{mat=Mtab0}, Mat0) ->
 	     {hframe, 
 	      [{custom,?PREVIEW_SIZE,?PREVIEW_SIZE+5,Preview},
 	       {vframe,
-		[{label,"Diffuse"},
-		 {label,"Ambient"},
-		 {label,"Specular"},
-		 {label,"Emission"}]
+		[{label,?STR(edit_dialog,1,"Diffuse")},
+		 {label,?STR(edit_dialog,2,"Ambient")},
+		 {label,?STR(edit_dialog,3,"Specular")},
+		 {label,?STR(edit_dialog,4,"Emission")}]
 	       },
 	       {vframe,
 		[{slider,{color,Diff0,[{key,diffuse}]}},
@@ -487,8 +496,8 @@ edit_dialog(Name, Assign, St=#st{mat=Mtab0}, Mat0) ->
 		 {slider,{color,Spec0,[{key,specular}]}},
 		 {slider,{color,Emiss0,[{key,emission}]}}
 		]}]},
-	     {hframe, [{vframe, [{label,"Shininess"},
-				 {label,"Opacity"}]},
+	     {hframe, [{vframe, [{label,?STR(edit_dialog,5,"Shininess")},
+				 {label,?STR(edit_dialog,6,"Opacity")}]},
 		       {vframe, [{slider,{text,Shine0,
 					  [{range,{0.0,1.0}},
 					   {key,shininess}]}},
@@ -499,8 +508,8 @@ edit_dialog(Name, Assign, St=#st{mat=Mtab0}, Mat0) ->
 	   }],
     Qs2 = wings_plugin:dialog({material_editor_setup,Name,Mat0}, Qs1),
     Qs = {hframe,[{vframe,Qs2},
-		  {vframe,[{button,"OK",done,[ok,{key,material_editor_ok}]},
-			   {button,cancel,[cancel]}]}]},
+		  {vframe,[{button,?STR(edit_dialog,7,"OK"),done,[ok,{key,material_editor_ok}]},
+			   {button,wings_s:cancel(),cancel,[cancel]}]}]},
     Ask = fun([{diffuse,Diff},{ambient,Amb},{specular,Spec},
 	       {emission,Emiss},{shininess,Shine},{opacity,Opacity}|More0]) ->
 		  OpenGL = [ask_prop_put(diffuse, Diff, Opacity),
@@ -529,9 +538,9 @@ plugin_results(Name, Mat0, Res0) ->
 	{Mat,[{material_editor_ok,false}]} ->
 	    {again,Mat};
 	{_,Res} ->
-	    io:format("Material editor plugin(s) left garbage:~n    ~P~n", 
+	    io:format(?STR(plugin_results,1,"Material editor plugin(s) left garbage:~n    ~P~n"), 
 		      [Res,20]),
-	    wings_util:error("Plugin(s) left garbage")
+		      wings_util:error(?STR(plugin_results,2,"Plugin(s) left garbage"))
     end.
 
 update_maps(Mat0, More0) ->
@@ -551,18 +560,18 @@ show_maps(Mat) ->
 	[] -> [];
 	Maps ->
 	    MapDisp = [show_map(M) || M <- sort(Maps)],
-	    [{vframe,MapDisp,[{title,"Textures"}]}]
+	    [{vframe,MapDisp,[{title,?STR(show_maps,1,"Textures")}]}]
     end.
 
 show_map({Type,Image}) ->
     Texture = 
 	case wings_image:info(Image) of
 	    none ->
-		[{label,flatten(io_lib:format("~p: <image deleted>", [Type]))}];
+		[{label,flatten(io_lib:format(?STR(show_map,1,"~p: <image deleted>"), [Type]))}];
 	    #e3d_image{name=Name,width=W,height=H,bytes_pp=PP} ->
-		Label = flatten(io_lib:format("~p: ~p [~px~px~p]",
+		Label = flatten(io_lib:format(?STR(show_map,2,"~p: ~p [~px~px~p]"),
 					      [Type,Name,W,H,PP*8])),
-		[{label, Label},{button,"Delete",done}]
+		[{label, Label},{button,?STR(show_map,3,"Delete"),done}]
 	end,
     {hframe, Texture}.
 
@@ -649,6 +658,7 @@ color(Face, UV, We, #st{mat=Mtab}) ->
 color(_Face, {_,_,_}=RGB, _We, _St) -> RGB.
 
 color_1(_, none) -> wings_color:white();
+color_1(none, _) -> wings_color:white();
 color_1({U0,V0}, #e3d_image{width=W,height=H,image=Bits}) ->
     U = (((round(U0*W) rem W) + W) rem W),
     V = ((round(V0*H) rem H) + H) rem H,
@@ -752,17 +762,16 @@ assign_materials([{F,M}|_]=MatFace0, We) when is_integer(F), is_atom(M) ->
     assign_materials(MatFace, We);
 assign_materials([], We) -> We.
 
-assign(Mat, _, #we{mat=Mat}=We) ->
-    We#we{mode=material};
+assign(Mat, _, #we{mat=Mat}=We) -> We;
 assign(Mat, Faces, #we{mat=Tab0,fs=Ftab}=We) when is_list(Faces) ->
     case length(Faces) =:= gb_trees:size(Ftab) of
 	true ->
-	    We#we{mode=material,mat=Mat};
+	    We#we{mat=Mat};
 	false ->
 	    Tab = force_list(Tab0, Ftab),
 	    NewTab = sort(make_tab(Mat, Faces)),
 	    MatTab = mat_merge(NewTab, Tab, []),
-	    We#we{mode=material,mat=MatTab}
+	    We#we{mat=MatTab}
     end;
 assign(Mat, Faces, We) ->
     assign(Mat, gb_sets:to_list(Faces), We).
