@@ -8,7 +8,7 @@
 %%  See the file "license.terms" for information on usage and redistribution
 %%  of this file, and for a DISCLAIMER OF ALL WARRANTIES.
 %%
-%%     $Id: wings_util.erl,v 1.93 2004/06/02 04:41:34 bjorng Exp $
+%%     $Id: wings_util.erl,v 1.99 2004/11/02 07:20:34 bjorng Exp $
 %%
 
 -module(wings_util).
@@ -16,14 +16,15 @@
 	 validate_mirror/1,rel2fam/1,
 	 button_message/1,button_message/2,button_message/3,
 	 button_format/1,button_format/2,button_format/3,
-	 rmb_format/1,
+	 rmb_format/1,key_format/2,
 	 join_msg/1,join_msg/2,
 	 message/1,
 	 magnet_string/0,
 	 yes_no/2,yes_no/3,yes_no_cancel/3,
 	 get_matrices/2,mirror_matrix/1,
 	 mirror_flatten/2,
-	 cap/1,upper/1,stringify/1,add_vpos/2,update_vpos/2,
+	 cap/1,upper/1,stringify/1,quote/1,
+	 add_vpos/2,update_vpos/2,
 	 gb_trees_smallest_key/1,gb_trees_largest_key/1,
 	 nice_float/1,
 	 menu_restriction/2,
@@ -33,7 +34,7 @@
 	 geom_windows/0,
 	 tc/3,export_we/2,crash_log/2,validate/1,validate/3,
 	 gl_error_string/1,
-	 min/2,max/2,lowpass/1,lowpass/2]).
+	 min/2,max/2,limit/2,lowpass/1,lowpass/2]).
 -export([check_error/2,dump_we/2]).
 
 -define(NEED_OPENGL, 1).
@@ -118,6 +119,9 @@ button_format(LmbMsg, MmbMsg, RmbMsg) ->
 rmb_format(Msg) ->
     wings_camera:rmb_format(Msg).
 
+key_format(Key, Msg) ->
+    [Key,160,Msg].
+
 join_msg(Messages) ->
     wings_camera:join_msg(Messages).
 
@@ -131,9 +135,9 @@ message(Message) ->
     wings_ask:dialog("", Qs, fun(_) -> ignore end).
 
 magnet_string() ->
-    ["(Magnet route:",
-     atom_to_list(wings_pref:get_value(magnet_distance_route))
-     ,")"].
+    ["(",?STR(magnet_string,1,"Magnet route:"),
+     atom_to_list(wings_pref:get_value(magnet_distance_route)),
+     ")"].
 
 get_matrices(Id, MM) ->
     wings_view:load_matrices(false),
@@ -178,16 +182,17 @@ yes_no(Question, Yes) ->
 yes_no(Question, Yes, No) ->
     Qs = {vframe,
 	  [{label,Question,[{break,45}]},
-	   {hframe,[{button,"Yes",yes_no_fun(Yes)},
-		    {button,"No",yes_no_fun(No),[cancel]}]}]},
+	   {hframe,[{button,wings_s:yes(),yes_no_fun(Yes)},
+		    {button,wings_s:no(),yes_no_fun(No),[cancel]}]}]},
     wings_ask:dialog("", Qs, fun(_) -> ignore end).
 
 yes_no_cancel(Question, Yes, No) ->
     Qs = {vframe,
 	  [{label,Question,[{break,45}]},
-	   {hframe,[{button,"Yes",yes_no_fun(Yes)},
-		    {button,"No",yes_no_fun(No)},
-		    {button,"Cancel",yes_no_fun(ignore),[cancel]}]}]},
+	   {hframe,[{button,wings_s:yes(),yes_no_fun(Yes)},
+		    {button,wings_s:no(),yes_no_fun(No)},
+		    {button,wings_s:cancel(),
+		     yes_no_fun(ignore),[cancel]}]}]},
     wings_ask:dialog("", Qs, fun(_) -> ignore end).
 
 yes_no_fun(ignore) -> fun(_) -> ignore end;
@@ -200,6 +205,9 @@ yes_no_fun(Fun) ->
 	    end
     end.
 
+quote(Str) when is_list(Str) ->
+    [$",Str,$"].
+
 stringify({Atom,Other}) when is_atom(Atom) ->
     cap(atom_to_list(Atom)) ++
 	case stringify(Other) of
@@ -677,6 +685,13 @@ max(_A, B) -> B.
 min(A, B) when A < B -> A;
 min(_A, B) -> B.
 
+limit(Val, {'-infinity',infinity}) -> Val;
+limit(Val, {Min,infinity}) when Val < Min -> Min;
+limit(Val, {'-infinity',Max}) when Val > Max -> Max;
+limit(Val, {Min,Max}) when Min < Max, Val < Min -> Min;
+limit(Val, {Min,Max}) when Min < Max, Val > Max -> Max;
+limit(Val, {Min,Max}) when Min < Max -> Val.
+
 lowpass(X, Y) ->
     case wings_pref:get_value(jumpy_camera) of
 	false -> {X,Y};
