@@ -8,7 +8,7 @@
 %%  See the file "license.terms" for information on usage and redistribution
 %%  of this file, and for a DISCLAIMER OF ALL WARRANTIES.
 %%
-%%     $Id: wings_view.erl,v 1.150 2004/05/18 09:52:43 raimo_niskanen Exp $
+%%     $Id: wings_view.erl,v 1.160 2004/11/02 07:20:34 bjorng Exp $
 %%
 
 -module(wings_view).
@@ -24,79 +24,75 @@
 -define(NEED_OPENGL, 1).
 -include("wings.hrl").
 
--import(lists, [foreach/2,foldl/3]).
+-import(lists, [foreach/2,foldl/3,zip/2]).
 
 menu(#st{views={CurrentView,Views}}=St) ->
     L = wings_pref:get_value(number_of_lights),
-    [{"Ground Plane",show_groundplane,"Show the ground plane",
+    [{?STR(menu,1,"Ground Plane"),show_groundplane,?STR(menu,2,"Show the ground plane"),
       crossmark(show_groundplane)},
-     {"Axes",show_axes,"Show the coordinate axes",crossmark(show_axes)},
+     {?STR(menu,3,"Axes"),show_axes,?STR(menu,4,"Show the coordinate axes"),crossmark(show_axes)},
      separator,
-     {"Workmode",workmode,"Toggle flat/smooth shading",
+     {?STR(menu,5,"Workmode"),workmode,?STR(menu,6,"Toggle flat/smooth shading"),
       crossmark(workmode)},
      separator,
-     {"Wireframe",wireframe,"Display selected objects as a wireframe "
-      "(same for all objects if nothing is selected)"},
-     {"Shade",shade,"Display selected objects as shaded "
-      "(same for all objects if nothing is selected)"},
-     {"Toggle Wireframe",toggle_wireframe,
-      "Toggle display mode for selected objects "
-      "(same for all objects if nothing is selected)",wireframe_crossmark(St)},
+     {?STR(menu,7,"Wireframe"),wireframe,?STR(menu,8,"Display selected objects as a wireframe (same for all objects if nothing is selected)")},
+     {?STR(menu,9,"Shade"),shade,?STR(menu,10,"Display selected objects as shaded (same for all objects if nothing is selected)")},
+     {?STR(menu,11,"Toggle Wireframe"),toggle_wireframe,
+      ?STR(menu,12,"Toggle display mode for selected objects (same for all objects if nothing is selected)"),wireframe_crossmark(St)},
      separator,
-     {"Toggle Proxy Mode",smooth_proxy,
-      "Toggle the smooth proxy mode for selected objects"},
-     {"Quick Smoothed Preview",quick_preview,
-      "Toggle the smooth proxy mode for all objects"},
+     {?STR(menu,13,"Toggle Proxy Mode"),smooth_proxy,
+      ?STR(menu,14,"Toggle the smooth proxy mode for selected objects")},
+     {?STR(menu,15,"Quick Smoothed Preview"),quick_preview,
+      ?STR(menu,16,"Toggle the smooth proxy mode for all objects")},
      separator,
-     {"Show Saved BB",show_bb,"Display any saved bounding box",crossmark(show_bb)},
-     {"Show Edges",show_edges,"Show edges in workmode",crossmark(show_edges)},
-     {"Show Wireframe Backfaces",show_wire_backfaces,
-      "Show wireframe backfaces",crossmark(show_wire_backfaces)},
-     {"Show Normals",show_normals,"Show normals for selected elements",
+     {?STR(menu,17,"Show Saved BB"),show_bb,?STR(menu,18,"Display any saved bounding box"),crossmark(show_bb)},
+     {?STR(menu,19,"Show Edges"),show_edges,?STR(menu,20,"Show edges in workmode"),crossmark(show_edges)},
+     {?STR(menu,21,"Show Wireframe Backfaces"),show_wire_backfaces,
+      ?STR(menu,22,"Show wireframe backfaces"),crossmark(show_wire_backfaces)},
+     {?STR(menu,23,"Show Normals"),show_normals,?STR(menu,24,"Show normals for selected elements"),
       crossmark(show_normals)},
      separator,
-     {"Reset View",reset,"Reset view to the default position"},
-     {"Aim",aim,"Aim the camera at the selected element"},
-     {"Frame",frame,"Dolly to show all selected elements "
-      "(or all objects if nothing is selected)"},
-     {"Orthographic View",orthogonal_view,
-      "Toggle between orthographic and perspective views",
+     {?STR(menu,25,"Reset View"),reset,?STR(menu,26,"Reset view to the default position")},
+     {?STR(menu,27,"Aim"),aim,?STR(menu,28,"Aim the camera at the selected element")},
+     {?STR(menu,29,"Frame"),frame,?STR(menu,30,"Dolly to show all selected elements (or all objects if nothing is selected)")},
+     {?STR(menu,31,"Orthographic View"),orthogonal_view,
+      ?STR(menu,32,"Toggle between orthographic and perspective views"),
       crossmark(orthogonal_view)},
-     {"Saved Views: "++integer_to_list(size(Views)),
+     {?STR(menu,33,"Saved Views: ")++integer_to_list(size(Views)),
       {views,views_submenu(CurrentView, Views)}},
      separator,
-     {"Camera Settings...",camera_settings,"Set field of view, and near and far clipping planes"},
+     {?STR(menu,34,"Camera Settings..."),camera_settings,?STR(menu,35,"Set field of view, and near and far clipping planes")},
      separator,
-     {"Scene Lights",scene_lights,
-      "Use the lights defined in the scene",
+     {?STR(menu,36,"Scene Lights"),scene_lights,
+      ?STR(menu,37,"Use the lights defined in the scene"),
       crossmark(scene_lights)},
-     {one_of(L == 1, "Two Lights", "One Light"),toggle_lights,
-      one_of(L == 1, "Use two work lights",
-	     "Use one work light")},
+     {one_of(L == 1, ?STR(menu,38,"Two Lights"),?STR(menu,39,"One Light")),toggle_lights,
+      one_of(L == 1, ?STR(menu,40,"Use two work lights"),
+	     ?STR(menu,41,"Use one work light"))},
      separator,
-     {"Show Colors",show_colors,
-      "Show vertex colors on objects in \"vertex\" mode",
+     {?STR(menu,42,"Show Colors"),show_colors,
+      ?STR(menu,43,"Show vertex colors on objects in \"vertex\" mode"),
       crossmark(show_colors)},
-     {"Show Materials",show_materials,
-      "Show materials on objects in \"material\" or \"uv\" modes",
+     {?STR(menu,44,"Show Materials"),show_materials,
+      ?STR(menu,45,"Show materials on objects in \"material\" mode"),
       crossmark(show_materials)},
-     {"Show Textures",show_textures,
-      "Show the texture on objects in \"uv\" mode",
+     {?STR(menu,46,"Show Textures"),show_textures,
+      ?STR(menu,47,"Show the texture on objects in \"material\" mode"),
       crossmark(show_textures)},
-     {"Show Info Text",show_info_text,
-      "Show an informational text at the top of this Geometry window",
+     {?STR(menu,48,"Show Info Text"),show_info_text,
+      ?STR(menu,49,"Show an informational text at the top of this Geometry window"),
       crossmark(show_info_text)},
      separator,
-     {"View Along",{along,[{"+X",x},
-			   {"+Y",y},
-			   {"+Z",z},
-			   {"-X",neg_x},
-			   {"-Y",neg_y},
-			   {"-Z",neg_z}]}},
+     {?STR(menu,50,"View Along"),{along,[{?STR(menu,51,"+X"),x},
+			   {?STR(menu,52,"+Y"),y},
+			   {?STR(menu,53,"+Z"),z},
+			   {?STR(menu,54,"-X"),neg_x},
+			   {?STR(menu,55,"-Y"),neg_y},
+			   {?STR(menu,56,"-Z"),neg_z}]}},
      separator,
-     {"Align to Selection",align_to_selection,
-      "Align the view to the normal of the selection"},
-     {"Auto Rotate",auto_rotate,"Spin the view"}].
+     {?STR(menu,57,"Align to Selection"),align_to_selection,
+      ?STR(menu,58,"Align the view to the normal of the selection")},
+     {?STR(menu,59,"Auto Rotate"),auto_rotate,?STR(menu,60,"Spin the view")}].
 
 crossmark(Key) ->
     Val = case wings_pref:get_value(Key) of
@@ -134,44 +130,45 @@ views_submenu(CurrentView, Views) ->
     Lines = wings_util:max((H div ?LINE_HEIGHT) - 3, 4),
     S = size(Views),
     C = if S > 0 -> view_index(CurrentView, S); true -> 0 end,
-    [{"Next",next,views_submenu_help(CurrentView, Views, next)},
-     {"Current",current,views_submenu_help(CurrentView, Views, current)},
-     {"Prev",prev,views_submenu_help(CurrentView, Views, prev)},
+    [{?STR(views_submenu,1,"Next"),next,views_submenu_help(CurrentView, Views, next)},
+     {?STR(views_submenu,2,"Current"),current,views_submenu_help(CurrentView, Views, current)},
+     {?STR(views_submenu,3,"Prev"),prev,views_submenu_help(CurrentView, Views, prev)},
      views_jumpmenu(CurrentView, Views, Lines),
-     {"Save",save,"Save this view at ["++integer_to_list(C+1)++"]",[option]},
+     {?STR(views_submenu,4,"Save"),save,
+      ?STR(views_submenu,5,"Save this view at ") ++"["++integer_to_list(C+1)++"]",[option]},
      views_movemenu(CurrentView, Views, Lines),
-     {"Rename...",rename,views_submenu_help(CurrentView, Views, rename)},
-     {"Delete",delete,views_submenu_help(CurrentView, Views, delete)},
-     {"Delete All...",delete_all,
+     {?STR(views_submenu,7,"Rename..."),rename,views_submenu_help(CurrentView, Views, rename)},
+     {?STR(views_submenu,8,"Delete"),delete,views_submenu_help(CurrentView, Views, delete)},
+     {?STR(views_submenu,9,"Delete All..."),delete_all,
       views_submenu_help(CurrentView, Views, delete_all)}].
 
 views_submenu_help(_CurrentView, {}, _Action) ->
-    "No saved views!";
+    ?STR(views_submenu_help,1,"No saved views!");
 views_submenu_help(CurrentView, Views, Action) ->
     S = size(Views),
     case Action of
 	next -> 
 	    N = view_index(CurrentView+1, S),
 	    {_,Legend} = element(N, Views),
-	    "Jump to \""++Legend++"\"["++integer_to_list(N)++"]";
+	    ?STR(views_submenu_help,2,"Jump to \"")++Legend++"\"["++integer_to_list(N)++"]";
 	current -> 
 	    C = view_index(CurrentView, S),
 	    {_,Legend} = element(C, Views),
-	    "Jump to \""++Legend++"\"["++integer_to_list(C)++"]";
+	    ?STR(views_submenu_help,2,"Jump to \"")++Legend++"\"["++integer_to_list(C)++"]";
 	prev -> 
 	    P = view_index(CurrentView-1, S),
 	    {_,Legend} = element(P, Views),
-	    "Jump to \""++Legend++"\"["++integer_to_list(P)++"]";
+	    ?STR(views_submenu_help,2,"Jump to \"")++Legend++"\"["++integer_to_list(P)++"]";
 	rename -> 
 	    C = view_index(CurrentView, S),
 	    {_,Legend} = element(C, Views),
-	    "Rename \""++Legend++"\"["++integer_to_list(C)++"]";
+	    ?STR(views_submenu_help,5,"Rename \"")++Legend++"\"["++integer_to_list(C)++"]";
 	delete ->
 	    C = view_index(CurrentView, S),
 	    {_,Legend} = element(view_index(CurrentView, S), Views),
-	    "Delete \""++Legend++"\"["++integer_to_list(C)++"]";
+	    ?STR(views_submenu_help,8,"Delete \"")++Legend++"\"["++integer_to_list(C)++"]";
 	delete_all ->
-	    "Delete all saved views"
+	    ?STR(views_submenu_help,11,"Delete all saved views")
     end.
 
 view_index(I, N) when integer(I), integer(N), N > 0 ->
@@ -181,7 +178,7 @@ view_index(I, N) when integer(I), integer(N), N > 0 ->
     end.
 
 views_jumpmenu(_CurrentView, {}, _Lines) ->
-    {"Jump",current,"No saved views!"};
+    {?STR(views_jumpmenu,1,"Jump"),current,?STR(views_jumpmenu,2,"No saved views!")};
 views_jumpmenu(CurrentView, Views, Lines) ->
     S = size(Views),
     P = view_index(CurrentView-1, S),
@@ -191,17 +188,17 @@ views_jumpmenu(CurrentView, Views, Lines) ->
 		{_,Legend} = element(I, Views),
 		Help =
 		    case I of
-			P -> "Jump to prev["++integer_to_list(I)++"]";
-			C -> "Jump to current["++integer_to_list(I)++"]";
-			N -> "Jump to next["++integer_to_list(I)++"]";
-			_ -> "Jump to ["++integer_to_list(I)++"]"
+			P -> ?STR(views_jumpmenu,3,"Jump to prev[")++integer_to_list(I)++"]";
+			C -> ?STR(views_jumpmenu,5,"Jump to current[")++integer_to_list(I)++"]";
+			N -> ?STR(views_jumpmenu,7,"Jump to next[")++integer_to_list(I)++"]";
+			_ -> ?STR(views_jumpmenu,9,"Jump to [")++integer_to_list(I)++"]"
 		    end,
 		{Legend,I,Help}
 	end,
-    {"Jump",{jump,viewmenu(F, S, C, Lines)}}.
+    {?STR(views_jumpmenu,11,"Jump"),{jump,viewmenu(F, S, C, Lines)}}.
 
 views_movemenu(_CurrentView, {}, _Lines) ->
-    {"Move Current",current,"No saved views!"};
+    {?STR(views_movemenu,1,"Move Current"),current,?STR(views_movemenu,2,"No saved views!")};
 views_movemenu(CurrentView, Views, Lines) ->
     S = size(Views),
     P = view_index(CurrentView-1, S),
@@ -214,14 +211,14 @@ views_movemenu(CurrentView, Views, Lines) ->
 		Li = integer_to_list(I),
 		Help =
 		    case I of
-			P -> "Move \""++CL++"\"["++Lc++"] to prev["++Li++"]";
-			C -> "Move \""++CL++"\"["++Lc++"] nowhere";
-			N -> "Move \""++CL++"\"["++Lc++"] to next["++Li++"]";
-			_ -> "Move \""++CL++"\"["++Lc++"] to ["++Li++"]"
+			P -> ?STR(views_movemenu, 3,"Move \"")++CL++"\"["++Lc++?STR(views_movemenu,5,"] to prev[")++Li++"]";
+			C -> ?STR(views_movemenu, 7,"Move \"")++CL++"\"["++Lc++?STR(views_movemenu,9,"] nowhere");
+			N -> ?STR(views_movemenu,10,"Move \"")++CL++"\"["++Lc++?STR(views_movemenu,12,"] to next[")++Li++"]";
+			_ -> ?STR(views_movemenu,14,"Move \"")++CL++"\"["++Lc++?STR(views_movemenu,16,"] to [")++Li++"]"
 		    end,
 		{Legend,I,Help}
 	end,
-    {"Move Current",{move,viewmenu(F, S, C, Lines)}}.
+    {?STR(views_movemenu,18,"Move Current"),{move,viewmenu(F, S, C, Lines)}}.
 
 %% Build a list of F(I) values for all view_index(I, S) starting from C
 %% half of them after C and half before, no more than Lines If the
@@ -365,7 +362,7 @@ virtual_mirror(create, #st{selmode=face}=St0) ->
     St = wings_sel:map(fun virtual_mirror_fun/2, St0),
     {save_state,St#st{sel=[]}};
 virtual_mirror(create, _) ->
-    wings_util:error("Virtual mirror requires a face selection.");
+    wings_util:error(?STR(virtual_mirror,1,"Virtual mirror requires a face selection."));
 virtual_mirror(break, St0) ->
     case break_mirror(St0) of
 	St0 -> St0;
@@ -415,7 +412,7 @@ virtual_mirror_fun(Faces, We) ->
 	[Face] ->
 	    We#we{mirror=Face};
 	_ ->
-	    wings_util:error("Only a single face must be selected per object.")
+	    wings_util:error(?STR(virtual_mirror_fun,1,"Only a single face must be selected per object."))
     end.
 
 break_mirror(#st{shapes=Shs0}=St) ->
@@ -442,23 +439,283 @@ sel_mirror_objects(St) ->
 		      (_, We, A) -> [We|A]
 		   end, [], St).
 
+
+
+-define(RANGE_FOV, {1.0,180.0}).
+-define(RANGE_NEAR_CLIP, {0.01,1000.0}).
+-define(RANGE_FAR_CLIP, {100.0,infinity}).
+-define(RANGE_ZOOM_SLIDER, {-1.0,4.0}).
+-define(RANGE_NEGATIVE_SIZE, {1,infinity}).
+
 camera() ->
     Active = wings_wm:this(),
     View0 = wings_wm:get_prop(Active, current_view),
+    NegH = wings_pref:get_value(negative_height),
+    NegW = wings_pref:get_value(negative_width),
     #view{fov=Fov0,hither=Hither0,yon=Yon0} = View0,
-    Qs = [{label_column,
-	   [{"Field of View",{text,Fov0,[{range,1.0,180.0}]}},
-	    {"Near Clipping Plane",{text,Hither0,
-				      [{range,0.001,1000.0}]}},
-	    {"Far Clipping Plane",{text,Yon0,
-				   [{range,100.0,9.9e307}]}}]}],
-    wings_ask:dialog("Camera Settings", Qs,
-		     fun([Fov,Hither,Yon]) ->
+    NegativeFormat =
+	case {NegH,NegW} of
+	    {24,36} -> {24,36};
+	    {60,60} -> {60,60};
+	    _ -> custom
+	end,
+    Props = 
+	camera_propconv(from_fov, 
+			[{negative_height,NegH},{negative_width,NegW},
+			 {fov,Fov0}]),
+    LensType = pget(lens_type, Props),
+    LensLength = pget(lens_length, Props),
+    Zoom = pget(zoom, Props),
+    ZoomSlider = pget(zoom_slider, Props),
+    FovHook = 
+	fun (update, {Var,_I,Val,Sto}) ->
+		{store,camera_update_1(Var, Val, Sto)};
+	    (_, _) -> void
+	end,
+    LensFrame =
+	{vframe,
+	 [{hframe,
+	   [{label,?STR(camera,1,"Negative Format")},
+	    {menu,
+	     [{"24x36",{24,36}},{"60x60",{60,60}},{?STR(camera,4,"Custom"),custom}],
+	     NegativeFormat,
+	     [{key,negative_format},layout,
+	     {hook,
+	      fun (update, {Var,_I,Val,Sto}) ->
+		      {store,camera_update_1(Var, Val, Sto)};
+		  (_, _)  -> void
+	      end}]},
+	    {hframe,
+	     [{text,
+	       NegH,
+	       [{key,negative_height},{range,?RANGE_NEGATIVE_SIZE},
+		{hook,
+		 fun (update, {Var,_I,Val,Sto}) ->
+			 {store,camera_update_1(Var, Val, Sto)};
+		     (_, _)  -> void
+		 end}]},
+	      {label,?STR(camera,5,"x")},
+	      {text,
+	       NegW,
+	       [{key,negative_width},{range,?RANGE_NEGATIVE_SIZE},
+		{hook,
+		 fun (update, {Var,_I,Val,Sto}) ->
+			 {store,camera_update_1(Var, Val, Sto)};
+		     (_, _)  -> void
+		 end}]}],
+	     [{hook,
+	       fun (is_disabled, {_Var,_I,Sto}) ->
+		       gbget(negative_format, Sto) =/= custom;
+		   (update, {Var,_I,Val,Sto}) ->
+		       {store,camera_update_2(Var, Val, Sto)};
+		   (_, _) -> void
+	       end}]}]},
+	  {hframe,
+	   [{menu,
+	     [{?STR(camera,6,"Wide-Angle Lens"),wide_angle},
+	      {?STR(camera,7,"Moderate Wide-Angle Lens"),moderate_wide_angle},
+	      {?STR(camera,8,"Standard Lens"),standard},
+	      {?STR(camera,9,"Portrait Telephoto Lens"),short_tele},
+	      {?STR(camera,10,"Telephoto Lens"),tele},
+	      {?STR(camera,11,"Custom Lens"),custom}],
+	     LensType,
+	     [{key,lens_type},
+	      {hook,
+	       fun (is_minimized, {_Var,_I,Sto}) ->
+		       gbget(negative_format, Sto) =:= custom;
+		   (update, {Var,_I,Val,Sto}) ->
+		       {store,camera_update_2(Var, Val, Sto)};
+		   (_, _) -> void
+	       end}]},
+	    panel,
+	    {label,?STR(camera,12,"Length")},
+	    {text,
+	     LensLength,
+	     [{key,lens_length},
+	      {hook,
+	       fun (update, {Var,_I,Val,Sto}) ->
+		       {store,camera_update_2(Var, Val, Sto)};
+		   (_, _) -> void
+	       end}]}]},
+	  {hframe,
+	   [{slider,
+	     [{key,zoom_slider},{range,?RANGE_ZOOM_SLIDER},
+	      {value,ZoomSlider},
+	      {hook,
+	       fun (update, {Var,_I,Val,Sto}) ->
+		       {store,camera_update_2(Var, Val, Sto)};
+		   (_, _) -> void
+	       end}]},
+	    {text,
+	     Zoom,
+	     [{key,zoom},
+	      {hook,
+	       fun (update, {Var,_I,Val,Sto}) ->
+		       {store,camera_update_2(Var, Val, Sto)};
+		   (_, _) -> void
+	       end}]},
+	    {label,?STR(camera,13,"x Zoom")}]}],
+	 [{title,?STR(camera,14,"Lens")},{minimized,true}]},
+    Qs = 
+	[LensFrame,
+	 {hframe,
+	  [{vframe,[{label,?STR(camera,15,"Field of View")},
+		    {label,?STR(camera,16,"Near Clipping Plane")},
+		    {label,?STR(camera,17,"Far Clipping Plane")}]},
+	   {vframe,[{text,Fov0,[{range,?RANGE_FOV},{key,fov},{hook,FovHook}]},
+		    {text,Hither0,[{range,?RANGE_NEAR_CLIP}]},
+		    {text,Yon0,[{range,?RANGE_FAR_CLIP}]}]},
+	   {vframe,[help_button(camera_settings_fov),
+		    panel,
+		    panel]}]}],
+    wings_ask:dialog(?STR(camera,18,"Camera Settings"), Qs,
+		     fun([_,
+			  {negative_format,_},
+			  {negative_height,_},{negative_width,_},
+			  {lens_type,_},{lens_length,_},
+			  {zoom_slider,_},{zoom,_},
+			  {fov,Fov},Hither,Yon]=Ps) ->
+			     {NH,NW} = camera_propconv_negative_format(Ps),
 			     View = View0#view{fov=Fov,hither=Hither,yon=Yon},
 			     wings_wm:set_prop(Active, current_view, View),
+			     wings_pref:set_value(negative_height, NH),
+			     wings_pref:set_value(negative_width, NW),
 			     ignore
 		     end).
 
+camera_update_1(Var, Val, Sto) ->
+    Props = gbget(lists:delete(Var, [fov,negative_format,
+				     negative_height,negative_width]), 
+		  Sto),
+    gbupdate(camera_propconv(from_fov, [{Var,Val}|Props]),
+	     gbupdate(Var, Val, Sto)).
+
+camera_update_2(Var, Val, Sto) ->
+    [_|Props0] = Props1 = 
+	gbget([lens_length,
+	       negative_format,negative_height,negative_width], Sto),
+    Props2 = camera_propconv(fov, [{Var,Val}|Props1])++Props0,
+    Props = camera_propconv(from_fov, Props2)++Props2,
+    gbupdate(proplists:delete(Var, Props), gbupdate(Var, Val, Sto)).
+
+camera_propconv(from_fov, Props) ->
+    {NegH,NegW} = NegF = camera_propconv_negative_format(Props),
+    Fov = pget(fov, Props),
+    LensLength = 
+	case catch 0.5*NegH/math:tan(Fov*math:pi()/360) of
+	    {'EXIT',_} -> 0.0;
+	    L when is_float(L) -> L
+	end,
+    LensType =
+	camera_lens_type(NegF, LensLength),
+    Zoom =
+	LensLength / math:sqrt(NegH*NegH + NegW*NegW),
+    ZoomSlider = 
+	case catch math:log(Zoom) / math:log(2) of
+	    {'EXIT',_} -> 
+		{Z,_} = ?RANGE_ZOOM_SLIDER,
+		Z;
+	    Z when is_float(Z) -> Z
+	end,
+    [{lens_length,LensLength},
+     {lens_type,LensType},
+     {zoom,Zoom},
+     {zoom_slider,wings_util:limit(ZoomSlider, ?RANGE_ZOOM_SLIDER)}];
+camera_propconv(fov, Props) ->
+    {NegH,NegW} = NegF = camera_propconv_negative_format(Props),
+    Zoom = 
+	case pget(zoom_slider, Props) of
+	    undefined -> 
+		pget(zoom, Props);
+	    ZoomSlider ->
+		math:pow(2, ZoomSlider)
+	end,
+    LensLength = 
+	case Zoom of
+	    undefined ->
+		case pget(lens_type, Props) of
+		    custom ->
+			pget(lens_length, Props);
+		    undefined ->
+			pget(lens_length, Props);
+		    LensType -> 
+			camera_lens_length(NegF, LensType)
+		end;
+	    _ ->
+		Zoom * math:sqrt(NegH*NegH + NegW*NegW)
+	end,
+    Fov = 
+	if is_float(LensLength) ->
+		case catch 360*math:atan(0.5*NegH/LensLength)/math:pi() of
+		    {'EXIT',_} -> 180.0;
+		    F when is_float(F) -> F
+		end
+	end,
+    [{fov,wings_util:limit(Fov, ?RANGE_FOV)}].
+
+camera_propconv_negative_format(Props) ->
+    case pget(negative_format, Props) of
+	{_,_} = NegativeFormat -> 
+	    NegativeFormat;
+	_ -> 
+	    {pget(negative_height, Props),
+	     pget(negative_width, Props)}
+    end.
+
+camera_lens_type({24,36}, LensLength) ->
+    case round(LensLength) of
+	24 -> wide_angle;
+	35 -> moderate_wide_angle;
+	50 -> standard;
+	85 -> short_tele;
+	135 -> tele;
+	_ -> custom
+    end;
+camera_lens_type({60,60}, LensLength) ->
+    case round(LensLength) of
+	40 -> wide_angle;
+	60 -> moderate_wide_angle;
+	80 -> standard;
+	150 -> short_tele;
+	250 -> tele;
+	_ -> custom
+    end;
+camera_lens_type(_, _) -> custom.
+
+camera_lens_length({24,36}, LensType) ->
+    float(case LensType of
+	      wide_angle          -> 24;
+	      moderate_wide_angle -> 35;
+	      standard            -> 50;
+	      short_tele          -> 85;
+	      tele                -> 135
+	  end);
+camera_lens_length({60,60}, LensType) ->
+    float(case LensType of
+	      wide_angle          -> 40;
+	      moderate_wide_angle -> 60;
+	      standard            -> 80;
+	      short_tele          -> 150;
+	      tele                -> 250
+	  end);
+camera_lens_length(_, _) -> undefined.
+
+
+
+gbget([], _Sto) -> [];
+gbget([Key|Keys], Sto) -> [{Key,gb_trees:get(Key, Sto)}|gbget(Keys, Sto)];
+gbget(Key, Sto) -> gb_trees:get(Key, Sto).
+
+gbupdate(Key, Val, Sto) -> gb_trees:update(Key, Val, Sto).
+
+gbupdate([], Sto) -> Sto;
+gbupdate([{Key,Val}|KVs], Sto) ->
+    gbupdate(KVs, gbupdate(Key, Val, Sto)).
+
+pget(Key, Props) -> proplists:get_value(Key, Props).
+%%% pget(Key, Props, Default) -> proplists:get_value(Key, Props, Default).
+
+
 
 %%%
 %%% The Auto Rotate command.
@@ -520,11 +777,13 @@ auto_rotate_redraw(#tim{st=#st{}=St}) ->
     wings_render:render(St).
 
 auto_rotate_help() ->
-    Msg1 = wings_util:button_format("Stop rotating"),
+    Msg1 = wings_util:button_format(?STR(auto_rotate_help,1,"Stop rotating")),
     Msg2 = wings_camera:help(),
-    Message = wings_util:join_msg([Msg1,Msg2,
-				   "[+] Increase speed",
-				   "[-] Decrease speed"]),
+    P = wings_util:key_format("[+]",
+			      ?STR(auto_rotate_help,2,"Increase speed")),
+    M = wings_util:key_format("[-]",
+			      ?STR(auto_rotate_help,3,"Decrease speed")),
+    Message = wings_util:join_msg([Msg1,Msg2,P,M]),
     wings_wm:message(Message).
 
 set_auto_rotate_timer(#tim{delay=Delay}=Tim) when Delay < 0 ->
@@ -613,8 +872,12 @@ load_matrices(IncludeLights) ->
 projection() ->
     {W,H} = wings_wm:win_size(),
     Aspect = W/H,
-    #view{distance=D,fov=Fov,hither=Hither,yon=Yon} = current(),
-    case wings_wm:get_prop(orthogonal_view) of
+    #view{distance=D,fov=Fov,hither=Hither,yon=Yon,along_axis=AA} = 
+	current(),
+    Ortho = wings_wm:get_prop(orthogonal_view) 
+	orelse ((AA =/= none) andalso 
+		wings_pref:get_value(force_ortho_along_axis)),
+    case Ortho of
 	false ->
 	    glu:perspective(Fov, Aspect, Hither, Yon);
 	true ->
@@ -686,7 +949,7 @@ frame_1([A,B]=BB) ->
     C = e3d_vec:average(BB),
     R = e3d_vec:len(e3d_vec:sub(A, B)) / 2,
     #view{fov=Fov} = View = current(),
-    Dist = R/math:tan(Fov*3.14159/2/180),
+    Dist = R/math:tan(Fov*math:pi()/2/180),
     set_current(View#view{origin=e3d_vec:neg(C),
 			  distance=Dist,pan_x=0.0,pan_y=0.0}).
 
@@ -702,7 +965,7 @@ views({save,Ask}, #st{views={CurrentView,Views}}) when is_atom(Ask) ->
     if S > 0 ->
 	    case element(view_index(CurrentView, S), Views) of
 		{View,_} ->
-		    wings_util:message("This view is alreay the current");
+		    wings_util:message(?STR(views,1,"This view is already the current"));
 		_ ->
 		    views_save_dialog(Ask, [view_legend(View)])
 	    end;
@@ -710,7 +973,7 @@ views({save,Ask}, #st{views={CurrentView,Views}}) when is_atom(Ask) ->
 	    views_save_dialog(Ask, [view_legend(View)])
     end;
 views(_, #st{views={_,{}}}) ->
-    wings_util:message("No saved views");
+    wings_util:message(?STR(views,2,"No saved views"));
 views(next, #st{views={CurrentView,Views}}=St) ->
     J = view_index(CurrentView + 1, size(Views)),
     {View,_} = element(J, Views),
@@ -733,7 +996,7 @@ views(rename, #st{views={CurrentView,Views}}=St) ->
     J = view_index(CurrentView, size(Views)),
     {View,Legend} = element(J, Views),
     wings_ask:dialog(
-      "Rename view",
+      ?STR(views,3,"Rename view"),
       views_rename_qs([Legend]),
       fun([NewLegend]) ->
 	      St#st{views={CurrentView,
@@ -748,24 +1011,24 @@ views(delete, #st{views={CurrentView,Views}}=St) ->
 	    {L1,[_|L2]} = lists:split(I, tuple_to_list(Views)),
 	    {save_state,St#st{views={I,list_to_tuple(L1++L2)}}};
 	_ ->
-	    wings_util:message("You have to be at the current view")
+	    wings_util:message(?STR(views,4,"You have to be at the current view"))
     end;
 views(delete_all, St) ->
     This = wings_wm:this(),
     wings_util:yes_no(
-      "Are you sure you want to delete all saved views?",
+      ?STR(views,5,"Are you sure you want to delete all saved views?"),
       fun() -> 
 	      wings_wm:send(This, {new_state,delete_all(St)}),
 	      ignore
       end).
 
 views_save_dialog(Ask, Options) ->
-    wings_ask:dialog(Ask, "Save view as", 
+    wings_ask:dialog(Ask, ?STR(views_save_dialog,1,"Save view as"), 
 		     views_rename_qs(Options),
 		     fun(Opts) -> {view,{views,{save,Opts}}} end).
 
 views_rename_qs([Legend]) ->
-    [{hframe,[{label,"Name"},{text,Legend}]}].
+    [{hframe,[{label,?STR(views_rename_qs,1,"Name")},{text,Legend}]}].
 
 views_jump(J, St, CurrentView, Views) ->
     S = size(Views),
@@ -779,7 +1042,7 @@ views_jump(J, St, CurrentView, Views) ->
 	    set_current(View),
 	    St#st{views={J,Views}};
 	_ ->
-	    wings_util:message("No such view slot")
+	    wings_util:message(?STR(views_jump,1,"No such view slot"))
     end.
 
 views_move(J, St, CurrentView, Views) ->
@@ -800,7 +1063,7 @@ views_move(J, St, CurrentView, Views) ->
 	    set_current(View),
 	    St#st{views={J,list_to_tuple(V1++(V3++[VL|V4]))}};
 	_ ->
-	    wings_util:message("No such view ["++integer_to_list(J)++"]")
+	    wings_util:message(?STR(views_move,1,"No such view [")++integer_to_list(J)++"]")
     end.
 
 toggle_lights() ->
@@ -893,7 +1156,7 @@ align_to_selection({Nx,Ny,Nz}, St) ->
     St.
 
 to_degrees(A) when is_float(A) ->
-    A*180.0/3.1415926536.
+    A*180.0/math:pi().
 
 one_of(true, S, _) -> S;
 one_of(false,_, S) -> S.
@@ -911,10 +1174,6 @@ export_views_1([{View,Name}|Views]) ->
     [{view,Props}|export_views_1(Views)];
 export_views_1([]) -> [].
 
-zip([H1|T1], [H2|T2]) ->
-    [{H1,H2}|zip(T1, T2)];
-zip([], []) -> [].
-
 import_views(Views, #st{views={CurrentView,_}}=St) ->
     St#st{views={CurrentView,list_to_tuple(import_views_1(Views))}}.
 
@@ -993,23 +1252,23 @@ view_legend(#view{distance=Dist,along_axis=Along}=View) ->
     [{Pos,Dir,_}] = camera_info([pos_dir_up], View),
     Legend = 
 	io_lib:format(
-	  "From ~s ~s distance ~.4g",
+	  ?STR(view_legend,1,"From ~s ~s distance ~.4g"),
 	  [pos_legend(Pos),
 	   case Along of
-	       x -> "along +X";
-	       y -> "along +Y";
-	       z -> "along +Z";
-	       neg_x -> "along -X";
-	       neg_y -> "along -Y";
-	       neg_z -> "along -Z";
+	       x -> ?STR(view_legend,2,"along +X");
+	       y -> ?STR(view_legend,3,"along +Y");
+	       z -> ?STR(view_legend,4,"along +Z");
+	       neg_x -> ?STR(view_legend,5,"along -X");
+	       neg_y -> ?STR(view_legend,6,"along -Y");
+	       neg_z -> ?STR(view_legend,7,"along -Z");
 	       none ->
 		   Aim = e3d_vec:add(Pos, Dir),
 		   AimLen = e3d_vec:len(Aim),
 		   if AimLen < Dist*0.000001 ->
 			   %% Close enough to Origin
-			   "towards Origin";
+			   ?STR(view_legend,8,"towards Origin");
 		      true ->
-			   "towards "++pos_legend(e3d_vec:neg(Dir))
+			   ?STR(view_legend,9,"towards ")++pos_legend(e3d_vec:neg(Dir))
 		   end
 	   end,
 	   Dist]),
@@ -1029,3 +1288,18 @@ pos_legend({X,Y,Z}) ->
     [[if Val < 0 -> $-;
 	 true    -> $+ end,Char] 
      || {Abs,Char,Val} <- L, Abs >= Max*0.4142135624]. % tan(22.5 degrees)
+
+
+
+help_button(Subject) ->
+    Title = help(title, Subject),
+    TextFun = fun () -> help(text, Subject) end,
+    {help,Title,TextFun}.
+
+help(title, camera_settings_fov) ->
+    ?STR(help,1,"Camera Settings: Field of View");
+help(text, camera_settings_fov) ->
+    [?STR(help,2,"Sets the vertical field of view, i.e. the angle from lower to upper border seen from the camera. "),
+     ?STR(help,3,"The Lens controls sets the field of view according to other perhaps more well-known entities, at least for a photographer. The negative size is stored among the user preferences."),
+     ?STR(help,4,"The Lens controls make the height of the picture right for the choosen lens, so if you do not have the same aspect ratio for the negative size in the Lens frame, as for the size of the Geometry window, the width will be in error."),
+     ?STR(help,5,"Note: the zoom factor is relative a standard lens length of the negative diagonal, while only the negative height affects the effective lens length.")].
