@@ -10,7 +10,7 @@
 %%  See the file "license.terms" for information on usage and redistribution
 %%  of this file, and for a DISCLAIMER OF ALL WARRANTIES.
 %%
-%%     $Id: wings_we.erl,v 1.80 2004/06/02 04:26:19 bjorng Exp $
+%%     $Id: wings_we.erl,v 1.82 2004/10/09 05:44:36 bjorng Exp $
 %%
 
 -module(wings_we).
@@ -32,7 +32,7 @@
 -include("wings.hrl").
 -include("e3d.hrl").
 -import(lists, [map/2,foreach/2,foldl/3,sort/1,keysort/2,
-		last/1,reverse/1,duplicate/2,seq/2,filter/2]).
+		last/1,reverse/1,duplicate/2,seq/2,filter/2,zip/2]).
 
 rebuild(#we{vc=undefined,fs=undefined,es=Etab0}=We) ->
     Etab = gb_trees:to_list(Etab0),
@@ -269,10 +269,6 @@ pairs([{V1,T1}|[{V2,T2}|_]=Vs], First, Acc) ->
 pairs([{V,T}], [{V1,T1},{V2,T2},{V3,T3}|_], Acc) ->
     [{{V3,V2},{T3,T2}},{{V2,V1},{T2,T1}},{{V1,V},{T1,T}}|Acc].
 
-zip([V|Vs], [UV|UVs]) ->
-    [{V,UV}|zip(Vs, UVs)];
-zip([], []) -> [].
-
 edge_name({Vs,Ve}=Name) when Vs < Ve -> Name;
 edge_name({Vs,Ve}) -> {Ve,Vs}.
 
@@ -745,7 +741,7 @@ transform_vs_1(Transform, #we{vp=Vtab0}=We) ->
 %%%
 
 normals(FaceNormals, #we{he=He}=We) ->
-    wings_pb:start("calculating soft normals"),
+    wings_pb:start(?STR(normals,1,"calculating soft normals")),
     Res = case FaceNormals of
 	      [_,_] ->
 		  two_faced(FaceNormals, We);
@@ -758,13 +754,13 @@ normals(FaceNormals, #we{he=He}=We) ->
     wings_pb:done(Res).
 
 all_soft(FaceNormals, #we{vp=Vtab}=We) ->
-    wings_pb:update(0.10, "preparing"),
+    wings_pb:update(0.10, ?STR(all_soft,1,"preparing")),
     VtxNormals = soft_vertex_normals(gb_trees:to_list(Vtab), FaceNormals, We),
     FoldFun = fun(V, VInfo, A) ->
 		      Normal = gb_trees:get(V, VtxNormals),
 		      [[VInfo|Normal]|A]
 	      end,
-    wings_pb:update(0.6, "collecting"),
+    wings_pb:update(0.6, ?STR(all_soft,2,"collecting")),
     all_soft_1(FoldFun, FaceNormals, We, []).
 
 all_soft_1(FoldFun, [{Face,_}|FNs], We, Acc) ->
@@ -787,12 +783,12 @@ soft_vertex_normals(Vtab, FaceNormals, We) ->
     gb_trees:from_orddict(reverse(Soft)).
 
 mixed_edges(FaceNormals0, We) ->
-    wings_pb:update(0.20, "preparing"),
+    wings_pb:update(0.20, ?STR(mixed_edges,1,"preparing")),
     G = digraph:new(),
     FaceNormals = gb_trees:from_orddict(FaceNormals0),
-    wings_pb:update(0.50, "vertex normals"),
+    wings_pb:update(0.50,  ?STR(mixed_edges,2,"vertex normals")),
     VtxNormals = vertex_normals(We, G, FaceNormals),
-    wings_pb:update(0.99, "vertex normals per face"),
+    wings_pb:update(0.99,  ?STR(mixed_edges,3,"vertex normals per face")),
     Ns = foldl(fun({Face,_}, Acc) ->
 		       Vs = n_face(Face, G, FaceNormals, VtxNormals, We),
 		       [{Face,Vs}|Acc]
