@@ -8,11 +8,11 @@
 %%  See the file "license.terms" for information on usage and redistribution
 %%  of this file, and for a DISCLAIMER OF ALL WARRANTIES.
 %%
-%%     $Id: wings_wm.erl,v 1.143 2004/05/23 14:37:50 bjorng Exp $
+%%     $Id: wings_wm.erl,v 1.147 2004/11/14 13:47:27 bjorng Exp $
 %%
 
 -module(wings_wm).
--export([toplevel/6,set_knob/3]).
+-export([toplevel/6,toplevel_title/2,set_knob/3]).
 -export([init/0,enter_event_loop/0,dirty/0,dirty_mode/1,pdirty/0,
 	 reinit_opengl/0,
 	 new/4,delete/1,raise/1,
@@ -61,8 +61,6 @@
 
 -define(Z_LOWEST_DYNAMIC, 10).
 
--compile(inline).
-
 -record(win,
 	{z,					%Z order.
 	 x,y,					%Position.
@@ -605,13 +603,8 @@ do_dispatch(Active, Ev) ->
     end.
 
 redraw_all() ->
-    case get(wm_dirty_mode) of
-	back -> ok;
-	front ->
-	    %% If front drawing is taking place (e.g. a marquee selection),
-	    %% buffers weren't cleared directly after buffer swapping.
-	    gl:clear(?GL_COLOR_BUFFER_BIT bor ?GL_DEPTH_BUFFER_BIT)
-    end,
+    %% Remove late buffers clear due to problems with ATI cards when AA.
+    gl:clear(?GL_COLOR_BUFFER_BIT bor ?GL_DEPTH_BUFFER_BIT),
     Windows = keysort(2, gb_trees:to_list(get(wm_windows))),
     foreach(fun({Name,_}) ->
 		    dispatch_matching(fun({wm,{send_to,N,_}}) ->
@@ -621,19 +614,6 @@ redraw_all() ->
 		    do_dispatch(Name, redraw)
 	    end, Windows),
     gl:swapBuffers(),
-
-    case get(wm_dirty_mode) of
-	back ->
-	    %% By clearing now, we might be able to overlap the time
-	    %% for clearing with calculations done by the CPU,
-	    %% thus pontentially gaining some speed.
-	    gl:clear(?GL_COLOR_BUFFER_BIT bor ?GL_DEPTH_BUFFER_BIT);
-	front ->
-	    %% If front drawing is taking place, clearing the back buffer
-	    %% might destroy the front buffer too. Do nothing.
-	    ok
-    end,
-
     clean(),
     wings_io:set_cursor(get(wm_cursor)),
     event_loop().
@@ -1072,7 +1052,7 @@ message_event(redraw) ->
 message_event({action,_}=Action) ->
     send(geom, Action);
 message_event(got_focus) ->
-    message("This is the information line"),
+    message(?STR(message_event,1,"This is the information line")),
     dirty();
 message_event(_) -> keep.
 
@@ -1131,6 +1111,10 @@ message_setup() ->
 toplevel(Name, Title, Pos, Size, Flags, Op) ->
     wings_wm_toplevel:toplevel(Name, Title, Pos, Size, Flags, Op).
 
+toplevel_title(Win, Title) ->
+    send({controller,Win}, {title,Title}),
+    ok.
+
 set_knob(Name, Pos, Proportion) ->
     wings_wm_toplevel:set_knob(Name, Pos, Proportion).
 
