@@ -8,7 +8,7 @@
 %%  See the file "license.terms" for information on usage and redistribution
 %%  of this file, and for a DISCLAIMER OF ALL WARRANTIES.
 %%
-%%     $Id: wings_wm_toplevel.erl,v 1.49 2004/04/15 20:35:55 raimo_niskanen Exp $
+%%     $Id: wings_wm_toplevel.erl,v 1.52 2004/11/14 13:47:27 bjorng Exp $
 %%
 
 -module(wings_wm_toplevel).
@@ -25,8 +25,6 @@
 -import(lists, [reverse/1,keysearch/3,sort/1,foreach/2]).
 -import(wings_util, [min/2,max/2]).
 
--compile(inline).
-
 toplevel(Name, Title, Pos, Size, Flags, Op) ->
     wings_wm:new(Name, Pos, Size, Op),
     new_controller(Name, Title, Flags).
@@ -200,13 +198,18 @@ ctrl_event({window_updated,Client}, _) ->
     keep;
 ctrl_event({action,{titlebar,Action}}, Cs) ->
     ctrl_command(Action, Cs);
+ctrl_event({title,Title}, Cs) ->
+    get_ctrl_event(Cs#ctrl{title=Title});
 ctrl_event(_, _) -> keep.
 
 ctrl_message() ->
-    M0 = wings_util:button_format("Drag to Move"),
+    M0 = wings_util:button_format( ?STR(ctrl_message,1,"Drag to Move")),
     M1 = case is_resizeable() of
 	     false -> [];
-	     true -> wings_util:button_format([], "Fit", "Show menu")
+	     true -> wings_util:button_format([],  
+				?STR(ctrl_message,2,"Fit"), 
+				?STR(ctrl_message,3,"Show menu") 
+				)
 	 end,
     M = wings_util:join_msg(M0, M1),
     wings_wm:message(M),
@@ -257,16 +260,16 @@ ctrl_constrain_move(Client, Dx0, Dy0) ->
     {Dx,Dy}.
 
 ctrl_menu(X, Y) ->
-    Menu = [{"Fit",
+    Menu = [{ ?STR(ctrl_menu,1,"Fit"),
 	     {fit,
-	      [{"Both",both,
-		"Let window use all available space by expanding in all directions"},
-	       {"Horizontal",horizontal,
-		"Let window use all available space by expanding it horizontally"},
-	       {"Vertical",vertical,
-		"Let window use all available space by expanding it vertically"}
+	      [{?STR(ctrl_menu,2,"Both"),both,
+		?STR(ctrl_menu,3,"Let window use all available space by expanding in all directions")},
+	       {?STR(ctrl_menu,4,"Horizontal"),horizontal,
+	 	?STR(ctrl_menu,5,"Let window use all available space by expanding it horizontally")},
+	       {?STR(ctrl_menu,6,"Vertical"),vertical,
+		?STR(ctrl_menu,7,"Let window use all available space by expanding it vertically")}
 	      ]}},
-	    {"Size",size,"Set window size numerically"}|ctrl_menu_toolbar()],
+	    {?STR(ctrl_menu,8,"Size"),size,?STR(ctrl_menu,9,"Set window size numerically")}|ctrl_menu_toolbar()],
     wings_menu:popup_menu(X, Y, titlebar, Menu).
 
 ctrl_menu_toolbar() ->
@@ -277,9 +280,9 @@ ctrl_menu_toolbar() ->
 	true ->
 	    case wings_wm:is_hidden(Toolbar) of
 		false ->
-		    [{"Hide Toolbar",hide_toolbar,"Hide the toolbar"}];
+		    [{?STR(ctrl_menu_toolbar,1,"Hide Toolbar"),hide_toolbar,?STR(ctrl_menu_toolbar,2,"Hide the toolbar")}];
 		true ->
-		    [{"Show Toolbar",show_toolbar,"Show the toolbar"}]
+		    [{?STR(ctrl_menu_toolbar,3,"Show Toolbar"),show_toolbar,?STR(ctrl_menu_toolbar,4,"Show the toolbar")}]
 	    end
     end.
 
@@ -303,9 +306,9 @@ ctrl_command({fit,Fit}, _) ->
 ctrl_command(size, _) ->
     {_,Client} = wings_wm:this(),
     {W0,H0} = wings_wm:win_size(Client),
-    Qs = [{"Width",W0},
-	  {"Height",H0}],
-    wings_ask:ask("Set Window Size", Qs,
+    Qs = [{?STR(ctrl_command,1,"Width"),W0},
+	  {?STR(ctrl_command,2,"Height"),H0}],
+	   wings_ask:ask(?STR(ctrl_command,3,"Set Window Size"), Qs,
 		  fun([W,H]) ->
 			  ctrl_resize(Client, W, H),
 			  ignore
@@ -315,7 +318,7 @@ ctrl_resize(Client, W, H) ->
     {TopW,TopH} = wings_wm:top_size(),
     if
 	W > TopW; H > TopH ->
-	    wings_util:error("Too large size specified");
+	    wings_util:error(?STR(ctrl_resize,1,"Too large size specified"));
 	true ->
 	    wings_wm:resize(Client, {W,H})
     end.
@@ -499,7 +502,8 @@ resize_event(redraw, #rsz{color=Color}) ->
 			end),
     keep;
 resize_event(got_focus, _) ->
-    wings_util:button_message("Resize", "Resize, keeping current aspect ratio"),
+    wings_util:button_message(?STR(resize_event,1,"Resize"), 
+		    		?STR(resize_event,2,"Resize, keeping current aspect ratio")),
     wings_wm:dirty();
 resize_event(#mousebutton{button=1,state=?SDL_PRESSED},
 	     #rsz{state=moving,prev_focus=Focus}=Rst) ->
@@ -720,7 +724,7 @@ close_event(redraw) ->
 	    keep
     end;
 close_event(got_focus) ->
-    wings_wm:message("Close this window"),
+    wings_wm:message(?STR(close_event,1,"Close this window")),
     wings_wm:dirty();
 close_event(#mousebutton{button=1,state=?SDL_RELEASED}) ->
     {_,Client} = wings_wm:this(),
